#///////////////////////////////////////////////////////////////////////////////
#// $Id: 00_build_tp_tools.b,v 1.6 2002/12/24 19:00:37 gmaxe Exp $
#//
##! @file
##! @brief The control script for all tp_tools. 
##!
##! This is used when building
##! from scratch. It checks out all other scripts and tools needed from CVS,
##! and then proceeds to call those scripts in the proper order.
##!
##! This is typically only run the first time. Thereafter, the 00_short.b
##! can be more efficient or even calling individual scripts.
##!
##! @note This file needs to be updated to support other directories.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: 00_build_tp_tools.b,v $
#// Revision 1.6  2002/12/24 19:00:37  gmaxe
#// New structure and new version using TOC and all.
#//
#// Revision 1.5  2002/11/21 19:50:50  gmaxe
#// Link to common script files.
#//
#// Revision 1.4  2002/01/21 17:39:50  gmaxe
#// Changed comment style.
#//
#//
#///////////////////////////////////////////////////////////////////////////////

# assumes that you are in the <rtfm/>techpubs/tp_tools directory  <home> where this
# script was found. ... 
####
echo "Update the tools needed for this task."
cd /rtfm
cvs update techpubs/tp_tools
cvs update techpubs/perl

cd /rtfm/techpubs/perl
chmod 775 *.pl

echo "Back <home>."
cd /rtfm/techpubs/tp_tools

echo "chmod 775 *.b"
chmod 775 *.b


# Link to common script files
ln -s /rtfm/techpubs/b_script/20_cp_com_files.b 20_cp_com_files.b
ln -s /rtfm/techpubs/b_script/35_gen_dox.b 35_gen_dox.b
ln -s /rtfm/techpubs/b_script/35_gen_tree.b 35_gen_tree.b
ln -s /rtfm/techpubs/b_script/45_latex_gen.b 45_latex_gen.b
ln -s /rtfm/techpubs/b_script/55_nav_cp.b 55_nav_cp.b
ln -s /rtfm/techpubs/b_script/55_nav_gen.b 55_nav_gen.b
ln -s /rtfm/techpubs/b_script/56_nav_index.b 56_nav_index.b
ln -s /rtfm/techpubs/b_script/56_nav_script.b 56_nav_script.b

ln -s /rtfm/books/tp_tools/2000/users_guide src_fm
ln -s /rtfm/techpubs/perl src_perl

cd doc_publish
ln -s /rtfm/books/tp_tools/2000/doc_publish/book_tp_tools book_tp_tools
ln -s /rtfm/books/help_on_help/1000/doc_publish/book_help_on_help book_help_on_help

cd /rtfm/techpubs/tp_tools

echo "Make needed support directores if they don't already exist."
mkdir zdoc_merge
mkdir doc_publish
mkdir doc_publish/book_tp_tools
mkdir doc_publish/cref_tp_tools

set def_ver = 'v2-00-0a'
if ($#argv > 0) then
   @ cnt = 1
   while ($#argv)
      # echo "$argv[1] and $cnt"
      if ($cnt == 1) then
         set rel_ver = $argv[1]
      endif
      @ cnt++
      shift
   end
   /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html $rel_ver
   set ret_val = $status
   if ($ret_val > 0) then
      echo "WARNING: Problems with master_update.pl 1."
      exit 1
   endif
else
   echo -n "Enter latest build version [$def_ver]:"
   set rel_ver = $<
   if ($rel_ver =~ "") then
      set rel_ver = $def_ver
      # If we're just using the default version from this script, then
      # the master itself will probably have more accurate version numbers.
      echo "Version from master file will be used and not '$rel_ver'."
      /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html
      set ret_val = $status
      if ($ret_val > 0) then
         echo "WARNING: Problems with master_update.pl 2."
         exit 1
      endif
      echo "Version existing in master file was used and not the default [$rel_ver]."
   else
      # we entered a valid version and should update with that date.
      /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html $rel_ver
      set ret_val = $status
      if ($ret_val > 0) then
         echo "WARNING: Problems with master_update.pl 3."
         exit 1
      endif
   endif
endif
# echo "Will look here: $rel_ver"




echo "Copy common files into respective subdirectories."
echo "Run 20_cp_com_files from 00_build_tp_tools.b ..."
20_cp_com_files.b
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with 20_cp_com_files.b ."
   exit 1
endif

echo "Perform the Doxygen software component builds."
echo "Run 30_tp_tools.b from 00_build_tp_tools.b ..."
30_tp_tools.b $rel_ver
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with 30_tp_tools.b ."
   exit 1
endif

# Handle the navigation
echo "Run 50_nav_update.b from 00_build_tp_tools.b ..."
50_nav_update.b $rel_ver
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with 50_nav_update.b ."
   exit 1
endif

# Do latex builds by hand
# echo "Run 40_latex_build.b from 00_build_tp_tools.b ..."
# 40_latex_build.b
# set ret_val = $status
# if ($ret_val > 0) then
#   echo "ERROR: Problems with 40_latex_build.b ."
#   exit 1
# endif


echo "Finished 00_build_tp_tools.b from 00_build_tp_tools.b ..."
exit 0
