#///////////////////////////////////////////////////////////////////////////////
#// $Id: 30_tp_tools.b,v 1.2 2002/12/24 19:00:38 gmaxe Exp $
#//
##! @file
##! @brief Calls a couple of other shell scripts which do CVS checkout
##! and then eventually a doxygen build. 
##!
##! True, in this simple example, nearly all of the functionality of
##! the shell scripts (eventually) called by this one 
##! (e.g., 31*.b and 32*.b, and deeper down 35*.b and 55*.b)
##! could be placed into this file.
##! 
##! However, this is only a simple example. In my real world, a documentation
##! suite includes several doxygen projects. Plus, I have many different
##! documentation suites covering different aspects of our code and aimed
##! at different (mostly internal) audiences.
##! 
##! In my environment, a given documentation suite has multiple 30*.b files,
##! which call associated 31*.b and 32*.b files. (Lately, I've been wrapping
##! yet another 30_build_all.b around all of the 30*.b files which can be
##! hooked into the software build process.)
##! 
##! With respect to the 31*.b and 31*.b separation, I found it beneficial to
##! compartmentalize the CVS checkout procedures (31*.b) from the actual
##! doxygen calls (32*.b), because it facilitated the maintenance/tweak mode
##! better. I could focus on one project and one area of a project to
##! get it correct when doing partial system builds.
##! 
##! Moreover, I found many operations that either were repeated frequently
##! within the build process or were being forgotten whenever I tried to
##! do a partial build. These were pulled out into 35*.b and 55*.b files.
##! Yes, in some cases, certain perl programs are called multiple times
##! on the same set of information. The run-time inefficiencies and 
##! delays (seconds) in repeating operations was deemed acceptable in 
##! comparison to the alternative, which was a compromise in the 
##! integrity of the system (readily evident in the TOC or index) if
##! I forgot a step in the partial build process.
##!
##! @note This file can be used as a template for other code directories
##! that Doxygen is being run against. Make sure than any new 30 files are
##! added to the 00 script files, in addition to appropriate
##! entries in the other script files.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: 30_tp_tools.b,v $
#// Revision 1.2  2002/12/24 19:00:38  gmaxe
#// New structure and new version using TOC and all.
#//
#//
#///////////////////////////////////////////////////////////////////////////////

# assumes that you are in the <rtfm/>techpubs/tp_tools directory  <home> where this
# script was found. ... 
echo "Entering 30_tp_tools..."

set def_ver = 'v2-00-0a'
if ($#argv > 0) then
   @ cnt = 1
   while ($#argv)
      # echo "$argv[1] and $cnt"
      if ($cnt == 1) then
         set rel_ver = $argv[1]
      endif
      @ cnt++
      shift
   end
   /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html $rel_ver
   set ret_val = $status
   if ($ret_val > 0) then
      echo "WARNING: Problems with master_update.pl 1."
      exit 1
   endif
else
   echo -n "Enter latest build version [$def_ver]:"
   set rel_ver = $<
   if ($rel_ver =~ "") then
      set rel_ver = $def_ver
      # If we're just using the default version from this script, then
      # the master itself will probably have more accurate version numbers.
      echo "Version from master file will be used and not '$rel_ver'."
      /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html
      set ret_val = $status
      if ($ret_val > 0) then
         echo "WARNING: Problems with master_update.pl 2."
         exit 1
      endif
      echo "Version existing in master file was used and not the default [$rel_ver]."
   else
      # we entered a valid version and should update with that date.
      /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html $rel_ver
      set ret_val = $status
      if ($ret_val > 0) then
         echo "WARNING: Problems with master_update.pl 3."
         exit 1
      endif
   endif
endif
# echo "Will look here: $rel_ver"


31_perl.b
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with 31_tp_tools.b ."
   exit 1
endif

31_script.b
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with 31_tp_tools.b ."
   exit 1
endif

32_perl.b
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with 32_tp_tools.b."
   exit 1
endif

32_script.b
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with 32_tp_tools.b."
   exit 1
endif

exit 0
