#///////////////////////////////////////////////////////////////////////////////
#// $Id: 35_gen_dox.b,v 1.2 2002/12/24 21:36:21 gmaxe Exp $
#//
##! @file
##! @brief Calls doxygen with the specified input directory and then copies
##! generated files where they need to go for later processing.
##! 
##! @param dox_path A portion of the path used to distinguish the project.
##! The final destination is assumed to be cref_<dox_path>. In addition,
##! it is assumed that a related <dox_path>.dox configuration file for
##! doxygen exists in the directory from where this was called.
##! 
##! @param master_nav The name of an master HTML file that has common HTML 
##! fragments which are to be inserted in all generated HTML pages.
##! 
##! @param master_proj The name of the project_toc.txt file that defines
##! all directories and their associated names, pdf files, and document numbers.
##! 
##! @bug This calls both 55_nav_gen.b and 56_nav_script.b with the information
##! passed into this script. These were two steps that were often forgotten.
##! Doxygen does not generate HTML based on the master navigation file, so
##! 55_nav_gen.b forces the insertion of the standard tags. 
##! 
##! Likewise, this
##! script by itself does not make sure that the master TOC script file is 
##! updated. As such, after copying over the mini-index and mini-TOC files
##! to a known location, this goes ahead regenerates the master TOC.
##! 
##! It does not regenerate the master index, because that is much more
##! time-consuming.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: 35_gen_dox.b,v $
#// Revision 1.2  2002/12/24 21:36:21  gmaxe
#// Commented version.
#//
#//
#///////////////////////////////////////////////////////////////////////////////
# assumes that you are in the <rtfm/>techpubs/<home> directory 
# where this script resides
# This is a general script to simpify calling it from other places
# echo "Entering 32_gen_dox..."
# echo "Input argument $# $1 $2"

set dox_path=$1
set master_nav=$2
set master_proj=$3

# Expect an error from this if the directory exists.
mkdir doc_publish/cref_${dox_path}

echo "Remove files from last run"
rm doc_publish/cref_${dox_path}/*
rm zlx_${dox_path}/*.*

cp /rtfm/techpubs/web_files/*.css doc_publish/cref_${dox_path}/
cp /rtfm/techpubs/web_files/*.gif doc_publish/cref_${dox_path}/

echo "doxygen ${dox_path}.dox..."
doxygen ${dox_path}.dox 
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Doxygen did not run on cref_${dox_path}."
   exit $ret_val
endif

echo "Calling tree_js_2_script.pl..."
/rtfm/techpubs/perl/tree_js_2_script.pl doc_publish/cref_${dox_path}/  doc_publish/cref_${dox_path}/tree.js
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Did not create tree file on cref_${dox_path}."
   exit $ret_val
endif

echo "Calling voyant_nav.pl..."
55_nav_gen.b doc_publish/ cref_${dox_path} ${master_nav} ${master_proj}
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Did not create tree file on cref_${dox_path}."
   exit $ret_val
endif
56_nav_script.b

echo "Okay 35_gen_dox.b ${dox_path} ${master_nav} ${master_proj}"
exit 0

