#///////////////////////////////////////////////////////////////////////////////
#// $Id: 45_latex_gen.b,v 1.2 2002/12/24 21:36:22 gmaxe Exp $
#//
##! @file
##! @brief Calls the requisite programs for ultimately generating a PDF file
##! from Latex that came from doxygen.
##! 
##! @param lat_path The shortened version of the directory name where
##! the latex files can be found. It is assumed that zlx_<lat_path>
##! exists in a subdirectory from where this was called.
##! 
##! In order to get a PDF from a doxygen project, you have to make sure
##! that doxygen generates latex files. Then you have to call various
##! commands to get latex to generate a postscript and then PDF file.
##! The steps are the same for all projects.
##! 
##! Rather than polluting the higher-level scripts that are essentially
##! the same for all doxygen projects, this separates the details from
##! the application-specific projects. Thus, if doxygen changes how PDF
##! files are generated, this can be modified in one place to handle it.
##! 
##! @bug This is intended to be called from a 40_latex_gen.b file that
##! determines the directory name.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: 45_latex_gen.b,v $
#// Revision 1.2  2002/12/24 21:36:22  gmaxe
#// Commented version.
#//
#//
#///////////////////////////////////////////////////////////////////////////////
# assumes that you are in the <rtfm/>techpubs/<home> directory 
# where this script resides

set lat_path=$1

echo "Entering 45_latex_gen.b ${lat_path} "

echo "Generate all PDF files."

# BAPI (works)
echo "Doing zlx_${lat_path}..."
cd zlx_${lat_path}
make clean
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with make clean ."
   exit 1
endif

echo "Running latex..."
latex refman.tex
set ret_val = $status
# if ($ret_val > 0) then
#   echo "ERROR: Problems with latex refman.tex ."
#   exit 1
endif

echo "Running makeindex..."
makeindex refman.idx
set ret_val = $status
if ($ret_val > 0) then
#   echo "ERROR: Problems with makeindex refman.idx ."
#   exit 1
endif

echo "Rerunning latex...."
latex refman.tex
set ret_val = $status
if ($ret_val > 0) then
#   echo "ERROR: Problems with latex refman.tex ."
#   exit 1
endif

make ps
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with make ps ."
   exit 1
endif

make pdf
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Problems with make pdf ."
   exit 1
endif

cd ..
cp zlx_${lat_path}/refman.pdf doc_publish/print_pdf/${lat_path}_crf.pdf


echo "Okay 45_latex_gen.b ${lat_path} "
exit 0
