#///////////////////////////////////////////////////////////////////////////////
#// $Id: 50_nav_update.b,v 1.3 2002/12/24 21:33:08 gmaxe Exp $
#//
##! @file
##! @brief Recreates the entire master TOC and index for the system.
##!
##! This calls either 55_nav_gen.b or 55_nav_cp.b to make sure that all
##! mini-TOC and mini-index files have been generated for all specified
##! sub-projects and have been copied into the proper location for
##! subsequent process by 56_master_index.b and 56_master_script.b.
##! 
##! In addition, the 55_nav_gen.b calls make sure that all HTML pages
##! are consistent with respect to their top/bottom navigation, copyright
##! notices, and other HTML internals (e.g., CSS callout, java applet).
##! 
##! @bug This file need to be maintained for each bigger project whenever
##! sub-projects are added or removed.
##! Also, this script determines which master files are used for the navigation.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// @author Glenn C. Maxey
#//
#// $Log: 50_nav_update.b,v $
#// Revision 1.3  2002/12/24 21:33:08  gmaxe
#// *** empty log message ***
#//
#// Revision 1.2  2002/12/24 19:00:39  gmaxe
#// New structure and new version using TOC and all.
#//
#// Revision 1.1  2002/12/24 15:27:45  gmaxe
#// New dir structure.
#//
#// Revision 1.5  2002/07/22 23:06:34  gmaxe
#// Added project parameter to pl calls.
#//
#// Revision 1.4  2002/01/21 17:39:52  gmaxe
#// Changed comment style.
#//
#//
#///////////////////////////////////////////////////////////////////////////////

# assumes that you are in the <rtfm/>techpubs/tp_tools directory  <home> where this
# script was found. ... 
echo "Entering 50_nav_update..."
set def_ver = 'v2-00-0a'
if ($#argv > 0) then
   @ cnt = 1
   while ($#argv)
      # echo "$argv[1] and $cnt"
      if ($cnt == 1) then
         set rel_ver = $argv[1]
      endif
      @ cnt++
      shift
   end
   /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html $rel_ver
   set ret_val = $status
   if ($ret_val > 0) then
      echo "WARNING: Problems with master_update.pl 1."
      exit 1
   endif
else
   echo -n "Enter latest build version [$def_ver]:"
   set rel_ver = $<
   if ($rel_ver =~ "") then
      set rel_ver = $def_ver
      # If we're just using the default version from this script, then
      # the master itself will probably have more accurate version numbers.
      echo "Version from master file will be used and not '$rel_ver'."
      /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html
      set ret_val = $status
      if ($ret_val > 0) then
         echo "WARNING: Problems with master_update.pl 2."
         exit 1
      endif
      echo "Version existing in master file was used and not the default [$rel_ver]."
   else
      # we entered a valid version and should update with that date.
      /rtfm/techpubs/perl/master_update.pl voyant_master_nav.html $rel_ver
      set ret_val = $status
      if ($ret_val > 0) then
         echo "WARNING: Problems with master_update.pl 3."
         exit 1
      endif
   endif
endif
# echo "Will look here: $rel_ver"



echo "Give all htm* files the same navigation bars, footers, etc."

55_nav_gen.b doc_publish/ book_tp_tools voyant_master_nav.html project_toc.txt        
55_nav_gen.b doc_publish/ cref_tp_perl voyant_master_nav.html project_toc.txt        
55_nav_gen.b doc_publish/ cref_tp_script voyant_master_nav.html project_toc.txt        
55_nav_cp.b doc_publish/ book_help_on_help voyant_master_nav.html project_toc.txt        

# It is important to get the navigation in any files in the main
# doc_publish directory, but the script files are probably
# of no use and aren't specified in the voy_order. The index files
# might be useful.
/rtfm/techpubs/perl/voyant_nav.pl doc_publish/ voyant_master_nav.html project_toc.txt
# cp doc_publish/_index_list zdoc_merge/index_doc_publish
# cp doc_publish/tree.script zdoc_merge/tree_doc_publish.script

echo "cp m_toc.html doc_publish/"
cp m_toc.html doc_publish/

56_nav_script.b
56_nav_index.b

echo "Okay 50_nav_update.b "
exit 0
