#///////////////////////////////////////////////////////////////////////////////
#// $Id: 55_nav_gen.b,v 1.2 2002/12/24 21:36:23 gmaxe Exp $
#//
##! @file
##! @brief Creates the mini-TOC and mini-index navigation files for
##! the specified project using information extracted from two input files.
##! 
##! @param nav_path The starting location to look for subdirectories.
##! Generally, this is "doc_publish/". It was broken out of the full
##! path, because the sim_name parameter was needed for other purposes. 
##! 
##! @param sim_name The simplified name of the destination project.
##! This parameter is used to locate both the source HTML files 
##! (as in <nav_path><sim_name> but also to name generated mini-index
##! and mini-TOC files appropriately for the project.
##! 
##! @param master_nav The name of an master HTML file that has common HTML 
##! fragments which are to be inserted in all generated HTML pages.
##! 
##! @param master_proj The name of the project_toc.txt file that defines
##! all directories and their associated names, pdf files, and document numbers.
##! 
##! This calls the voyant_nav.pl perl program. It processes the files specified
##! in the <nav_path><sim_name> directory. It requires the master_nav 
##! file and the master_proj file.
##! 
##! The master_nav file is an HTML file that contain tags which are to be
##! placed in every single HTML file in the directory. 
##! 
##! The voyant_nav.pl perl program generates a mini-index and mini-TOC file.
##! When finished, this script needs to copy them to a known location for
##! subsequent processing. It must name them uniquely for the project, which
##! is why the <sim_name> is required to be broken out.
##!
##! In addition, getting individual project directories updated with the
##! latest CSS was error prone. This handles it while processing, so that
##! it doesn't get forgotten.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: 55_nav_gen.b,v $
#// Revision 1.2  2002/12/24 21:36:23  gmaxe
#// Commented version.
#//
#//
#///////////////////////////////////////////////////////////////////////////////
# assumes that you are in the <rtfm/>techpubs/<home> directory 
# where this script resides

set nav_path=$1
set sim_name=$2
set master_nav=$3
set master_proj=$4
# echo "Give all htm* files the same navigation bars, footers, etc."
echo "Entering 55_nav_gen with ${nav_path} ${sim_name} ${master_nav} ${master_proj}..."

/rtfm/techpubs/perl/voyant_nav.pl ${nav_path}${sim_name}/ ${master_nav} ${master_proj}
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: voyant_nav.pl didn't build ${nav_path}${sim_name}."
   exit $ret_val
endif


cp ${nav_path}${sim_name}/_index_list zdoc_merge/index_${sim_name}
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Could not find index_${sim_name}."
   exit $ret_val
endif

cp ${nav_path}${sim_name}/tree.script zdoc_merge/tree_${sim_name}.script
set ret_val = $status
if ($ret_val > 0) then
   echo "ERROR: Could not find tree_${sim_name}."
   exit $ret_val
endif

echo "copy CSS into ${nav_path}${sim_name}" 
cp /rtfm/techpubs/web_files/*.css ${nav_path}${sim_name}

echo "Okay 55_nav_gen.b ${nav_path} ${sim_name} ${master_nav} "
exit 0
