#///////////////////////////////////////////////////////////////////////////////
#// $Id: 56_nav_index.b,v 1.2 2002/12/24 21:36:23 gmaxe Exp $
#//
##! @file
##! @brief Generates master index pages for all found mini-index files
##! in the specified directory.
##!
##! This makes some assumptions about where template files are found
##! and where to process them.
##! 
##! The template file for an individual index page should be found in
##! /rtfm/techpubs/web_files/voyant_master_index.html
##! If this is not the directory or file you want to use, you can change it
##! here in one place. Generally, all of my projects can get by with only
##! one template file for the index.
##! 
##! This script copies the index file into all possible letters.
##! Then it calls the voyant_indexer.pl with the same template file
##! and the destination directory. The voyant_indexer.pl will overwrite
##! any default index letter files with what it generates. However, you 
##! need to have all default files for all letters in order for the 
##! navigation in the index to work.
##! 
##! Depending upon the number of mini-index files that are in the 
##! specified directory, voyant_indexer.pl can churn away for more than
##! just a few minutes.
##! 
##! When finished, this script copies the generated index files into 
##! the destination for publishing.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: 56_nav_index.b,v $
#// Revision 1.2  2002/12/24 21:36:23  gmaxe
#// Commented version.
#//
#//
#///////////////////////////////////////////////////////////////////////////////
# assumes that you are in the <rtfm/>techpubs/<home> directory 
# where this script resides
echo "Entering 56_nav_index..."

# Expect this to fail most of the time
mkdir zdoc_merge/

echo "Prepare by removing the old zdoc masters."
rm zdoc_merge/m_idx_*.*

echo "Default the master index files."
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_a.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_b.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_c.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_d.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_e.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_f.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_g.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_h.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_i.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_j.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_k.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_l.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_m.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_n.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_o.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_p.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_q.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_r.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_s.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_t.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_u.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_v.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_w.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_x.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_y.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_z.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_0.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_1.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_2.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_3.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_4.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_5.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_6.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_7.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_8.html
cp /rtfm/techpubs/web_files/voyant_master_index.html zdoc_merge/m_idx_9.html

echo "Create master index files."
/rtfm/techpubs/perl/voyant_indexer.pl zdoc_merge/ /rtfm/techpubs/web_files/voyant_master_index.html /rtfm/techpubs/perl/ignoreterms.cfg

echo "cp zdoc_merge/m_idx_*.htm* doc_publish/"
cp zdoc_merge/m_idx_*.htm* doc_publish/


echo "Okay 56_nav_index.b  "
exit 0
