#///////////////////////////////////////////////////////////////////////////////
#// $Id: 56_nav_script.b,v 1.2 2002/12/24 21:36:24 gmaxe Exp $
#//
##! @file
##! @brief Generates a master TOC script file appropriate for all found
##! mini-TOC files in the specified directory.
##! 
##! This script calls voyant_mt_app.pl (master tree application). This
##! looks at all tree_<...>.script files in the specified project file.
##! If a tree file doesn't exist, a warning is output.
##! It creates a master tree script file that references the mini-TOC tree script
##! files.
##! 
##! In addition, it creates master_tree script files for each of the
##! mini-TOC files, whereby the paths to the individual topics are resolved
##! for where they actually reside with respect to the top-level publication
##! directory.
##! 
##! In other words, the mini-TOC tree files are intended to work from where
##! they were generated. When plugged into a bigger system, the master tree
##! navigation files typically reside at a lever higher.
##! 
##! This two-stage process has benefits, most of them along the lines of
##! how a modular system can be generated from different TOC files. A given
##! directory can be symbolically linked in several different places and
##! be referenced in several different project files. As such, information
##! can be re-used efficient with accurate master TOCs for the project.
##!
##! @ingroup tp_tools tp_shell
##!
##! @author Glenn C. Maxey
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: 56_nav_script.b,v $
#// Revision 1.2  2002/12/24 21:36:24  gmaxe
#// Commented version.
#//
#//
#///////////////////////////////////////////////////////////////////////////////
# assumes that you are in the <rtfm/>techpubs/<home> directory 
# where this script resides
echo "Entering 56_nav_script..."

# Expect this to fail most of the time
mkdir zdoc_merge/

echo "Prepare by removing the old zdoc masters."
rm zdoc_merge/m_tree*.script

echo "Create the master TOC script files."
/rtfm/techpubs/perl/voyant_mt_app.pl zdoc_merge/ project_toc.txt
set ret_val = $status
if ($ret_val > 0) then
   echo "WARNING: Problems creating the master TOC."
endif

echo "rm doc_publish/m_tree*.script"
rm doc_publish/m_tree*.script
echo "cp zdoc_merge/m_tre*.script doc_publish/"
cp zdoc_merge/m_tre*.script doc_publish/


#####
# Special case
# Need to handle the special nested m_tree* scripts for vxworks.
#####
# cp doc_publish/book_vxworks/m_tree*.script doc_publish/


echo "Okay 56_nav_script.b  "
exit 0
