#!/usr/local/bin/perl

###############################################################################/
##
##! @file
##! @brief Changes the b shell script files so that it "looks" like a 
##! C file with respect to the comment format and function names.
##! 
##! This is intended as an input filter to Doxygen.
##!
##! @param Input file that uses shell (hash-hash-bang) comments.
##! @return Output is a file that uses C-style comments required
##! by Doxygen. 
##!
##! If a line begins with hash-hash-bang, it is considered part of a Doxygen comment
##! block.
##! Lines that only have hash are turned into // comments and are later ignored
##! by Doxygen.
##!  
##! @ingroup tp_tools
##!
##! @author Glenn C. Maxey
##
## $Id: csh_comment_chg.pl,v 1.1 2002/12/24 20:16:51 gmaxe Exp $
##
## 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Permission to use, copy, modify, and distribute this software and its 
## documentation under the terms of the GNU General Public License is hereby 
## granted. No representations are made about the suitability of this software 
## for any purpose. It is provided "as is" without express or implied warranty. 
## See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
## for more details.
## 
## Documents produced by this script are derivative works derived from the 
## input used in their production; they are not affected by this license.
##
## $Log: csh_comment_chg.pl,v $
## Revision 1.1  2002/12/24 20:16:51  gmaxe
## Used for changing shell scripts for doxygen.
##
## Revision 1.7  2002/04/06 01:55:13  gmaxe
## New files and new general constructs for handling tags. Uses globals better.
## The html_look* files are designed to handle spider tracing of html systems.
##
## Revision 1.6  2002/01/16 19:49:08  gmaxe
## Removed much of revision history in comments.
##
##
###############################################################################/

BEGIN {
   local ($comment_count) = 0;  # count if you're in the comments somewhere.
   print "\/\/ This file has been modified on-the-fly with an input filter\n";
   print "\/\/ to change it from shell syntax to C++ strictly for the purposes\n";
   print "\/\/ of faking out Doxygen. Modifications include:\n\n";
   print "\/\/ - changing \# comments to C++ comments.\n";
   print "\/\/ - ...\n\n";
   print "\/\/ If you see other strangeness in the HTML version of the script file,\n";
   print "\/\/ it comes from getting it to look more C++ like.\n\n\n";
}

NEW_LINE: while (<>) {

   ######### 
   # Begin replacement of ##! comments with /** ... **/ comments
   # This purposely does not do NEW_LINE or printing.
   ######### 
   if (/^\#\#\!/) {
      if ($comment_count == 0){
         # first line of a comment block
         $comment_count++;
         $_ =~ s/\#\#\!/\/\*\*/;
      } else {
         # Some line in the middle of a comment block.
         $comment_count++;
         ###
         # Changed to have middle stuff with asterix.
         # $_ =~ s/\#\#\!//;
         ###
         $_ =~ s/\#\#\!/ \*\*/;
      }
   } elsif ($comment_count > 0){
      # We were in a comment block; need to terminate it.
      $comment_count = 0;
      $_ = " \*\*\/\n" . $_ ;
   }
   ######### 
   # This purposely does not do NEW_LINE or printing.
   ######### 
   ######### End Comment style change.

   ######### 
   # Begin replacement of # comments with // comments
   # This purposely does not do NEW_LINE or printing.
   ######### 
   if (/^\#\!/) {
      # Take care of the first line in a perl file.
      $_ =~ s/\#\!/\/\/ \#\!/;
   } elsif (/^[\#]/) {
      # Take care of lines that begin with a # perl comment
      $_ =~ s/\#/\/\//;
   } elsif (/[\#]/) {
      # Take care of lines that have a # perl comment somewhere
      $_ =~ s/\#/\/\/ /;
   }
   ######### 
   # This purposely does not do NEW_LINE or printing.
   #########
   if (/globe\'/){
      $_ =~ s/globe\'/globe\_/g;
   }
   ######### End Comment style change.

   ######### 
   # Begin replacement of \bug or @bug to @lim
   ######### 
   if (/\\bug|\@bug/) {
      $_ =~ s/\\bug/\@lim/;
      $_ =~ s/\@bug/\@lim/;
      print $_;
      next NEW_LINE;
   } 
   
   ######### 
   # Default Fall through
   ######### 
   print $_;

}  








