#!/usr/local/bin/perl

#///////////////////////////////////////////////////////////////////////////////
#//
##! @file
##! @brief Used when Doxygenating C/C++ code files.
##!
##! This is a tool to change the Doxygen comment style from C-style comments
##! to comments with slash-slash-bang. The reason is that influential members of the coding
##! convention group decided that they liked more C++ style comments and
##! did not want to have mixed comment styles. Hence, our coding conventions
##! specify this style.
##!
##! This tool changes existing C style comments to this slash-slash-bang comments.
##!
##! @note Input filters need to be used to change those slash-slash-bang comments into
##! the other style, because the other style works better in Doxygen.
##! If the engineers want to see one style comments in the code that they own,
##! more power to them. The filters will get it back to what Doxygen likes better.
##! 
##! I suspect that this tool will not actually be used much despite the coding
##! conventions. Existing comment styles are grandfathered in. Also, new
##! development tools like TogetherSoft produce more JavaDoc like files and
##! prefer those comments. Hence, another suspicion is that our coding conventions
##! will flop back to the C-style JavaDoc comments making this tool
##! never used.
##!
##! @param Input source file.
##! @return Output is file with changes.
##!
##! @ingroup tp_tools tp_dox
##!
##! @author Glenn C. Maxey
#//
#// $Id: dox_comment_chg.pl,v 1.6 2002/04/06 01:55:09 gmaxe Exp $
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: dox_comment_chg.pl,v $
#// Revision 1.6  2002/04/06 01:55:09  gmaxe
#// New files and new general constructs for handling tags. Uses globals better.
#// The html_look* files are designed to handle spider tracing of html systems.
#//
#// Revision 1.5  2002/01/16 19:49:07  gmaxe
#// Removed much of revision history in comments.
#//
#//
#///////////////////////////////////////////////////////////////////////////////

BEGIN {
   local ($comment_count) = 0;  # count if you're in the comments somewhere.
}

NEW_LINE: while (<>) {

   ######### 
   # Begin replacement of /** ... **/ comments with //! comments
   # This purposely does not do NEW_LINE or printing.
   #########
   if (/^\/[\*]*\//) {
      $_ =~ s/[\*]/\//g;
   } 
   if (/^[\s]*\/\*\*/) {
      $comment_count = 1;
      $_ =~ s/[\s]*\/\*\*/\/\/\!/;
   } elsif (/[\s]*\*\*\//) {
      $comment_count = 0;
      $_ =~ s/[\s]*\*\*\//\/\/\!/;
   } elsif (/[\s]*\*\*/) {
      $comment_count++;
      $_ =~ s/[\s]*\*\*/\/\/\!/;
   }

   
   ######### End Comment style change.
    
   ######### 
   # Default Fall through
   ######### 
   print $_;

}  







