#!/usr/local/bin/perl

#///////////////////////////////////////////////////////////////////////////////
##! @file
##! @brief Input filter for Doxygen for most IVE code files.
##! 
##! This "fakes-out" Doxygen into thinking that IVE (Pascal-like) code files
##! are really C files.
##!
##! @param Input source file.
##! @return Output is file with changes.
##!
##! @ingroup tp_tools tp_dox
##!
##! @author Glenn C. Maxey
#//
#// $Id: dox_ive_filter.pl,v 1.6 2002/01/16 19:49:08 gmaxe Exp $
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: dox_ive_filter.pl,v $
#// Revision 1.6  2002/01/16 19:49:08  gmaxe
#// Removed much of revision history in comments.
#//
#// Revision 1.5  2002/01/03 00:25:07  gmaxe
#// Changed the license agreement.
#//
#///////////////////////////////////////////////////////////////////////////////

BEGIN {
   $inEnum = 0;
   $inVar = 0;
   $inFunDecl = 0;
   $enum_name = "_nothing_";
   # print "Starting up...\n";
   local ($comment_count) = 0;  # count if you're in the comments somewhere.
}

LINE_IN: while (<>) {


   ######### 
   # Begin replacement of //! comments with /** ... **/ comments
   # This purposely does not do NEW_LINE or printing.
   ######### 
   if (/^\/\/\!/) {
      if ($comment_count == 0){
         # first line of a comment block
         $comment_count++;
         $_ =~ s/\/\/\!/\/\*\*/;
      } else {
         # Some line in the middle of a comment block.
         $comment_count++;
         ###
         # Changed to have middle stuff with no asterix.
         # $_ =~ s/\/\/\!/ \*\*/;
         ###
         $_ =~ s/\/\/\!//;
      }
   } elsif ($comment_count > 0){
      # We were in a comment block; need to terminate it.
      $comment_count = 0;
      $_ = " \*\*\/\n" . $_ ;
   }
   ######### End Comment style change.


   ######### 
   # Begin comment out var's
   # need to know whether or not a function declaration is being made
   ######### 
   if ((/^function/) || (/^vfunction/) || (/^xfunction/)) {
      if (/\)[\s]*\;/) {
         # function declaration begins and ends on this line
         # it's really a prototype
         $inFunDecl = 0;
      } elsif (/\)[\s]*\n/) {
         # function declaration ends on this line
         $inFunDecl = 1;
      } else {
         # declaration may end on another line
         $inFunDecl = 1;
      }
      print $_;
      next LINE_IN;
   }
   
   if (($inFunDecl) && (/\)[\s]*\;/)) {
      # function declaration begins and ends on this line
      # it's really a prototype
      $inFunDecl = 0;
      print $_;
      next LINE_IN;
   }
  
   if (($inFunDecl) && ( /\)/ )) {
      # function declaration began on another line and ends on this line
      # need to remember that a declaration was started
      $inFunDecl = 1;
      print $_;
      next LINE_IN;
   }
  
   # Variables are handled differently
   if (($inFunDecl) && (/^var[\s]*/)) {
      $inVar = 1;
      $inFunDecl = 0;
      print "\/\/\+\+ $_"; 
      next LINE_IN;
   }
   
   if (($inFunDecl) && (/^begin[\s]*/) ) {
      $inVar = 0;
      $inFunDecl = 0;
      # print ";  \/\/\+\+ turns into C++ prototype; not needed in IVE\n{  \/\/\+\+ $_";
      print "{  \/\/\+\+ $_";
      next LINE_IN;
   }
  
   if (($inVar) && (/^begin[\s]*/) ) {
      $inVar = 0;
      $inFunDecl = 0;
      # print ";  \/\/\+\+ turns into C++ prototype; not needed in IVE\n{  \/\/\+\+ $_";
      print "{  \/\/\+\+ $_";
      next LINE_IN;
   } elsif ($inVar) {
      # Comment out any variable lines
      print "\/\/\+\+ $_";
      next LINE_IN;
   }
   ######### End comment out var's

   ######### 
   # Script functions use Pascal-like "begin" and "end" that need to be fixed.
   ######### Begin replacement of "begin/end"
   if (/^begin[\s]*/) {
      $inVar = 0;
      $inFunDecl = 0;
      # Convert the IVE "begin" statement into something more C/C++ like.
      # print "{  \/\/\+\+ $_  \/\/ The curly brace is non-standard IVE syntax.\n";
      # print ";  \/\/\+\+ turns into C++ prototype; not needed in IVE\n{  \/\/\+\+ $_";
      print "{  \/\/\+\+ $_";
      next LINE_IN;
   } 
   if (/^end[\s]*/) {
      # Convert the IVE "end" statement into something more C/C++ like.
      # print "}  \/\/\+\+ $_  \/\/ The curly brace is non-standard IVE syntax.\n";
      print "}  \/\/\+\+ $_";
      $inVar = 0;
      $inFunDecl = 0;
      next LINE_IN;
   } 
   ######### End replacement of "begin/end"


   
   ######### 
   # Enumerations are also handled differently if it is a script.
   # Reformats the enum call to be more C-like for Doxygen.
   ######### Begin Enum
   if (/^enum[ \t]+[a-zA-Z][a-zA-Z0-9_]*[ \t]*$/) {
      $enum_name = $_;   # remember the enumeration name for later
      $enum_name =~ s/enum//;
      $enum_name =~ s/\s//g;
      $inEnum = 1;
      # Doxygen comments begin
      print "\/\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\/"; 
      print "\/\*\* $enum_name is an IVE enumeration. " ;

      print " \*\* \@ingroup s_func" ;
      print " \*\*\/"; 
      print " \/\/ If you are reading this in the online HTML system,";
      print " \/\/ note that syntax is NOT the exact IVE syntax.";
      print "\/\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\/"; 
      # Doxygen comments end

      # Enum Prototype Definition begin
      print "typedef enum\n{"; 
      next LINE_IN;
   }
   if (($inEnum) && (/^[ \t]*[a-zA-Z][a-zA-Z0-9_]*[ \t]*$/)) {
      print $_; 
      next LINE_IN;
   }

   if (($inEnum) && (/^[ \t]*[a-zA-Z][a-zA-Z0-9_]*[ \t]+=[ \t]+-*[0-9]+[ \t]*$/)) {
      print $_; 
      next LINE_IN;
   }

   if (($inEnum) && (/^endenum;[ \t]*$/)) {
      print "}\n$enum_name;\n";
      $enum_name = "_nothing_";   # reset the enumeration name
      $inEnum = 0;
      next LINE_IN;
   }
   ######### End Enum


   
   ######### 
   # Begin replacement of \bug or @bug to @lim
   ######### 
   if (/\\bug|\@bug/) {
      $_ =~ s/\\bug/\@lim/;
      $_ =~ s/\@bug/\@lim/;
      print $_;
      next LINE_IN;
   } 
   ######### End replacement of \bug or @bug to @lim


   ######### Default fall through
   print $_;

} # end of while loop


