#!/usr/local/bin/perl

#///////////////////////////////////////////////////////////////////////////////
##! @file
##! @brief Input filter for Doxygen for most VxWorks code files.
##! 
##! This changes code comments and other items on-the-fly so that it can be
##! more effectively processed by Doxygen. 
##! 
##!
##! @param Input source file.
##! @return Output is file with changes.
##!
##! @ingroup tp_tools tp_dox
##!
##! @author Glenn C. Maxey
##
#// $Id: dox_vxworks_filter.pl,v 1.1 2002/02/16 00:59:40 gmaxe Exp $
#//
#// 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
#//
#// Permission to use, copy, modify, and distribute this software and its 
#// documentation under the terms of the GNU General Public License is hereby 
#// granted. No representations are made about the suitability of this software 
#// for any purpose. It is provided "as is" without express or implied warranty. 
#// See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
#// for more details.
#// 
#// Documents produced by this script are derivative works derived from the 
#// input used in their production; they are not affected by this license.
#//
#// $Log: dox_vxworks_filter.pl,v $
#// Revision 1.1  2002/02/16 00:59:40  gmaxe
#// Added support for script files needed for TOC applet.
#//
#//
#///////////////////////////////////////////////////////////////////////////////

BEGIN {
   local ($comment_tot_count) = 0;  # total number of comments.
   local ($comment_count) = 0;  # count if you're in the comments somewhere.
   local ($period_count) = 0;  # count of number of periods in brief statement.
}

NEW_LINE: while (<>) {

   ######### 
   # Begin replacement of //! comments with /** ... **/ comments
   # This purposely does not do NEW_LINE or printing.
   ######### 
   if ((/^\/\*\*\*/) && (!/\*\//)) {
      # if first comment has lots of asterix
      if ($comment_count == 0){
         # first line of a comment block
         $comment_count++;
         print "\/\*\*\n";
         if ($comment_tot_count == 0){
            $comment_tot_count++;
            print "\@file\n";
            # print "\@brief ";
         } else {
            print "\@brief ";
         }
         next NEW_LINE;
      }
   } elsif ((/^\/\*[\s]*/) && (!/\*\//)) {
      # if first comment is standard
      if ($comment_count == 0){
         # first line of a comment block
         $comment_count++;
         print "\/\*\*\n";
         if ($comment_tot_count == 0){
            $comment_tot_count++;
            print "\@file\n";
            # print "\@brief ";
         } else {
            print "\@brief ";
         }
         next NEW_LINE;
      }
   } elsif (!(/\*\//) && ($comment_count > 0)){
      # We were in a comment block; 
      $comment_count++;
      $_ =~ s/^[\s]*[\*][\s]+//g;
      $_ =~ s/^[\s]*[\*\*][\s]+//g;
      $_ =~ s/^[\s]*[\*\*]//g;
      if (($_ !~ /[\w]+/) && ($comment_count < 3)){
         # if the line is empty now and we're still near beginning
         # of comment block, remove it
         next NEW_LINE;
      }
      if ((0) && ($_ =~ /\./) && ($period_count == 0)){
         # add a carriage return
         $_ =~ s/\./\.\n\n/;
         $period_count++;
      }
      $_ =~ s/Abstract\:/\@brief/;
      $_ =~ s/Authors\:/\@author/;
      $_ =~ s/Caveats\:[\s]*/\@lim /;
      $_ =~ s/Argument\(s\)\:/\@param/;
      $_ =~ s/Return Value\(s\)\:/\@retval/;
      $_ =~ s/RETURNS\:/\@return/;
      $_ =~ s/SEE ALSO\:/\@sa/;
      if ($comment_count == 3){
         $comment_count++;
         print "$_\.\n\n";
         next NEW_LINE;
      } 
      
   } elsif ((/\*\//) && ($comment_count > 0)){
      # We were in a comment block; need to terminate it.
      $comment_count = 0;
      $period_count = 0;
      print "\@ingroup vxworks\n";
   
   }
   ######### End Comment style change.
    


   ######### 
   # Begin replacement of \bug or @bug to @lim
   ######### 
   if (/\\bug|\@bug/) {
      $_ =~ s/\\bug/\@lim/;
      $_ =~ s/\@bug/\@lim/;
      print $_;
      next NEW_LINE;
   } 
   
   ######### 
   # Default Fall through
   ######### 
   print $_;

}  







