#!/usr/local/bin/perl
package globe;

##############################################################################
##! @file
##! @brief Defines global variables that can be used by other programs.
##! 
##! Most of the variables refer to tags that we expect to find in the 
##! HTML files. Others define variables that we want to use in a global way.
##!
##! Tag names that are used in the master template files, all HTML files of
##! the system, Mif2Go mif2htm.ini files, and Doxgyen header/footer files
##! are defined here for all programs. They are in one place to facilitate
##! their editing and maintenance.
##!
##! @ingroup tp_tools
##!
##! @author Glenn C. Maxey
##!
##    $Id: globe.pm,v 1.26 2002/11/21 17:17:57 gmaxe Exp $
##
## 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Permission to use, copy, modify, and distribute this software and its 
## documentation under the terms of the GNU General Public License is hereby 
## granted. No representations are made about the suitability of this software 
## for any purpose. It is provided "as is" without express or implied warranty. 
## See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
## for more details.
## 
## Documents produced by this script are derivative works derived from the 
## input used in their production; they are not affected by this license.
##
##    Revision Information:
##
##       $Log: globe.pm,v $
##       Revision 1.26  2002/11/21 17:17:57  gmaxe
##       Tools to automatically update version and date in an HTML file.
##       Important when SCM runs our tool.
##
##       Revision 1.25  2002/10/11 20:20:05  gmaxe
##       Minor changes to condition for target testing.
##
##       Revision 1.24  2002/08/08 21:37:57  gmaxe
##       Further enhancements for debugging and to get the man pages
##       generated properly.
##
##       Revision 1.23  2002/08/07 01:27:11  gmaxe
##       Changed the {comment} to a {comment_full} where appropriate.
##       Enhanced the ini gen as appropriate to get the desired output.
##
##       Revision 1.22  2002/08/05 21:46:17  gmaxe
##       Added more debugging; fixed problems with levels; added output file
##       that is a tagged sample of what came in for ease in converting data.
##
##       Revision 1.21  2002/08/03 00:58:24  gmaxe
##       Updates before the weekend. Now it parses target files better;
##       had problems with scoping for variables.
##
##       Revision 1.19  2002/07/26 18:56:09  gmaxe
##       Got rid of old definitions and migrated new structures into all;
##       enhanced xhelp output file names and format; now everything is
##       alphebetized.
##
##       Revision 1.18  2002/07/26 00:28:44  gmaxe
##       Added support for xscope::sapi and other sapi data structures so
##       that its unique generation requirements are handled properly.
##       Output also sorts things alphebetically, a requirement for doxygen.
##
##       Revision 1.17  2002/07/22 21:51:29  gmaxe
##       Created separate configuration file; allow for two inputs to
##       the voyant_latex.pl. commented out old variables in globe.pm.
##
##       Revision 1.16  2002/07/03 23:07:22  gmaxe
##       Added support for mif2go index tokens. Later, remove the old
##       style index tokens and objects.
##
##############################################################################

##############################################################################
##! @fn int BEGIN
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_tools
##############################################################################
sub BEGIN {
   if (0) {
      print "globe.pm BEGIN routine.\n";
   }
} # BEGIN

##############################################################################
##! @fn int main
##! @brief Main routine.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_tools
##############################################################################
# sub main {
{
   ######
   ## Main Subroutine doesn't do anything
   ######
   if (0) {
      print "globe.pm main routine.\n";
   }
   
   if (0){
      print "=== Definitions 1 \n";
      print "=== voy header defb ==== $globe::m_define{header}[0]\n";
      print "=== voy header defb ==== $globe::m_define{header}[0]\n";
   }
   
   # Needed in order for the package to work correctly from other perl programs.
   return(1);
}

##############################################################################
##! @fn int declare_variables
##! @brief Allocates memory for all globally used variables.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_tools
##############################################################################
sub declare_variables {

   ##############################################################################
   ## Declarations: all variables are global.
   ## All of the following are used in voyant_nav.pl (the first program).
   ## Some are used in other programs.
   ##############################################################################
   @valid_f_type = ("htm");
   @special_files = ("index.html", "tree.html", "topper.html", "side.html");
   @exclude_prefix = ("_start_");
   $path = "./";
   $master_nav_file = "none";
   $master_nav = "";
   #####
   # structure to define common tags in files.
   # Array element [0] is the start tag.
   # Array element [1] is the end tag.
   #####
   @{$globe::m_define{date}} = ("<!-- voy_date -->", "<!-- end -->");
   @{$globe::m_define{version}} = ("<!-- voy_version -->", "<!-- end -->");
   @{$globe::m_define{header}} = ("<!-- begin voy_header -->", "<!-- end voy_header -->");
   @{$globe::m_define{nav_common}} = ("<!-- begin voy_common_top -->", "<!-- end voy_common_top -->");
   @{$globe::m_define{nav_doxygen}} = ("<!-- begin voy_dox -->", "<!-- end voy_dox -->");
   @{$globe::m_define{nav_book}} = ("<!-- begin voy_fm_book -->", "<!-- end voy_fm_book -->");
   @{$globe::m_define{footer}} = ("<!-- begin voy_footer -->", "<!-- end voy_footer -->");
   @{$globe::m_define{variable}} = ("<!-- begin voyant_variables", "## -->");  # watchout! Not complete!
   @{$globe::m_define{html_zap}} = ("<!-- begin voy_html_zap -->", "<!-- end voy_html_zap -->");
   @{$globe::m_define{order}} = ("<!-- begin voy_order --", "!-- end voy_order -->");
   @{$globe::m_define{topic_list}} = ("<!-- begin voy_topic_list -->", "<!-- end voy_topic_list -->");
   @{$globe::m_define{title}} = ("<title>", "</title>");
   #####
   # 07/03/2002 Will eventually no longer be needed.
   #####
   @{$globe::m_define{index_token}} = ("<!-- index_token \"", "\" !-->");
   #####
   # 07/03/2002 Newly supported Mif2Go index tokens that I export as:
   # <a name="zzzz" class="v_index" value="token text">
   # I'll keep the old structure around until the new can be completely implemented.
   # Eventually, I'll no longer be generating the old style <!-- index_token !--> comments.
   #####
   @{$globe::m_define{index_token2}} = ("<a name=\"", ">", "class=\"v_index", "\"", "value=\"", "\"");
   # @{$globe::m_define{index_token2}} = ("<a ", ">", "class=\"v_index", "\"", "value=\"", "\"");
   
   # used in voyant indexer
   @{$globe::m_define{structure}} = ("<!-- begin voy_structure -->", "<!-- end voy_structure -->");
   # used in voyant latex
   @{$globe::m_define{latex}} = ("<!-- begin voy_latex --", "!-- end voy_latex -->");  
   @{$globe::m_define{latex_header}} = ("<!-- begin voy_latex_head --", "!-- end voy_latex_head -->");

   @{$globe::x_define{logmsg}} = ("<logmsg", ">", "</logmsg>");  
   @{$globe::x_define{initag}} = ("<initag", ">", "</initag>");  
   # 07/31/2002 @{$globe::x_define{isect}} = ("\[", "\]", "\[\]");  
   @{$globe::x_define{comment}} = ("\/\/\!", "\/\/", "\/\*\*", "\/\*", "\*\*", "\#\#\!", "\#\#", "\#");  
   @{$globe::x_define{comment}} = ("\/\/\!", "\/\/", "\#\#\!", "\#\#", "\#");  
 

   $m_var{define}{curr} = "##curr##";
   $m_var{define}{curr2} = "##curr2##";  # needed to separate header needs from mif2go.
   $m_var{define}{prev} = "##prev##";
   $m_var{define}{next} = "##next##";
   $m_var{define}{firstlast} = "##firstlast##";
   $m_var{define}{end} = "##";
   
   #####
   $voy_curr = "";
   $voy_prev = "";
   $voy_next = "";
   $voy_variable = "";
   $rel_path_to_start_point_def = "##rp2start##";
   $rel_path_to_start_point = "./";
   $pdf_dir = "print_pdf/";
   $this_pdf_flag = "##this_pdf##";
   $fix_path_flag = "zz_go_parallel_zz";
   $fix_path_to = "..";
   $voy_variable_prev_dox = "##prev_dox##";
   $voy_variable_next_dox = "##next_dox##";

   #####
   # The following are variables containing HTML syntax observed in
   # Doxygen output. This is text that I don't want so that it can
   # be replaced with the navigation I do want from the master files.
   # If Doxygen changes, this syntax can also change and would
   # need to be updated.
   #####
   $dox_nav_defb1 = "\<center\>\n\<a class\=\"qindex\" href\=\"main.html\"\>Main Page\<\/a\>";
   $dox_nav_defb2 = "\<center\>\n\<a class\=\"qindex\" href\=\"index.html\"\>Main Page\<\/a\>";
   $dox_nav_defe = "\<\/center>\n\<hr\>";
   $dox_nav_defb3 = "\<a class\=\"qindex\"";
   $dox_nav_defe3 = "\<\/a\>";

   $xmanual = "##xmanual##";
   $xgroup = "##xgroup##";
   undef (%book_topics);         # key is html file name, <Title> is stored
   undef (%book_topic_order);    # key is html file name, next html file is stored
   undef (@book_first_topic);    # ea ch in dir has own first/last
   undef (@book_last_topic);     # ea ch in dir has own first/last
   undef (@file_errors);
   undef (%book_topic_level);    # key is html file name, <Headn> is stored
   undef (@fm_format_levels);    # ("TitleChapter", "Head1", "Head2", "Head3");  
   $last_book_level = 0;
   #######################

   #####
   # CAREFUL: The index numbers are explicitly referenced... In particular,
   # a fake out to equate [4] with [0] is done later in this program
   # If you add other formats to watch, be careful about this.
   #
   # These use FrameMaker format names that are specific to our implementation.
   # If you use other format names, they need to changed here.
   #####
   $fm_format_levels[0] = "NONE"; # Top Level
   $fm_format_levels[1] = "TitleChapter"; # This comes from FM and mif2go.ini
   $fm_format_levels[2] = "Head1";
   $fm_format_levels[3] = "Head2";
   $fm_format_levels[4] = "Head3";
   $fm_format_levels[5] = "TitleAppendix"; # This comes from mif2go.ini
   $fm_format_levels[6] = "TitleFront"; # This comes from mif2go.ini
   $fm_format_levels[7] = "Indexhead"; # This comes from mif2go.ini
   $fm_format_levels[8] = "TitleIndex"; # This comes from mif2go.ini
   $fm_format_levels[9] = "TitleIntro"; # This comes from mif2go.ini
   $fm_format_levels[10] = "TitleNoPrefix"; # This comes from mif2go.ini

   $empty_html = "##empty_html##";
   undef (@voy_html_zap);
   $voy_variable_next_chapter = "##next_chapter##";
   $voy_variable_prev_chapter = "##prev_chapter##";

   $voy_variable_doc_num = "##doc_number##";
   undef (@master_order);  # key is an index, short dir name is stored
   undef (@master_order_title);  # key is an index, short dir name is stored
   undef (@master_order_pdf);  # key is an index, pdf file is stored
   undef (@master_order_doc_num);  # key is an index, doc number is stored
   $master_order_key = "0"; # key into the other master order items

   $voy_h1_b = "<h1";
   $voy_h1_e = "</h1>";
   undef (@index_info);
   #####
   # Index tokens are passed from FM using Mif2Go and its MS-HTML output.
   # Tokens are placed in an object tag.
   # 07/03/2002 object tag support is going away; these will be removed soon
   #####
   $ms_object_b = "<object";  # careful... does not contain closing ">"
   $ms_object_e = "</object>";  
   $ms_value_b = "value\=\"";
   $ms_value_e = "\">";
   # 07/03/2002 above
   
   
   $word_c_boundary = "\:\;\:";
   $word_url_boundary = "\:\,\:";

   #####
   # The following are variables containing HTML syntax observed in
   # Doxygen output. 
   # If Doxygen changes, this syntax can also change and would
   # need to be updated.
   #####
   $gi_filename_clue = "group";  # clue that files beginning with "group" need additional parsing.
   $gi_name = "<a name\=";  # gi: first clue to look for for index
   $gi_dox = "doxytag\="; # gi: second clue to look for for index
   $gi_group_separator = "\:\:"; # gi: separator from group and code item; code item is indexed
   
   #####
   # Used to create additional index tokens so that
   # we can cross reference an packet to the calling functions in the index
   #####
   $ack_used_by = "Used by "; # Reference to who uses it.
   $ack_text_flag = "Acknowledgement Packets\:";  # what to look for in for 
   @ack_ignore = ("_TBD_", "None");
   $ack_end = "\<\/dl\>";  # delimiter for end of reference
   $ack_or = "or";

   $chapter_division = "<!--voy_LEV";
   $chapter_division_e = "-->";

   ##############################################################################
   ## All of the following are used in voyant_indexer.pl.
   ## Some are used in other programs following.
   ##############################################################################
   $master_index_html = "";
   $master_raw = "";   # master list coming from all files
   undef (%master_index);  # key is title, title and url are stored
   undef (@ignore_terms);
   $ignore_terms_file = "";

   $relative_path = "";
   $total_entry_in = 0;
   $total_entry_out = 0;
   $division_mult_entry = "\:\;\;\;\:";

   ##############################################################################
   ## All of the following are used in voyant_latex.pl.
   ## Some are used in other programs following.
   ##############################################################################
   $flag_title = "zz-doctitle-zz";
   $flag_doc_num = "zz-docnum-zz";
   $flag_date = "zz-doxgendate-zz";

   undef (%latex_flags);

   $master_header_tex = "##master_header_tex##";  # Has flag, then filename, then the entire master input file
   $master_doxygen_sty = "##master_doxygen_sty##"; # Has flag, then filename, then the entire master input file
   $output_header_tex = "##output_header_tex##";  # Has flag, then filename, then the entire output file
   $output_doxygen_sty = "##output_doxygen_sty##"; # Has flag, then filename, then the entire output file

   undef (@latex_title);  # The first elements is file, second is latex start, third is latex stop
   $latex_title_page = "";

   ##############################################################################
   ## All of the following are used in voyant_mt_nav.pl, voyant_mt_app.pl
   ## Some are used in other programs following.
   ##############################################################################
   $pdf_name_from_dir = "book_cit_dtoss.pdf";
   $master_tree_html = "";
   $mt_empty_html = "m_tree_master.html";
   $topic_division = "</pre>";
   $nav_doc = "nav_doc.gif";
   $nav_folderopen = "nav_folderopen.gif";
   $nav_folderclosed = "nav_folderclosed.gif";
   $script_prefix = "tree_";
   $script_master = "m_tree.script";
   $script_master_prefix = "m_";

   ##############################################################################
   ## All of the following are used in voyant_gen_nav.pl.
   ## Some are used in other programs following.
   ##############################################################################
   $comment_b = "\<\!\-\-";
   $comment_e = "\-\-\>";
   @toc_level = ("L1", "L2", "L3", "L4", "L5", "L6", "L7", "L8", "L9", "L10");
   $anchor_b = "\<a";
   $anchor_e = "\>";
   $href_b = "href\=";

   if (0){
      print "=== Definitions 2 \n";
      print "=== header ==== $globe::m_define{header}[0]\n";
   }
   ##############################################################################
   ## All of the following are used in tree_js_2_script.pl.
   ## Some are used in other programs following.
   ##############################################################################
   # Name changing for Doxygen Output
   $dox_name_change{Modules} = "Groups";
   $dox_name_change{"File Members"} = "Globals \[C\+\+, IVE\] Functions, Enum Struct, Class, Define, etc.";
   $dox_name_change{"Compound List"} = "Classes (annotated)";
   # $dox_name_change{Classes} = "Classes (listed)";
   $dox_name_change{Hierarchy} = "Class Hierarchy";
   $dox_name_change{"Compound Members"} = "Class Member Functions";
   $dox_name_change{FileList} = "Files in Project";
   

   return (1);

} # declare_variables

##############################################################################
##! @fn int memory_clean_up
##! @brief Cleans up after all variables that were defined.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub memory_clean_up {
   undef (@valid_f_type);     # ("htm");;
   undef (@special_files);     # ("index.html", "main.html", "tree.html");
   undef ($path);     # "./";
   undef ($master_nav_file);     # "none";
   undef ($master_nav);     # "";
   undef ($voy_curr);     # "";
   undef ($voy_prev);     # "";
   undef ($voy_next);     # "";
   undef ($voy_variable);     # "";
   undef ($rel_path_to_start_point_def);     # "##rp2start##";
   undef ($rel_path_to_start_point);     # "./";
   undef ($pdf_dir);     # "print_pdf/";
   undef ($this_pdf_flag);     # "##this_pdf##";
   undef ($fix_path_flag);     # "zz_go_parallel_zz";
   undef ($fix_path_to);     # "..";
   undef ($dox_nav_defb1);     # "\<center\>\n\<a class\=\"qindex\" href\=\"main.html\"\>Main Page\<\/a\>";
   undef ($dox_nav_defb2);     # "\<center\>\n\<a class\=\"qindex\" href\=\"index.html\"\>Main Page\<\/a\>";
   undef ($dox_nav_defe);     # "\<\/center>\n\<hr\>";
   undef ($dox_nav_defb3);   # = "\"qindex\"\";
   undef ($dox_nav_defe3);   # = "\<\/a\>";

   undef ($xmanual);     # "##xmanual##";
   undef ($xgroup);     # "##xgroup##";
   undef (%book_topics);         # key is html file name, <Title> is stored
   undef (%book_topic_order);    # key is html file name, next html file is stored
   undef (@book_first_topic);    # ea ch in dir has own first/last
   undef (@book_last_topic);     # ea ch in dir has own first/last
   undef (@file_errors);
   undef (%book_topic_level);    # key is html file name, <Headn> is stored
   undef (@fm_format_levels);    # = ("TitleChapter", "Head1", "Head2", "Head3");  # This comes from mif2go.ini
   undef ($last_book_level);     # = 0;
   undef ($voy_master_topic_orderb);   # = "<!-- begin voy_master_topic_order -->";
   undef ($voy_master_topic_ordere);   # = "<!-- end voy_master_topic_order -->";
   undef (@voy_master_topic_order);
   undef ($empty_html);           # = "##empty_html##";
   undef (@voy_html_zap);
   undef ($voy_variable_next_chapter);           # = "##next_chapter##";
   undef ($voy_variable_prev_chapter);           # = "##prev_chapter##";
   undef ($voy_variable_doc_num);           # = "##doc_number##";
   undef (@master_order);  # key is an index, short dir name is stored
   undef (@master_order_title);  # key is an index, short dir name is stored
   undef (@master_order_pdf);  # key is an index, pdf file is stored
   undef (@master_order_doc_num);  # key is an index, doc number is stored
   undef ($master_order_key);           # = "0"; # key into the other master order items
   undef ($voy_h1_b); #  = "<h1";
   undef ($voy_h1_e); #  = "</h1>";

   undef ($ms_object_b); #   = "<object";  # careful... does not contain closing ">"
   undef ($ms_object_e); #   = "</object>";  
   undef ($ms_value_b); #   = "value\=\"";
   undef ($ms_value_e); #   = "\">";

   undef ($gi_filename_clue); # = "group";  # clue that files beginning with "group" need additional parsing.
   undef ($gi_name); # = "<a name\=";  # gi: first clue to look for for index
   undef ($gi_dox); # = "doxytag\="; # gi: second clue to look for for index
   undef ($gi_group_separator); # = "\:\:"; # gi: separator from group and code item; code item is indexed
   undef ($chapter_division); # = "<!-- chapter_chunk -->\n";
   
   ##############################################################################
   ## All of the following are used in voyant_indexer.pl.
   ## Some are used in other programs appearing later in file.
   ##############################################################################
   undef ($master_index_html); # = "";
   undef ($master_raw); # = "";   # master list coming from all files
   undef (%master_index);  # key is title, title and url are stored
   undef (@ignore_terms);
   undef ($ignore_terms_file); # = "";

   undef ($relative_path); # = "";
   undef ($total_entry_in); # = 0;
   undef ($total_entry_out); # = 0;
   undef ($division_mult_entry); # = "\:\;\;\;\:";

   ##############################################################################
   ## All of the following are used in voyant_latex.pl.
   ## Some are used in other programs following.
   ##############################################################################
   undef ($flag_title); # = "zz-doctitle-zz";
   undef ($flag_doc_num); # = "zz-docnum-zz";
   undef ($flag_date); # = "zz-doxgendate-zz";

   undef (%latex_flags);

   undef ($master_header_tex); # = "##master_header_tex##";  # Has flag, then filename, then the entire master input file
   undef ($master_doxygen_sty); # = "##master_doxygen_sty##"; # Has flag, then filename, then the entire master input file
   undef ($output_header_tex); # = "##output_header_tex##";  # Has flag, then filename, then the entire output file
   undef ($output_doxygen_sty); # = "##output_doxygen_sty##"; # Has flag, then filename, then the entire output file

   undef (@latex_title);  # The first elements is file, second is latex start, third is latex stop
   undef ($latex_title_page); # = "";

   ##############################################################################
   ## All of the following are used in voyant_mt_nav.pl.
   ## Some are used in other programs following.
   ##############################################################################
   undef ($pdf_name_from_dir); # = "book_cit_dtoss.pdf";
   undef ($master_tree_html); # = "";
   undef ($mt_empty_html); # = "m_tree_master.html";
   undef ($topic_division); # = "</pre>";
   undef ($nav_doc); # = "nav_doc.gif";
   undef ($nav_folderopen); # = "nav_folderopen.gif";
   undef ($nav_folderclosed); # = "nav_folderclosed.gif";
   
   return (1);
}



##############################################################################
##! New
##! @fn int get_master_nav_info  ( )
##! @brief Gets the ordering out of the master file.
##! @param None.
##! @return Stores information into global variables for use in updating variables
##! and other parts of the topic HTML files..
##! 
##! This routine looks for paired HTML comments containing a
##! tag name and "begin" or "end". Whatever is between these paired comments
##! is the updated information to be placed in the individual HTML files.
##! This reads in the entire master navigation file at one time and parses
##! out the pieces of information that are important. Important tags are: <ul>
##!
##! <li> Paired $globe::m_define{header}[0] & $globe::m_define{header}[1] for the
##! header.</li>
##!
##! <li> Paired $globe::m_define{footer}[0] & $globe::m_define{footer}[1] for the
##! footer.</li>
##!
##! <li> Paired $globe::m_define{nav_common}[0] & $globe::m_define{nav_common}[1] for the
##! common navigation bar.</li>
##!
##! <li> Paired $globe::m_define{nav_doxygen}[0] & $globe::m_define{nav_doxygen}[1] for the
##! Doxygen navigation bar.</li>
##!
##! <li> Paired $globe::m_define{nav_book}[0] & $globe::m_define{nav_book}[1] for the
##! FM navigation bar.</li>
##!
##! <li>Sometimes HTML is generated by Doxygen or Mif2Go that hard-codes fonts
##! or styles that cannot be controlled by a CSS. 
##! One of the lists that can be defined in the master file is a "zap"
##! list. This is read into @globe::voy_html_zap hash. Each item in the zap list has
##! three elements:<ol>
##! <li>start of the offending tag but incomplete.</li>
##! <li>end of first offending tag.</li>
##! <li>end of tag pairs.</li></ol>
##! The HTML tags defined in the zap list get taken out of the HTML document.</li>
##! </ul>
##!
##! This also extracts from the master file the order that the directories are 
##! to be processed in. This establishes a hash table with the order of the
##! sections in the HTML system, their associated PDF file names, their
##! associated document numbers.
##! 
##! @bug None.
##! @ingroup tp_nav
##############################################################################
sub get_master_nav_info {
   undef(@pre_chunks);
   undef(@post_chunks);
   $value_2_return = 0;
   
   ## $globe::master_nav has to be available.
   if (0){
      print "=== master nav ====\n$globe::master_nav\n";
      print "=== Definitions 4\n";
      print "=== header ==== $globe::m_define{header}[0]\n";
   }


   foreach $def_type (keys %{globe::m_define}) {
      if (0) {
         print "$def_type... $globe::m_define{$def_type}[0] and $globe::m_define{$def_type}[1]\n";
      }
      # Non-critical items that we don't need to record error messages
      $very_critical = 0;
      if ((0) && ($def_type =~ /order/)) {
         # Critical items that we want to record error messages
         # Needed later in voyant_nav.pl, because we want the doc-numbers.
         $very_critical = 1;
      }
      ($before, $piece, $after) = &globe::get_tag_chunk( $globe::master_nav,
                                  $globe::m_define{$def_type}[0], 
                                  $globe::m_define{$def_type}[1], 
                                  $very_critical);
      if ($piece) {
         if (($def_type =~ /order/) || ($def_type =~ /html_zap/)) {
            # Special case does not need to have tags back.
            $globe::m_info{$def_type} .= "$piece";
         } else {
            # Normal cases need to have tags placed back in.
            $globe::m_info{$def_type} = "$globe::m_define{$def_type}[0]";
            $globe::m_info{$def_type} .= "$piece";
            $globe::m_info{$def_type} .= "$globe::m_define{$def_type}[1]";
         }
      } elsif ($very_critical) {
         $value_2_return++;
      }
   }

   ##################
   # Handling the HTML zap tags, things we don't want.
   ##################
   if (exists ($globe::m_info{html_zap})) {
      @{$globe::m_info{_array_html_zap}} = split (/\n/, $globe::m_info{html_zap});
      if (0){
         print "HTML zapping ===\n";
         # print "$piece\n";
         for ($i=0; $i <= $#{$globe::m_info{_array_html_zap}}; $i++){
            print "$i $globe::m_info{_array_html_zap}[$i]\n";
         }
      }
   }
   ##################
   # Handling the ordering of the subdirectories in the structure
   ##################
   if (exists ($globe::m_info{order})) {
      @pre_chunks = split ( /\n/, $globe::m_info{order});
      for ($i=0; $i <= $#pre_chunks; $i++){
         if ($pre_chunks[$i] =~ /[\w]+/) {
            @order_chunk = split ( /\,/, $pre_chunks[$i], 4);
            for ($j = 0; $j <= $#order_chunk; $j++){
               # strip out quotation marks
               $order_chunk[$j] =~ s/[\s]*\"[\s]*//g;
            }
            #####
            # New structure
            #####
            $globe::m_info{_array_order}[$i]{dir} = $order_chunk[0];
            $globe::m_info{_array_order}[$i]{title} = $order_chunk[1];
            $globe::m_info{_array_order}[$i]{pdf} = $order_chunk[2];
            $globe::m_info{_array_order}[$i]{num} = $order_chunk[3];
         }
      }  # for $i
      if (0){
         print "Order ===\n";
         # print "$piece\n";
         for ($i=0; $i <= $#{$globe::m_info{_array_order}}; $i++){
            foreach $k (keys %{$globe::m_info{_array_order}[$i]} ) {
               print "$i $k $globe::m_info{_array_order}[$i]{$k}\n";
            } # each $k key
         } # each $i
      } # debug
   }

   
   #####
   # Support for the old
   #####
   if (0) {
      if (exists ($globe::m_info{header})) {
         $globe::voy_header = $globe::m_info{header};
      }
      if (exists ($globe::m_info{nav_common})) {
         $globe::voy_common_top = $globe::m_info{nav_common};
      }
      if (exists ($globe::m_info{nav_doxygen})) {
         $globe::voy_dox = $globe::m_info{nav_doxygen};
      }
      if (exists ($globe::m_info{nav_book})) {
         $globe::voy_fm_book = $globe::m_info{nav_book};
      }
      if (exists ($globe::m_info{title})) {
         $globe::m_default_title = $globe::m_info{title};
      }
   }
   if (1) {
      if (exists ($globe::m_info{footer})) {
         $globe::voy_footer = $globe::m_info{footer};
      }
      if (exists ($globe::m_info{html_zap})) {
         @globe::voy_html_zap = @{$globe::m_info{_array_html_zap}}
      } else {
         push (@globe::voy_html_zap, "_none_");
      }
      if (exists ($globe::m_info{order})) {
         for ($i=0; $i <= $#{$globe::m_info{_array_order}}; $i++){
            $globe::master_order[$i] = $globe::m_info{_array_order}[$i]{dir};
            $globe::master_order_title[$i] = $globe::m_info{_array_order}[$i]{title};
            $globe::master_order_pdf[$i] = $globe::m_info{_array_order}[$i]{pdf};
            $globe::master_order_doc_num[$i] = $globe::m_info{_array_order}[$i]{num};
         } # each $i
      }

   }

   # Successful Return
   return($value_2_return);
   
} # get_master_nav_info

##############################################################################
##! @fn int get_tag_chunk
##! @brief Gets the information between two tags.
##!
##! @param input_chunk Text to parse for tags.
##! @param start_flag starting tag to look for.
##! @param end_flag ending flag to look for.
##! @param critical specifies whether or not to record error messages.
##!
##! @retval before the information before the starting tag, 0 if one of tags is missing.
##! @retval info the information between the tags,  0 if one of tags is missing.
##! @retval after the information after the ending tag,  0 if one of tags is missing.
##! 
##! @bug Does not return the tags.
##! @ingroup tp_toc
##############################################################################
sub get_tag_chunk {
   $input_chunk = $_[0];
   $start_flag = $_[1];
   $end_flag = $_[2];
   $critical = $_[3];
   $case_insensitive = $_[4];
   
   if (0) {
      print "get_tag_chunk...\n";
      print "start = \"$start_flag\"\n";
      print "end   = \"$end_flag\"\n";
   }

   if ($case_insensitive) {
      # Case insensitive
      if ($input_chunk !~ /$start_flag/i) {
         if ($critical) {
            print "Input does not contain $start_flag\n";
            push (@globe::file_errors, "ERROR: Missing $start_flag.");
         }
         return (0, 0, 0);
      }   
      if ($input_chunk !~ /$end_flag/i) {
         if ($critical) {
            print "Input does not contain $end_flag\n";
            push (@globe::file_errors, "ERROR: Missing $end_flag.");
         }
         return (0, 0, 0);
      }
      @sm_chunk1 = split (/$start_flag/i, $input_chunk, 2);
      @sm_chunk2 = split (/$end_flag/i, $sm_chunk1[1], 2);
   } else {
      # Case sensitive
      if ($input_chunk !~ /$start_flag/) {
         if ($critical) {
            print "Input does not contain $start_flag\n";
            push (@globe::file_errors, "ERROR: Missing $start_flag.");
         }
         return (0, 0, 0);
      }   
      if ($input_chunk !~ /$end_flag/) {
         if ($critical) {
            print "Input does not contain $end_flag\n";
            push (@globe::file_errors, "ERROR: Missing $end_flag.");
         }
         return (0, 0, 0);
      }
      @sm_chunk1 = split (/$start_flag/, $input_chunk, 2);
      @sm_chunk2 = split (/$end_flag/, $sm_chunk1[1], 2);
   }
   
   undef ($input_chunk);
   undef ($start_flag);
   undef ($end_flag);
   # undef (@sm_chunk1);
   return ($sm_chunk1[0], $sm_chunk2[0], $sm_chunk2[1]);   

   
} # get_tag_chunk


##############################################################################
##! @fn int replace_tag_chunk
##! @brief Replaces the information in between the tags with the supplied
##! information.
##! 
##! @param input_chunk Text to parse for tags.
##! @param start_flag starting tag to look for.
##! @param end_flag ending flag to look for.
##! @param replacement_text Text to put between tags.
##! @param reinsert_tags specifies whether or not tags should be reinserted
##! @param critical specifies whether or not to record error messages.
##!
##! @retval success 1 if found tags, 0 if one of tags is missing.
##! @retval output with replacement text and tags if found tags, 
##! the original input text if one of tags is missing.
##! 
##! @bug None.
##! @ingroup tp_toc
##############################################################################
sub replace_tag_chunk {
   $input_chunk = $_[0];
   $start_flag = $_[1];
   $end_flag = $_[2];
   $replacement_text = $_[3];
   $reinsert_tags = $_[4];
   $critical = $_[5];
   $output = "";

   if (0) {
      print "replace_tag_chunk...\n";
   }



   if ($input_chunk !~ /$start_flag/) {
      if ($critical) {
         print "Input does not contain $start_flag\n";
         push (@globe::file_errors, "ERROR: Master file does not have proper $where definition.");
         push (@globe::file_errors, "Missing $start_flag.");
      }
      return (0, $input_chunk);
   }   
   if ($input_chunk !~ /$end_flag/) {
      if ($critical) {
         print "Input does not contain $end_flag\n";
         push (@globe::file_errors, "ERROR: Master file does not have proper $where definition.");
         push (@globe::file_errors, "Missing $end_flag.");
      }
      if (0) {
         print "replace_tag_chunk... 0\n";
      }

      return (0, $input_chunk);
   }
   @sm_chunk1 = split (/$start_flag/, $input_chunk, 2);
   @sm_chunk2 = split (/$end_flag/, $sm_chunk1[1], 2);
   
   if ($reinsert_tags) {
      $output = $sm_chunk1[0];
      $output .= $start_flag;
      $output .= $replacement_text;
      $output .= $end_flag;
      $output .= $sm_chunk2[1];
   } else {
      $output = $sm_chunk1[0];
      $output .= $replacement_text;
      $output .= $sm_chunk2[1];
   }
   
   undef (@sm_chunk1);
   undef (@sm_chunk2);
   if (0) {
      print "replace_tag_chunk... 1\n";
   }
   
   return (1, $output);   

   
} # replace_tag_chunk



##############################################################################
##! @brief Reads in the entire file but separates code from comments in two
##! different structures.
##! 
##! @param src_file The file to open and read in.
##! @param separate_comments 0 puts it all into $entire_file; 1 puts all
##! of the comments into $comment_file and everything else into
##! $entire_file.
##! 
##! @bug 
##############################################################################
sub read_code_n_comment {
   local ($src_file) = $_[0];
   local ($separate_comments) = $_[1];
   $ret_val = 1;
   local ($rcnc_debug) = 0;
   
   if ($rcnc_debug) {
      print "rcnc:src_file $src_file\n";
      print "rcnc:separate_comments $separate_comments\n";
   }

   undef ($globe::entire_file);
   undef ($globe::comment_file);
   unless (open ( IN_SRC, $src_file)) {
      print "1 Cannot open file \"$src_file\"\n";
      push (@file_errors, "1 Cannot open file \"$src_file\"\n");
      $ret_val = 0;
      return (0);
   }
   if ($separate_comments) {
      while (<IN_SRC>) {
         # The only time we need to separate comments from
         # text is when the input file is the message text file
         # and we are picking up areas that have tags and
         # areas that are the log lines.
         if (($_ =~ /^[\s]*\/\//)
            || ($_ =~ /^[\s]*\/\*/)
            || ($_ =~ /^[\s]*\*\*/)
            || ($_ =~ /^[\s]*\*/)
            ){
            # Only read in comment lines.
            if ((0) && ($rcnc_debug)) {
               print "rcnc:Comment... $_";  # no \n needed
            }
            $globe::comment_file .= $_;
         } elsif ($_ =~ /^[\s]+$/) {
            # Not interested in blank lines at all.
         } else {
            # Only read in non-comment lines.
            $globe::entire_file .= $_;
         }
      }
   } else {
      while (<IN_SRC>) {
         # XML files or other text files don't need to
         # separate code comments out from regular text.
         # For that matter, code files don't really need to
         # separate things.
         $globe::comment_file .= $_;
      }
   }
   close (IN_SRC);
   if ($rcnc_debug) {
      print "=====\nrcnc:entire_file\n=====\n$globe::entire_file";
      print "=====\nrcnc:comment_file\n=====\n $globe::comment_file";
      # exit;
   }
   return ($ret_val);
} # read_code_n_comment


##############################################################################
##! @brief Processes the comment lines from the file that may have tags 
##! in them.
##!
##! @param src_file is the file associated with the comments.
##!
##! @param tag_name defines the tag identifier to look for.
##!
##! This calls another routine that puts $tag_name name tags into a data structure.
##! When this routine is done, we no longer need globe::comment_file.
##! 
##! @bug Does not test the tag_name is valid or has been defined.
##############################################################################
sub process_comments_from_file {
   local ($src_file) = $_[0];
   local ($tag_name) = $_[1];
   local ($debug_pcff) = 0;

   if ((0) && ($debug_pcff)) {
      # print "pcff:entire_file $globe::entire_file\n";
      print "pcff:These are the comments:\n";
      print "pcff:comment_file $globe::comment_file\n";
      print "pcff:========== $tag_name \n";
      for ($i=0; $i <= $#{$globe::x_define{$tag_name}}; $i++) {
         print "$i: $globe::x_define{$tag_name}[$i]\n"
      }
      print "pcff:shrink $shrinking_chunk\n";
      # exit;
   }

   
   if ($globe::comment_file =~ /$globe::x_define{$tag_name}[0]/) {
      $shrinking_chunk = $globe::comment_file;
      if ($debug_pcff) {
         print "pcff:shrink $shrinking_chunk\n";
         print "pcff:x_define $globe::x_define{$tag_name}[0]\n";
      }
      while ($shrinking_chunk =~ /$globe::x_define{$tag_name}[0]/) {
         $shrinking_chunk = &globe::piece_btn_tag($shrinking_chunk, 
                            "name", $src_file, $tag_name);
      }
      if ($debug_pcff) {
         print "pcff:tag_mess1...\n";
         foreach $f (sort keys %{$globe::tag_mess{file}}) {
            foreach $t (sort keys %{$globe::tag_mess{file}{$f}}){
               print "pcff:file\"$f\":tag \"$t\":\n$globe::tag_mess{file}{$f}{$t}\n";
            }
         }
         print "pcff:tag_mess2...\n";
         foreach $t (sort keys %{$globe::tag_mess{name}}) {
            foreach $i (@{$globe::tag_mess{name}{$t}}){
               print "pcff:name \"$t\":file $globe::tag_mess{name}{$t}[$i]\n";
            }
         }
         print "pcff:tag_mess3...\n";
      }
   } else {
      ####
      # 08/07/2002 We may not have found any tags, but we still processed the
      # file; mark it as such
      ####
      $globe::tag_mess{file}{$src_file} = $src_file;
      print "pcff: Not an error; found no \"$globe::x_define{$tag_name}[0] ";
      print "$globe::x_define{$tag_name}[1]\"  named targets in comments ";
      print "from \"$src_file\".\n";
      if ($debug_pcff) {
         print "pcff:tag_mess4...\n";
         foreach $f (sort keys %{$globe::tag_mess{file}}) {
            foreach $t (sort keys %{$globe::tag_mess{file}{$f}}){
               print "pcff:file\"$f\":tag \"$t\":\n$globe::tag_mess{file}{$f}{$t}\n";
            }
         }
      }
      return (0);
   }
   return (1);
   
} # process_comments_from_file





##############################################################################
##! @brief Creates a data structure from each line in the message file.
##! This builds a data structure:
##! 
##!  $globe::tag_mess{file}{$file_name}{$name} = $data;
##!  $globe::tag_mess{name}{$name}[$i] = $file_name;
##! 
##!       $hname = created name for hash made from actual names of parents and
##!                children 
##!       $globe::xtag_struct{$hname}{name} = actual item name
##!       $globe::xtag_struct{$hname}{parent} = owning section
##!       $globe::xtag_struct{$hname}{child}[] = array of associated children
##!       $globe::xtag_struct{$hname}{comment} = comment flag
##!       $globe::xtag_struct{$hname}{desc_html} = description in html
##!       $globe::xtag_struct{$hname}{desc_man} = description in man
##! 
##!       $name = name associated with number
##!       $msg_struct{$name}{severity} = severity associated with number
##!       $msg_struct{$name}{message} = message associated with number
##!       $msg_struct{$name}{comment} = comment associated with number
##! 
##! 
##! @bug 
##############################################################################
sub xtag_into_mess_structure {
   local ($src_file) = $_[0];
   local ($tag_xml) = $_[1];
   local ($debug_xims) = 0;
   

   foreach $name (sort keys %{xtag_struct}){
      if ($debug_xims) {
         print "xims:org ===\n$globe::xtag_struct{$name}{comment}\n";
      }
      $changing_chunk = $globe::xtag_struct{$name}{comment};
      while ($changing_chunk =~ /target\=/i) {
         $changing_chunk = &globe::xml_tag_target ($changing_chunk, $src_file, $tag_xml);
      }
      if ($debug_xims) {
         print "xims:new ===\n$changing_chunk\n++++++++++++++\n";
      }
      $globe::xtag_struct{$name}{comment_full} = $changing_chunk;
   }

   return (1);
} # xtag_into_mess_structure




##############################################################################
##! @brief Gets the data associated with a target in the chunk
##! and passes the chunk back.
##!
##! @param chunk_to_parse A chunk with a target inside of it.
##! @param src_file_name The file associated with the target.
##!
##! @return The chunk with the target expanded into what it is after
##! hunting it down. The $tag_name tag with the target in it is removed
##! whether or not any associated data could be hunted down.
##!
##! This calls the validate_and_fetch routine, which in turn
##! calls the globe::read_code_n_comment and process_..._file that were called
##! earlier at a higher level. They are reused in hunting down targets
##! and the targets' data.
##!
##! @bug This does no error checking if the the chunk has the 
##! the target in it or if the $tag_name (xml identifier) is defined
##! in the globe.pm file under $globe::m_define{$tag_name}.
##! 
##############################################################################
sub xml_tag_target {
   local ($chunk_to_parse) = $_[0];
   local ($src_file_name) = $_[1];
   local ($tag_name) = $_[2];
   local ($name) = 0;
   local ($targ_f_name) = 0;
   local ($data) = 0;
   local ($before) = "";
   local ($after) = "";
   local ($debug_xtt) = 0;
   
   if ($debug_xtt) {
      print "x_t_t:chunk_to_parse \"$chunk_to_parse\"\n";
      print "x_t_t:src_file_name \"$src_file_name\"\n";
      print "x_t_t:tag_name \"$tag_name\"\n";
   }
   
   ($before, $lm_label, $after) = &globe::get_tag_chunk( $chunk_to_parse,
              $globe::x_define{$tag_name}[0], 
              $globe::x_define{$tag_name}[1], 
              0);
   #####
   # Get the xml target to look for
   #####
   local ($tag_param) = "target";
   if ((0) && ($debug_xtt)) {
      print "x_t_t:tag_param \"$tag_param\"\n";
      print "x_t_t:lm_label \"$lm_label\"\n";
   }
   if ($lm_label =~ /$tag_param/i) {
      #####
      # Get the name of the $tag_name tag for the database
      #####
      @name_chunk = split (/$tag_param[\s]*\=/, $lm_label, 2);
      ($g1, $target, $g2) = &globe::get_tag_chunk( $name_chunk[1], "\"", "\"", 0);
      if ($debug_xtt) {
         print "we got a target: \"$target\"\n";
         # exit;
      }
   } else {
      # There is no target, but let's rebuild without $tag_name tag and return
      $chunk_to_parse = join ("", $before, $after);
      return ($chunk_to_parse, 0, $src_file_name);
   }
   
   #####
   # Get the file to look for
   #####
   $tag_param = "file";
   if ($lm_label =~ /$tag_param[\s]*\=/i) {
      # updated conditional to not catch any "file" that might be part
      # of an item name.
      #####
      # Get the name of the $tag_name tag for the database
      #####
      @name_chunk = split (/$tag_param[\s]*\=/, $lm_label, 2);
      ($g1, $targ_f_name, $g2) = &globe::get_tag_chunk( $name_chunk[1], "\"", "\"", 0);
      if ((0) && ($debug_xtt)) {
         print "x_t_t:label chunk: \"$lm_label\"\n";
         print "x_t_t:we got a file: \"$targ_f_name\"\n";
         # exit;
      }
      @in_file = split (/\//, $src_file_name);
      pop (@in_file); # removes the old file name from the array
      if ($targ_f_name =~ /^\//) {
         # if $targ_f_name begins with a slash, then we have an absolute path;
         # use this name as is.
         $src_file_name = $targ_f_name;
      } elsif ($targ_f_name =~ /^\.\//) {
         # The file is located in the same directory.
         # strip off file from input file name;
         # strip of ./ from this target file.
         # combine the two.
         $targ_f_name =~ s/^\.\///;
         $src_file_name = join ("/", @in_file, $targ_f_name);
      } elsif ($targ_f_name =~ /^\.\.\//) {
         # The file is located a relative directory.
         # strip off file from input file name;
         # for every ../ from this target file, pop off something.
         # combine what's left.
         while ($targ_f_name =~ /^\.\.\//) {
            $targ_f_name =~ s/^\.\.\///;
            pop (@in_file);
         }
         $src_file_name = join ("/", @in_file, $targ_f_name);
      } else {
         # The file is okay to append as is.
         $src_file_name = join ("/", @in_file, $targ_f_name);
      }
      if ($debug_xtt) {
         print "x_t_t:src_file_name $src_file_name\n";
      }
   } else {
      # There is no target to catch
      if ($debug_xtt) {
         print "x_t_t:src_file_name $src_file_name\n";
      }
   }  # $tag_param = "file";
   
   #####
   # Now it is time to see if that file is valid and to get
   # information out of it.
   #####
   $data = &globe::validate_and_fetch ($src_file_name, $tag_name, $target);
   if ($debug_xtt) {
      print "x_t_t:target=$target; $data \n+++++++\n";
   }
   if ($data) {
      #####
      # If validate_and_fetch returned something other than
      # 0, append this to the $before stuff. This effectively
      # will replace the $tag_name tag with the data
      # when the comment gets rebuilt.
      #####
      # 08/02/2002 cludge
      # return ($data);
      # 08/02/2002 was
      $before .= $data;
   }
   if ((0) && ($debug_xtt)) {
      print "x_t_t:target=$target; file=$src_file_name\n";
   }
   if ((0) && ($debug_xtt)) {
      print "x_t_t:before $before\n";
      print "x_t_t:data $data\n";
      print "x_t_t:after $after\n";
   }
   
   
   #####
   # Rebuild the chunk without the information that was parsed out.
   # This means either we've removed just the erroneous beginning
   # $tag_name tag,
   # or the beginning/ending tags and the data in between.
   # Either way, we're removing the conditions that keep this
   # routine from being called.
   #####
   $chunk_to_parse = join ("", $before, $after);

   return ($chunk_to_parse);
} # xml_tag_target

##############################################################################
##! @brief Removes the tags and associated data
##! from the data chunk that is passed in.
##! 
##! @param chunk_to_parse A piece of text that has at least one beginning
##! $tag_name tag.
##! @param file_name The name of the file to which the $tag_name tag is associated.
##!
##! This builds a data structure:
##! 
##!  $globe::tag_mess{file}{$file_name}{$name} = $data;
##!  $globe::tag_mess{name}{$name}[$i] = $file_name;
##! 
##! It allows us to be efficient in the future, whereby we won't have to
##! open a file that we've already visited, because we'll already have acquired
##! all associated $tag_name item for that file. Also, we'll know later if
##! a $tag_name name has been defined in more than one file.
##! 
##! @bug Error checking for the existence of a file name in the structure
##! needs to happen before this routine is called. 
##############################################################################
sub piece_btn_tag {
   local ($chunk_to_parse) = $_[0];
   local ($tag_param) = $_[1];
   local ($file_name) = $_[2];
   local ($tag_name) = $_[3];
   local ($before) = "";
   local ($after) = "";
   local ($lm_label) = "";
   local ($data) = "";
   local ($debug_pbt) = 0;

   if ($debug_pbt) {
      print "pbt:chunk_to_parse $chunk_to_parse\n";
      print "pbt:tag_param $tag_param\n";
      print "pbt:file_name $file_name\n";
      print "pbt:tag_name $tag_name\n";
      print "pbt:globe::x_define{$tag_name}[0] $globe::x_define{$tag_name}[0]\n";
      print "pbt:globe::x_define{$tag_name}[1] $globe::x_define{$tag_name}[1]\n";
   }
   ($before, $lm_label, $after) = &globe::get_tag_chunk( $chunk_to_parse,
              $globe::x_define{$tag_name}[0], 
              $globe::x_define{$tag_name}[1], 
              0);
   if ($debug_pbt) {
      print "pbt:lm_label $lm_label\n";
   }
   #####
   # Get the name target for the given tag
   #####
   if ($lm_label =~ /$tag_param/i) {
      #####
      # Get the name of the $tag_name tag for the database
      #####
      # @name_chunk = split (/name[\s]*\=/, $lm_label, 2);
      @name_chunk = split (/$tag_param[\s]*\=/, $lm_label, 2);
      ($g1, $name, $g2) = &globe::get_tag_chunk( $name_chunk[1], "\"", "\"", 0);
      if ($debug_pbt) {
         print "we got a name: \"$name\"\n";
         # exit;
      }
      #####
      # Look for the ending $tag_name tag
      #####
      if ($after =~ /$globe::x_define{$tag_name}[2]/) {
         #####
         # Everything up to this chunk is of interest to us.
         # At this point we know there's an opening and closing
         # $tag_name tag with information inbetween
         #####
         @lm_data = split (/$globe::x_define{$tag_name}[2]/, $after, 2);
         @lm_data2 = split (/\n/, $lm_data[0]);
         for ($j=0; $j <= $#lm_data2; $j++) {
            for ($k=0; $k <= $#{$globe::x_define{comment}}; $k++){
               if ($debug_pbt) {
                  print "comment type: $globe::x_define{comment}[$k]\n";
               }
               if ($lm_data2[$j] =~ /[\s]*$globe::x_define{comment}[$k]/) {
                  $lm_data2[$j] =~ s/[\s]*$globe::x_define{comment}[$k]//;
               }
            }
            #####
            # Special case comment lines that are C style with *'s.
            #####
            if ($lm_data2[$j] =~ /^[\s]*\/\*\*/) {
               $lm_data2[$j] =~ s/^[\s]*\/\*\*//;
            } elsif ($lm_data2[$j] =~ /^[\s]*\/\*/) {
               $lm_data2[$j] =~ s/^[\s]*\/\*//;
            } elsif ($lm_data2[$j] =~ /^[\s]*\*\*/) {
               $lm_data2[$j] =~ s/^[\s]*\*\*//;
            } elsif ($lm_data2[$j] =~ /^[\s]*\*/) {
               $lm_data2[$j] =~ s/^[\s]*\*//;
            }
            $lm_data2[$j] =~ s/^[\s]*//;
         }
         $data = join ("\n", @lm_data2);
         if ($debug_pbt) {
            print "we got stripped data====\n$data\n=====\n";
         }

         #####
         # Put the name that we found, the file name, and
         # the associated data into the data structure.
         #####
         if (exists ($globe::tag_mess{file}{$file_name}{$name})) {
            # if it exists, append it so that we get everything.
            $globe::tag_mess{file}{$file_name}{$name} .= $data;
         } else {
            $globe::tag_mess{file}{$file_name}{$name} = $data;
            if ((0) || ($debug_pbt)) {
               print "pbt:new file \"$file_name\" or new tag \"$name\"\n";
            }
         }
         push (@{$globe::tag_mess{name}{$name}}, $file_name);

         #####
         # We partial rebuild and remove data.
         # A full rebuild happens as the default fall through.
         #####
         $after = $lm_data[1];
      } else {
         # This is not well formed and there is no ending $tag_name tag.
         # remove the $tag_name, rebuild, and return.
         # This is the default fall through
      }
   } elsif ($lm_label =~ /[\w]+/) {
     # The name is not composed of white space, but also isn't completely
     # what we want; it might be a $tag_name target;
     # remove the tag, though, so loop will stop.
     # This is the default fall through.
   } else {
      # name is composed of white space
      # Remove the $tag_name tag, rebuild and return.
      # If this has a problem, it might be leaving a closing
      # $tag_name tag.
      if ($debug_pbt) {
        print "Hola 2\n";
      }
   }
   #####
   # Rebuild the chunk without the information that was parsed out.
   # This means either we've removed just the erroneous beginning
   # $tag_name tag,
   # or the beginning/ending tags and the data in between.
   # Either way, we're removing the conditions that keep this
   # routine from being called.
   #####
   $chunk_to_parse = join ("", $before, $after);
   
   # 08/07/2002 Put an entry into the hash that says we processed
   # the file.
   if (exists ($globe::tag_mess{file}{$file_name})) {
      # We're okay, no need to do anything.
   } else {
      # The file was processed, but no entries found.
      # Put file into hash.
      # $globe::tag_mess{file}{$file_name}{$name}
      $globe::tag_mess{file}{$file_name} = $file_name;
   }

   return ($chunk_to_parse);
} # piece_btn_tag

##############################################################################
##! @brief Determines if we have already processed the file for the tag and
##! does the processing.
##! 
##! @param f_name The path and name of the file.
##!
##! @param tag_xml The name of the xml tag to look for in the file.
##!  
##! @param targ_xml_tag The name of the target within the xml tag to look for.
##! 
##! If the file and tag have not been processes, this performs the processing
##! on the file for that specific tag.
##!
##! The processing involves adding information to the data structure.
##! 
##!  $globe::tag_mess{file}{$file_name}{$name} = $data;
##!  $globe::tag_mess{name}{$name}[$i] = $file_name;
##! 
##! @bug This overwrites globe::comment_file and globe::entire_file because
##! it calls globe::read_code_n_comment. The information in these global
##! variables is then immediately processed.
##!
##############################################################################
sub validate_and_fetch {
   local ($f_name) = $_[0];
   local ($tag_xml) = $_[1];
   local ($targ_xml_tag) = $_[2];
   local ($debug_v_n_f) = 0;
   local ($temp_return) = "";
   
   if ((0) && ($debug_v_n_f)) {
      print "v_n_f:f_name $f_name\n";
      print "v_n_f:tag_xml $tag_xml\n";
      print "v_n_f:targ_xml_tag $targ_xml_tag\n";
   }
   if ($debug_v_n_f) {
      print "v_n_f:targ_xml_tag $targ_xml_tag\n";
      print "v_n_f:return $globe::tag_mess{file}{$f_name}{$targ_xml_tag}\n";
   }
   
   if (exists ($globe::tag_mess{file}{$f_name})) {
      #####
      # Then we have already parsed the file for information.
      # 08/07/2002 However, the tags should be cummulative over many
      # files. No longer such a thing as the same tag being unique
      # just because it is in its own file. If the file's been
      # parsed, the tag gets put into the pool.
      #####
      if (exists ($globe::tag_mess{name}{$targ_xml_tag})) {
         for ($j=0; $j <= $#{$globe::tag_mess{name}{$targ_xml_tag}}; $j++) {
            $f_name = $globe::tag_mess{name}{$targ_xml_tag}[$j];
            $temp_return .= $globe::tag_mess{file}{$f_name}{$targ_xml_tag};
         }
         return ($temp_return);
      } else {
         # File has been processed previously, but no tags exist.
         # Hence, we should return with a zero.
         if ($debug_v_n_f) {
            print "v_n_f:targ_xml_tag $targ_xml_tag\n";
            print "v_n_f:2a Did not find tag\n";
         }
         return (0);
      }
   } else {
      #####
      # Then we have to see if that file is even valid and information
      # out of it and into our data structure.
      #####
      if ($debug_v_n_f) {
         print "v_n_f and pre-rcnc: $f_name\n";
      }

      if ((1) && (!(&globe::read_code_n_comment($f_name, 0)))){
         # The 0 passed in means that we don't care what's in the file;
         # we're going to find our tags no matter what.
         # If this returns 0 and fails, it means that the file could not be
         # found or opened.
         # Hence, we should return with a zero.
         if ($debug_v_n_f) {
            print "v_n_f:targ_xml_tag $targ_xml_tag\n";
            print "v_n_f:return $globe::tag_mess{file}{$f_name}{$targ_xml_tag}\n";
            print "v_n_f:2 Did not find tag\n";
         }
         return (0);
      }   
      # At this point, $globe::comment_file was created and populated,
      # which is what we care about.
      # $globe::entire_file was also created, but we don't care about it.

      if ((1) && (!(&globe::process_comments_from_file($f_name, $tag_xml)))){
         print "2 Processing comments from \"$f_name\" failed some things.\n";
      } 
      #####
      # At this point, $globe::comment_file has been parsed and
      # can be overwritten.
      # Also at this point, we have more things in the data structure.
      # Supposedly, our target and data are there.
      #####

      if (exists ($globe::tag_mess{file}{$f_name})) {
         # Then we have already parsed the file for information.
         if (exists ($globe::tag_mess{file}{$f_name}{$targ_xml_tag})) {
            # Then the tag name is valid and we can return what we
            # have already stored. We need to return all occurrences
            # in the data base, not just the one for this file.
            for ($j=0; $j <= $#{$globe::tag_mess{name}{$targ_xml_tag}}; $j++) {
               $f_name = $globe::tag_mess{name}{$targ_xml_tag}[$j];
               $temp_return .= $globe::tag_mess{file}{$f_name}{$targ_xml_tag};
            }
            return ($temp_return);
         } else {
            # Then we did not find the desired tag name when we
            # parsed the file. Return a 0.
            if ($debug_v_n_f) {
               print "v_n_f:targ_xml_tag $targ_xml_tag\n";
               print "v_n_f:3Did not find tag\n";
            }
            return (0);
         }
      } else {
         # Maybe the file had no entries to merit an
         # entry into our data structure. At this point, we
         # tried our best to get the data and had none.
         return (0);
      }
   }
} # validate_and_fetch




##############################################################################
##! @fn int do_errors  ( )
##! @brief Prints out all accumulated error messages.
##! @param None
##! @return None
##! 
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub do_errors {
   if (@globe::file_errors) {
      print "\n============  Summary of errors =============================globe\n";
      for ($i=0; $i<@globe::file_errors; $i++){
         print "$i = $globe::file_errors[$i]\n";
      }
   }
}



##############################################################################
##! @fn int END
##! @brief Code to run at the end of this routine.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_tools
##############################################################################
sub END {
   if (0) {
      print "globe.pm END routine.\n";
   }
}

