#!/usr/local/bin/perl
package xscope;


##############################################################################
##! @file
##! @brief Defines global variables that can be used by other programs.
##! 
##! Most of the variables refer to tags that we expect to find in the 
##! HTML files. Others define variables that we want to use in a global way.
##!
##! @ingroup tp_tools tp_xhelp
##!
##! @author Glenn C. Maxey
##
##    $Id: html_look_integrate.pm,v 1.2 2002/04/10 00:35:28 gmaxe Exp $
##
## Copyright 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Revision Information:
##
## $Log: html_look_integrate.pm,v $
## Revision 1.2  2002/04/10 00:35:28  gmaxe
## Tracing tool now works;
## indexer ignores more special characters;
## pm file is the first version; contains the proper data structures
## to handle bsp/vxworks
##
## Revision 1.1  2002/04/06 01:55:12  gmaxe
## New files and new general constructs for handling tags. Uses globals better.
## The html_look* files are designed to handle spider tracing of html systems.
##
##
##############################################################################


##############################################################################
##! @fn int BEGIN
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub BEGIN {
   if (0) {
      print "xhelp_all.pm BEGIN routine.\n";
   }
} # BEGIN

##############################################################################
##! @fn int main
##! @brief Main routine.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
# sub main {
{
   ######
   ## Main Subroutine doesn't do anything
   ######
   if (0) {
      print "xhelp_all.pm main routine.\n";
   }
  
   # Needed in order for the package to work correctly from other perl programs.
   return(1);
}

##############################################################################
##! @fn int declare_variables
##! @brief Allocates memory for all globally used variables.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub declare_variables {

   ##############################################################################
   ## Declarations: all variables are global.
   ##############################################################################

   # Assume that the starting point is the first element of the top files.
   @top_files = ("/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/BSP_Developers_Kit.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/BSP_Reference.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/gnu/gdb/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/gnu/make/make_toc.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/gnu/toolkit/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/pcmcia/relsupp/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado/api/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/GUI_Tcl_Library__UNIX_.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/GUI_Tcl_Library__WINODWS_.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Project_Database_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Project_Scripting_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Target_Server_Back_End_Interface.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Target_Server_Internal_Routines.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WDB_Protocol.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_C_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_Prototcol.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_Tcl_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/Tornado_Reference.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado/unixguide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/trueffs/guide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/VxWorks_Reference_Manual.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/snmp/supplement/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/streams/supplement/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/windview/guide/index.html"
   );  # @top_files
   @top_files = ("/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/BSP_Developers_Kit.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/BSP_Reference.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/gnu/gdb/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/gnu/make/make_toc.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/gnu/toolkit/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado/api/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/GUI_Tcl_Library__UNIX_.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/GUI_Tcl_Library__WINDOWS_.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Project_Database_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Project_Scripting_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Target_Server_Back_End_Interface.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Target_Server_Internal_Routines.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WDB_Protocol.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_C_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_Protocol.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_Tcl_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/Tornado_Reference.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado/unixguide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/trueffs/guide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/VxWorks_Reference_Manual.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/snmp/supplement/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/streams/supplement/index.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/windview/guide/index.html"
   );  # @top_files

   $globe::path = "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/";
   $globe::nav_purge_path{local} = "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/";
   $globe::nav_purge_path{master} = "/rtfm/techpubs/cms_drivers/doc_publish/";
   
   $globe::top_most_level{title} = "Bridge Support Package (BSP)/VxWorks";
   $globe::top_most_level{url} = "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/books.html";

   @ex_as_child = ("/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/books.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/BSP_Developers_Kit.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/BSP_Reference.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/BSP_VTS.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/Tornado_API_Reference.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/GUI_Tcl_Library__UNIX_.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/GUI_Tcl_Library__WINDOWS_.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Project_Database_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Project_Scripting_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Target_Server_Back_End_Interface.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/Target_Server_Internal_Routines.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WDB_Protocol.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_C_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_Protocol.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/WTX_Tcl_Library.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/Tornado_Reference.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/VxWorks_Reference_Manual.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/rtnIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/bspkit/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/gnu/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/pcmcia/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/snmp/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/streams/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado/rtnIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/tornado-api/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/trueffs/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/windview/libIndex.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX1.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX2.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX3.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX4.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX5.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX6.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX7.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX8.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX9.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX10.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX11.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX12.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX13.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX14.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX15.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX16.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX17.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX18.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX19.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX20.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX21.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX22.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX23.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX24.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIX25.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/GuideIXTOC.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX1.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX2.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX3.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX4.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX5.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX6.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX7.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX8.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX9.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX10.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX11.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX12.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX13.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX14.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX15.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX16.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX17.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX18.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX19.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX20.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX21.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIX22.html",
                 "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/netguide/netguideIXTOC.html"
   );  # @ex_as_child
   @gen_ex_child = ("libIndex.html",
                    "rtnIndex.html");
                 
if (0) {
   # When testing, this reduces the list and the number of greps.
   # @x_names = ("bogus");
               
   @top_files = ( "/rtfm/techpubs/cms_drivers/doc_publish/book_vxworks/vxworks/guide/index.html");
   
}
   
   if (0) {
      print "=== Definitions 2 \n";
   }

   return (1);

} # declare_variables

##############################################################################
##! @fn int memory_clean_up
##! @brief Cleans up after all variables that were defined.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub memory_clean_up {
   undef (@exclude_f_type);
                      
   undef (@exclude_file);
   undef (@exclude_dir);
   
   undef (@x_names);
               
   undef (@return_types);
   
   return (1);
}

##############################################################################
##! @fn int END
##! @brief Code to run at the end of this routine.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_tools
##############################################################################
sub END {
   if (0) {
      print "xhelp_all.pm END routine.\n";
   }
}

