#!/usr/local/bin/perl
package scheck;
##############################################################################
##! @file
##! @brief Generates one or more HTML files based upon what is found in
##! the ini file.
##!
##! ../perl/ini_html_gen.pl 
##! @param root_dir something like doc_publish/cref_sysxini/ 
##! @param src_ini source file to parse ./z_include/sysx.ini 
##! @param html_tmpl template for generated files voyant_master_nav.html 
##! @param org_title optional title for the example file.
##! @param a_header optional additional source file with tag information.
##! 
##! 
##! @ingroup tp_tools tp_xhelp
##!
##! @author Glenn C. Maxey
##
##    $Id: ini_html_gen.pl,v 1.9 2002/11/20 15:23:02 gmaxe Exp $
##
## Copyright 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Revision Information:
##
## $Log: ini_html_gen.pl,v $
## Revision 1.9  2002/11/20 15:23:02  gmaxe
## Added exit codes so that wrapper scripts can catch errors properly.
##
## Revision 1.8  2002/11/14 00:36:51  gmaxe
## Added creation of doc and tag files while outputting sample.txt file.
##
## Revision 1.7  2002/10/11 20:20:06  gmaxe
## Minor changes to condition for target testing.
##
## Revision 1.6  2002/08/08 21:37:58  gmaxe
## Further enhancements for debugging and to get the man pages
## generated properly.
##
## Revision 1.3  2002/08/03 00:58:25  gmaxe
## Updates before the weekend. Now it parses target files better;
## had problems with scoping for variables.
##
## Revision 1.3  2002/07/02 23:10:53  gmaxe
## Tweaked to get rid of extra expanding line feeds in voyant_nav.pl.
## Added topper.html and other files to skip list in globe.pm.
## Enhanced ini_html_gen.gl to handle new format of message.txt;
## it now has a flatter data structure and no enums numbers.
##
## Revision 1.2  2002/04/22 17:53:33  gmaxe
## Minor enhancements to sorting order and TOC display.
## Also support an additional argument for a title.
##
## Revision 1.1  2002/04/20 11:31:22  gmaxe
## Added support for a tool that generates HTML for the log
## messages. This is one step away from a tool that does
## XML parsing to generate HTML. Some comments are still
## lacking.
##
## Revision 1.1  2002/04/12 22:52:25  gmaxe
## New tool to check on whether or not all SAPI commands are present.
## Uses the files generated by the vproto_gen.pl tool.
##
##
##############################################################################

##############################################################################
##! @fn int BEGIN
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub BEGIN {
   print "\n============  Starting ini_html_gen.pl ==================================\n";
   $_file_list = "_file_list";
   $in_file = "";
   $root_path = "./";
   $scope_pm = "globe.pm";
   $tag_xml = "initag";            # used globally within this file.
   $globe::struct_start = "__";    # used as the starting key into the xtag_struct.
   $globe::start_name = "INI";     # used as the default name for the starting element
   $globe::pre_class = "Level1IX"; # used to indicate what class to assign the HTML element
   $debug_verbose = 0;
   $copyright = "Created by Voyant Technologies, Inc.";
   
   $no_scope_file = 0;
   $arg_cnt=0;
   $yes_output = 1;  # turns on and off whether html, script, and man pages are output
   $yes_output_man = 1; # turns on and off man pages
   
   use POSIX 'strftime';   # Use a selected function from POSIX library
   $date = strftime "%m/%d/%Y", localtime;
   
   
   push (@INC, `pwd`);
   push (@INC, '../perl');
   if (0){
      print (@INC, "\n");
   }
   

   
   # Get scope files if there is one.
   if (1) { # used
      # $scope_pm = @ARGV[$arg_cnt];
      #####
      # All global variables are defined in the following file
      #####
      
      unless (open ( IN_LIST, $scope_pm)) {
         unless (open ( IN_LIST, "../perl/$scope_pm")) {
            push (@file_errors, "Cannot open file \"$scope_pm\"");
            $no_scope_file++;
         }
      }
      close (IN_LIST);
      push (@INC, $scope_pm);
      
      if (!@file_errors) {
         #####
         # All global variables are defined in the following file
         #####
         require $scope_pm;
   
         if (&globe::declare_variables()) {
            print "Variables initialized from $scope_pm.\n";
         } else {
            push (@file_errors, "Could not initialize variables from $scope_pm.");
         }
      } # if not @file_errors
   } else {
      push (@file_errors, "Need to provide a scope file as first argument.");
   } # if 1 or more arguments

 
 if (0) { # commented out because not used
   # Get scope files if there is one.
   if (@ARGV > $arg_cnt) {
      $scope_pm = @ARGV[$arg_cnt];
      
      unless (open ( IN_LIST, $scope_pm)) {
         push (@file_errors, "Cannot open file \"$scope_pm\"");
         $no_scope_file++;
      }
      close (IN_LIST);
      push (@INC, $scope_pm);
      
      if (!@file_errors) {
         #####
         # All global variables are defined in the following file
         #####
         require $scope_pm;
   
         if (&globe::declare_variables()) {
            print "Variables initialized from $scope_pm.\n";
         } else {
            push (@file_errors, "Could not initialize variables from $scope_pm.");
         }
      } # if not @file_errors
   } else {
      push (@file_errors, "Need to provide a scope file as first argument.");
   } # if 1 or more arguments
   $arg_cnt++;
 } # commented out because not used  
 
 
   if (!(exists ($globe::x_define{$tag_xml}))) {
      push (@file_errors, "\"$tag_xml\" does not exist in the global definitions (\"$scope_pm\").");
   }


   # Get path to code files 
   if (@ARGV > $arg_cnt) {
      $root_path = @ARGV[$arg_cnt];
      $globe::path = @ARGV[$arg_cnt];
      if ($root_path =~ /\/$/) {
         print "The path specified is $root_path\n";
      } else {
         push (@file_errors, "The input argument \"$root_path\" requires a forward slash (\/) at the end.\n");
      }
   } # if 1 or more arguments
   $arg_cnt++;


   if (@ARGV > $arg_cnt) {
      $org_ini = @ARGV[$arg_cnt];
      print "The message input file is $org_ini \n";
   } else {
      push (@file_errors, "Need to have the input message file.");
   } # if 3 or more arguments
   $arg_cnt++;

   if (@ARGV > $arg_cnt) {
      $globe::master_nav_file = @ARGV[$arg_cnt];
      print "The master HTML input file is $globe::master_nav_file \n";
      unless (open ( IN_MASTER, $globe::master_nav_file)) {
         push (@file_errors, "Cannot open file \"$globe::master_nav_file\"\n");
      }
   } else {
      push (@file_errors, "Need to have the input master HTML file.");
   } # if 3 or more arguments
   $arg_cnt++;


   if (@ARGV > $arg_cnt) {
      $org_title = @ARGV[$arg_cnt];
      print "The title for output is \"$org_title\" \n";
   } else {
      $org_title = "Sample INI File";
      print "The title for output is \"$org_title\" \n";
   } # if 3 or more arguments
   $arg_cnt++;


   if (@ARGV > $arg_cnt) {
      $a_header = @ARGV[$arg_cnt];
      print "The additional header input file is $a_header \n";
      unless (open ( IN_HEADER, $a_header)) {
         push (@file_errors, "Cannot open file \"$a_header\"\n");
      }
   } else {
      undef ($a_header);
   } # if 3 or more arguments
   $arg_cnt++;

} # BEGIN
   
   
   

##############################################################################
##! @fn int main
##! @brief The main program.
##!
##! @param file_errors If file errors are encountered before the main
##! routine is hit, it prints them out and exits. 
##!
##! @return None.
##!
##! If the needed data structures in $scope_pm have not been enabled,
##! this program errors out immediately.
##! 
##! Controls the program flow. It creates a temporary file with the
##! error results.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
# sub main {
{
   ##############################################################################
   ## Start of Main Program
   ##############################################################################
   if (@file_errors) {
      # Makes no sense to go on if input parameters are off.
      print "\n============  Summary of errors ==================================\n";
      for ($i=0; $i<= $#file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
      &scheck::using_scheck();
      exit(1);
   }


   if ((1) && (!(&master_file))){
      print "Master file failed some things.\n";
   } 
   
   if ($debug_verbose) {
      print "main and pre-rcnc with \"$org_ini\"\n";
   }
   
   if ((1) && (!(&globe::read_code_n_comment($org_ini, 1)))){
      # The 1 means that we care about separating comments in the
      # message.txt file from the lines that represent a log message.
      print "\"$org_ini\": INI file failed some things.\n";
   } 
   # At this point, $globe::comment_file was created and populated,
   # as was $globe::entire_file.
   if (0) {
      print $globe::entire_file;
      exit(1);
   }
   if (0) {
      print $globe::comment_file;
      exit(1);
   }


   if ((1) && (!(&create_data_structure))){
      print "Creating data structure failed some things.\n";
   } 
   # At this point, the $globe::entire_file, which had the
   # INI data in it, has been processed into a data structure.
   #    $globe::xtag_struct{$hname}{name}
   # Hence, $globe::entire_file is no longer needed.
   # However the $globe::comment_file is still needed.
   

   if ((1) && (!(&globe::process_comments_from_file($org_ini, $tag_xml)))){
      print "1 Processing comments from \"$org_ini\" failed some things.\n";
   } 
   if ($debug_verbose) {
      print "Processed comments from \"$org_ini\" with \"$tag_xml\"\n";
   }
   # At this point, $globe::comment_file has been parsed and
   # can be overwritten.


   #####
   # Do something dirty here by immediately reading in another
   # file that we want to use as data as well.
   #####
   if ($debug_verbose) {
      print "main and pre-rcnc with \"$a_header\"\n";
   }
   if ($a_header =~ /[\w]+/) {
      # if the name is not empty then
      print "Processing \"$a_header\" as an additional file...\n";
      if ((1) && (!(&globe::read_code_n_comment($a_header, 0)))){
         # The 0 means that we do not care about separating comments in the
         # message.txt file from the lines that represent a log message.
         print "Getting additional information from \"$a_header\" failed some things.\n";
      }
      if ($debug_verbose) {
         print "Finished reading into the comment_file from \"$a_header\"\n";
      }
      if ((1) && (!(&globe::process_comments_from_file($a_header, $tag_xml)))){
         print "1 Processing comment_file from \"$a_header\" failed some things.\n";
      } 
      if ($debug_verbose) {
         print "Processed comments from \"$a_header\" with \"$tag_xml\"\n";
      }
   } else {
      if ($debug_verbose) {
         print "We did not try to process \"$a_header\"\n";
      }
   }

   # At this point, $globe::comment_file has been parsed and
   # can be overwritten.
   #####
   
   
   
   # At this point it is safe to call &globe::read_code_n_comment again with
   # another file, because $globe::comment_file and $globe::entire_file
   # have both been placed into the data structure already.
   # Deep down in the next routine it does get called.

   if ((0) && ($debug_verbose)) {
      foreach $name (sort keys %{globe::xtag_struct}){
         if ($debug_verbose) {
            print "d1 $name:$globe::xtag_struct{$name}{comment_full}\n";
            print "d2 $name:$globe::xtag_struct{$name}{comment}\n";
            print "d3 $name:$globe::xtag_struct{$name}{html}\n";
         }
      }
      exit(1);
   }
   
   if ($debug_verbose) {
      print "Converting xtag into mess for \"$org_ini\" with \"$tag_xml\"\n";
   }
   #####
   # Having two structures is important: xtag_struct and tag_mess.
   #
   # The xtag_struct is built based upon what is found when it is read it.
   # It contains the complete list of what is important to worry about (rather
   # than extra things it might have found). Moreover, the xtag_struct
   # is designed for a heirarchy and tracing using children arrays and parent
   # hashes. 
   # Although the top-level name is the same, only a few of the elements
   # actually overlap with implementation in other files (globe.pm). 
   # Additional hash structures are created here as needed.
   #
   #   $hname = created name for hash made from actual names of parents and
   #            children 
   #   $globe::xtag_struct{$hname}{name} = actual item name
   #   $globe::xtag_struct{$hname}{parent} = owning section
   #   $globe::xtag_struct{$hname}{child}[] = array of associated children
   #   $globe::xtag_struct{$hname}{comment} = comment flag
   #   $globe::xtag_struct{$hname}{comment_full} = complete comments with no tags
   #   $globe::xtag_struct{$hname}{desc_html} = description in html
   #   $globe::xtag_struct{$hname}{desc_man} = description in man
   #
   # The tag_mess structure contains information found while chasing
   # references in other files. It can ultimately contain much more information
   # than what is truly needed for a given ini file.
   #
   # The tag_mess{file}{$file}{$identifier_tags}
   # contains to information found for each.
   # Its purpose is to quickly tell me if a referenced file has been
   # processed for the $tag_xml items yet.
   # The tag_mess{name}{$name} merely has an array with associated filenames.
   # The filename is intended to allow you access to the complete data under
   # tag_mess{file}. So the purpose is to see a complete list of what is
   # available by tag.
   #
   # At this point, when need to get the xtag_struct into the tag_mess.
   #####
   if ((1) && (!(&globe::xtag_into_mess_structure($org_ini, $tag_xml)))){
      print "Getting xtag into message structure failed some things; name \"$org_ini\".\n";
   } 
   if ($debug_verbose) {
      print "Converted xtag into mess for \"$org_ini\" with \"$tag_xml\"\n";
   }

   if ((0) && ($debug_verbose)) {
      foreach $name (sort keys %{globe::xtag_struct}){
         if ($debug_verbose) {
            print "d1 $name:$globe::xtag_struct{$name}{comment_full}\n";
            print "d2 $name:$globe::xtag_struct{$name}{comment}\n";
            print "d3 $name:$globe::xtag_struct{$name}{html}\n";
         }
      }
      exit(1);
   }

   #####
   # Comment zap... get rid of html that we don't like.
   #####
   if (1) {
      foreach $name (sort keys %{globe::xtag_struct}){
         if ((0) && ($debug_verbose)) { # debug
            # print "$name:$globe::xtag_struct{$name}{comment_full}\n";
            # print "======\n$globe::xtag_struct{$name}{html}\n";
            print "======\n$name\n";
         } # debug
         $globe::xtag_struct{$name}{comment_full} = &comment_zap($globe::xtag_struct{$name}{comment_full});
         if ((0) && ($debug_verbose)) { # debug
            print "$name:$globe::xtag_struct{$name}{comment_full}\n";
            # print "$name:$globe::xtag_struct{$name}{html}\n";
         } # debug
      }
      # exit(1);
   }
   

   if ((0) && ($debug_verbose)) { # debug
      print "Searching1 for stuff...\n";
      foreach $f (sort keys %{$globe::tag_mess{file}}) {
         foreach $t (sort keys %{$globe::tag_mess{file}{$f}}){
            print "\"$f\":\"$t\":\n$globe::tag_mess{file}{$f}{$t}\n";
         }
      }
      print "Searching2 for stuff...\n";
      foreach $t (sort keys %{$globe::tag_mess{name}}) {
         foreach $i (@{$globe::tag_mess{name}{$t}}){
            print "\"$t\":$globe::tag_mess{name}{$t}[$i]\n";
         }
      }
      print "Searching3 for stuff...\n";
      exit(1);
   } # debug
   
   if ((1) && (!(&find_topic_order ($globe::struct_start, 0)))) {
      print "Finding topic order failed some things.\n";
   }
   


   if ((1) && (!(&topic_prev_next ($globe::struct_start, 0)))) {
      print "Topic prev_next failed some things.\n";
   }
   if ((1) && (!(&generate_desc_html ()))) {
      print "generate descriptive html failed some things.\n";
   }

   if ((0) && ($debug_verbose)) {
      foreach $name (sort keys %{globe::xtag_struct}){
         if (1) {
            # print "$name:$globe::xtag_struct{$name}{comment_full}\n";
            print "$name:$globe::xtag_struct{$name}{desc_html}\n";
         }
      }
      exit(1);
   }

   
   #####
   # Error checking to make sure that things are complete
   #   $globe::xtag_struct{$hname}{name} = actual item name
   #   $globe::xtag_struct{$hname}{comment} = comment flag
   #   $globe::xtag_struct{$hname}{desc_html} = comment flag
   #####
   if (1) { 
      &check_tbd ();
      # exit(1);
   }


   
   #####
   # Output the individual HTML files.
   #####
   if ($yes_output) {
      &write_output_html();
   }
   
   #####
   # The index is also generated in another step
   # hence we don't have to waste time here, but could
   #####
   if (0) {
      &generate_index;
   }

   if ($yes_output) {
      $file_out = "$globe::path";
      $file_out .= "tree.script";
      unless (open ( OUT_SCRIPT, ">$file_out")) {
         push (@file_errors, "2 Cannot open file \"$file_out\"\n");
         print "2 Cannot open file \"$file_out\"\n";
         return (0);
      }
      @path_piece = split (/\//, $globe::path);
      $strip_p = pop (@path_piece);
      $strip_p = pop (@path_piece);
      $strip_p .= "\/";

      #####
      # This top title is part of the master and does not need to be
      # part of this.
      #####
      print (OUT_SCRIPT "Item level=1 ");
      print (OUT_SCRIPT "image=nav_doc.gif ");
      print (OUT_SCRIPT "url=$globe::xtag_struct{$globe::struct_start}{html},basefrm ");
      print (OUT_SCRIPT "selected=YES ");
      print (OUT_SCRIPT "text=$globe::xtag_struct{$globe::struct_start}{name}");
      print (OUT_SCRIPT "\r\n");

      $out_string = "";
      &rec_generate_script ($globe::struct_start, 1);
      print (OUT_SCRIPT "$out_string\r\n");
      close (OUT_SCRIPT);
      if (1) {
         print "Creating $file_out...\n";
      }
   }
   if (0) { # debug
      print "Topic ordering\n=====\n";
      for ($i=0; $i <= $#globe::topic_order; $i++) {
         print "$i:$globe::topic_order[$i]\n";
      }
      # exit(1);
   }

   
   #####
   # This has to be place here well after the HTML is generated,
   # because the first topic does not have the required {desc_html}
   # until then.
   #####
   if ((1) && (!(&generate_desc_man ()))) {
      print "Generating descriptive man failed some things.\n";
      $yes_output_man = 0;
   }
   
   #####
   # Generate the man pages. We'll aim for a [5]
   #   1    User commands that may be started by everyone.
   #   2    System calls, that is, functions provided by the kernel.
   #   3    Subroutines, that is, library functions.
   #   4    Devices, that is, special files in the /dev directory.
   #   5    File format descriptions, e.g. /etc/passwd.
   #   6    Games, self-explanatory.
   #   7    Miscellaneous, e.g. macro packages, conventions.
   #   8    System administration tools that only root can execute.
   #   9    Another (Linux specific) place for kernel routine documentation.
   #   n    (Deprecated) New documentation, that may be moved to a more appropriate section.
   #   o    (Deprecated) Old documentation, that may be kept for a grace period.
   #   l    (Deprecated) Local documentation referring to this particular system.
   #
   #####
   if ((1) && ($yes_output) && ($yes_output_man)) {
      $man_extension = "5";
      $mpath_out = "$globe::path";
      $mpath_out .= "man$man_extension/";
      print "$mpath_out\n";
      &rec_write_man($globe::struct_start);
   }
   
   
   #####
   # Output a sample file with tags.
   #####
   if (1) {
      # print "Here at the end.\n";
      &write_sample();
   }
    
   
   ##############################################################################
   ## Spit out errors
   ##############################################################################
   if ((1) && (@file_errors)) {
      print "\n============  Summary of errors ==================================main\n";
      for ($i=0; $i<@file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
   }

   ##############################################################################
   ## End of most of work
   ##############################################################################
   exit(0);
} # main


##############################################################################
##! @brief Reads in the entire master HTML file
##! 
##! @bug Fills up $globe::master variables.
##############################################################################
sub master_file {
   local ($before) = "";
   local ($after) = "";
   local ($piece) = "";
   local ($debug_master) = 0;
   
   # Get the master definitions
   while (<IN_MASTER>){    # entire master file into memory.
      $globe::master_nav .= $_;
   }

   if (&globe::get_master_nav_info() >= 1){
      print "Master Navigation file \"$globe::master_nav_file\" has definition errors.\n";
      if (@file_errors) {
         print "\n============  Summary of errors ==================================master\n";
         for ($i=0; $i<@file_errors; $i++){
            print "$i = $file_errors[$i]\n";
         }
      }
      return (0);
   } else {
     
     # Create special navigation in the footer that repeats buttons.
     $globe::voy_footer_fm = "$globe::m_define{footer}[0]\n<br><br>\n";
     $def_type = "nav_book";
     ($before, $piece, $after) = &globe::get_tag_chunk( $globe::m_info{$def_type},
                                 $globe::m_define{$def_type}[0], 
                                 $globe::m_define{$def_type}[1], 
                                 $very_critical);

     $globe::voy_footer_fm .= "$piece\n";
     
     $def_type = "footer";
     ($before, $piece, $after) = &globe::get_tag_chunk( $globe::m_info{$def_type},
                                 $globe::m_define{$def_type}[0], 
                                 $globe::m_define{$def_type}[1], 
                                 $very_critical);
     
     $globe::voy_footer_fm .= "$piece\n";
     $globe::voy_footer_fm .= "$globe::m_define{footer}[1]\n";

     
     $globe::master_order_key = @globe::master_order;  # sets the default value
     for ($i=0; $i <= $#globe::master_order -1; $i++){
        # does not go to the last one, because that is the default, already set
        if ($globe::path =~ /$globe::master_order[$i]/){
           $globe::master_order_key = $i;
           if ($debug_master) {
              print "$i = $globe::master_order[$globe::master_order_key]\n";
           }
        }
     }
     if ($debug_master) {
        print "=== Master Header ==\n$globe::m_info{header}\n";
        print "=== Master Common Top ==\n$globe::m_info{nav_common}\n";
        print "=== Master Doxygen ==\n$globe::m_info{nav_doxygen}\n";
        print "=== Master FM Book ==\n$globe::m_info{nav_book}\n";
        print "=== Master Footer ==\n$globe::m_info{footer}\n";
        print "=== Master Dox Footer ==\n$globe::voy_footer_dox\n";
        print "=== Master FM Footer ==\n$globe::voy_footer_fm\n";
        print "=== Master variables ==\n$globe::m_info{variable}\n";
        # return (0);
     }
   }
   return (1);
} # master file


##############################################################################
##! @brief Creates a data structure from each line in the message file.
##! 
##!       $hname = created name for hash made from actual names of parents and
##!                children 
##!       $globe::xtag_struct{$hname}{name} = actual item name
##!       $globe::xtag_struct{$hname}{parent} = owning section
##!       $globe::xtag_struct{$hname}{child}[] = array of associated children
##!       $globe::xtag_struct{$hname}{comment} = comment flag
##!       $globe::xtag_struct{$hname}{comment_full} = complete comments with no tags
##!       $globe::xtag_struct{$hname}{desc_html} = description in html
##!       $globe::xtag_struct{$hname}{desc_man} = description in man
##! 
##! @bug 
##############################################################################
sub create_data_structure {

   local ($debug_cds) = 0;
   undef (%globe::xtag_struct);
   
   $hname = $globe::struct_start;
   $globe::xtag_struct{$hname}{name} = "$org_title";
   $globe::xtag_struct{$hname}{parent} = "$globe::struct_start";
   
   @_input_line = split (/\n/, $globe::entire_file);
   
   $file_index = 0;
   
   while ($file_index <= $#_input_line) {
      if ($debug_cds) {
         print "$file_index parent: \"$hname\"\n";
      }
      $hname = &build_trace_xtag_struct ($hname, $file_index);
      $file_index++;
   }
   
   if (0) {
      # &rec_trace_struct ($globe::struct_start);
      &rec_trace_struct2 ($globe::struct_start, 0);
      exit(1);
   }
   
   return (1);
} # create_data_structure

##############################################################################
##! @brief Traces through the input and builds an xtag_struct for the INI file.
##!
##! @param h_ind The name of the parent section as the hash index.
##! @param file_line_index The index to the file array.
##!
##! @return The hash index for the parent; that is either the parent
##! already passed in, or itself if it is determined to have children
##! based on whether or not it is enclosed in braces. In addition, the
##! xtag_struct is built up for the element.
##!
##!  $name is the key into the hash made up of all previous levels
##! and this level.
##!       $globe::xtag_struct{$name}{name} = Display name for item.
##!
##############################################################################
sub build_trace_xtag_struct {
   $h_ind = $_[0];
   $file_line_index = $_[1];
   local ($debug_btxs) = 0;
   undef (@line_chunk);
   undef (@line_chunk2);
   undef (@line_chunk3);
   local ($plus) = 0;
   
   $line = $_input_line[$file_line_index];
   $line =~ s/^\s*//;  # strip out leading spaces
   $line =~ s/\s*$//;  # strip out trailing spaces
   if ($debug_btxs) {
      print "line:$line\n";
   }

   # 08/12/2002 handle appending to a variable
   if ($line =~ /\/\//) {
      # get to the components of the line
      @line_chunk = split (/\/\//, $line, 2);
      if ($line_chunk[0] =~ /\+\=/) {
         # Then it has a default value
         @line_chunk2 = split (/\+\=/, $line_chunk[0], 2);
         $name = $line_chunk2[0];   # name is in front of +=
         # get default value which is after the ='s.
         $default = $line_chunk2[1];
         $plus = 1;
      } elsif ($line_chunk[0] =~ /\=/) {
         # Then it has a default value
         @line_chunk2 = split (/\=/, $line_chunk[0], 2);
         $name = $line_chunk2[0];   # name is in front of =
         # get default value which is after the ='s.
         $default = $line_chunk2[1];
      } else {
         # Then it does not a default value; probably a section
         $default = "";
         $name = $line_chunk[0];  # name is in front of //
      }
   } else {
      # Item has no comments
      if ($line =~ /\+\=/) {
         # Then it has a default value
         @line_chunk2 = split (/\+\=/, $line, 2);
         $name = $line_chunk2[0];   # name is in front of +=
         # get default value which is after the +='s.
         $default = $line_chunk2[1];
         $plus = 1;
      } elsif ($line =~ /\=/) {
         # Then it has a default value
         @line_chunk2 = split (/\=/, $line, 2);
         $name = $line_chunk2[0];   # name is in front of =
         # get default value which is after the ='s.
         $default = $line_chunk2[1];
      } else {
         # Then it does not a default value; probably a section
         $default = "";
         $name = $line;  # name is the line
      }
   }
   $name =~ s/^[\s]*//g;  # get rid of leading space
   $name =~ s/[\s]*$//g;  # get rid of trailing space

   if ($debug_btxs) {
      print "name1:$name\n";
   }
   $name =~ s/\s*$//;  # strip out trailing spaces
   $bracket_flag = 0;
   if ($name =~ /\[/) {
      $name =~ s/\[//;    # strip out brackets if section
      $name =~ s/\]//;    # strip out brackets if section
      $bracket_flag = 1;
   }
   if ($debug_btxs) {
      print "name2:$name\n";
   }
   $default =~ s/^\s*//;  # strip out leading spaces
   $default =~ s/\s*$//;  # strip out trailing spaces
   if ($debug_btxs) {
      print "default:$default\n";
   }
   $new_h_ind = "$h_ind\-$name";

   # We check for comments again because when we do it above,
   # it is too detailed in a special case for defaults.
   if ($line =~ /\/\//) {
      # redefine line_chunk; okay
      @line_chunk = split (/\/\//, $line, 2);
      $comment = $line_chunk[1];
      $comment =~ s/^[\s]*//;  # strip out leading spaces
   } else {
      $comment = "";
   }
   
   #####
   # Create the data structure for messages.
   #####
   if ($line =~ /^\[\]/) {
      #####
      # End of a section
      #####
      if ($debug_btxs) {
         print "end   :\"$line\" with parent:\"$globe::xtag_struct{$h_ind}{parent}\"\n";
      }
      return ($globe::xtag_struct{$h_ind}{parent});

   } elsif ($line =~ /^\[/) {
      #####
      # Start of another section
      #####
      if ($debug_btxs) {
         print "begin   :\"$name\" and \"$new_h_ind\"\n";
      }
      
      push (@{$globe::xtag_struct{$h_ind}{child}}, $new_h_ind);
      if ($comment) {
         $globe::xtag_struct{$new_h_ind}{comment} = $comment;
      }
      $globe::xtag_struct{$new_h_ind}{parent} = $h_ind;
      if ($bracket_flag) {
         $globe::xtag_struct{$new_h_ind}{name} = "\[$name\]";
      } else {
         $globe::xtag_struct{$new_h_ind}{name} = $name;
      }
      return ($new_h_ind);

   } else {
      #####
      # A child of the section
      #####
      if ($debug_btxs) {
         print "middle1   :$name, default :$default\n";
      }
      
      push (@{$globe::xtag_struct{$h_ind}{child}}, $new_h_ind);
      if ($comment) {
         $globe::xtag_struct{$new_h_ind}{comment} = $comment;
      }
      # $default =~ s/\(//g;
      # $default =~ s/\)//g;
      if ($debug_btxs) {
         print "middle2   :$name, default :$default\n";
      }
      if ($default) {
         $globe::xtag_struct{$new_h_ind}{default} = "$default";
      }
      if ($plus) {
         $globe::xtag_struct{$new_h_ind}{plus} = "\+";
      }
      if ($debug_btxs) {
         print "middle3   :$name, default :$globe::xtag_struct{$new_h_ind}{default}\n";
      }

      $globe::xtag_struct{$new_h_ind}{parent} = $h_ind;
      if ($bracket_flag) {
         $globe::xtag_struct{$new_h_ind}{name} = "\[$name\]";
      } else {
         $globe::xtag_struct{$new_h_ind}{name} = $name;
      }
      return ($h_ind);
   } # Create the data structure for messages.
   
} # build_trace_xtag_struct


##############################################################################
##! @brief Used as an example to trace through the structure.
##!
##! @param seed The current element in the structure.
##! @param level The current level for the seed; used to print out
##! the appropriate amount of leading spaces for indents.
##!
##! This routine isn't used as is. However, it forms the basis for
##! other specific routines for printing out HTML, script files,
##! or other things. When in doubt about what is being built,
##! this routine can be called.
##!
##! @bug This routine is called recursively as it traces through the structure.
##! It steps through the children array for a given item.
##############################################################################
sub rec_trace_struct2 {
   local ($seed) = $_[0];
   local ($level) = $_[1];
   
   if (0) {
      print "seed: \"$seed\"\n";
   }
   if (exists ($globe::xtag_struct{$seed}{child})) {
      # means we need to trace down into it
      if ($globe::struct_start !~ /$seed/) {
         for ($i=1; $i<=$level; $i++){
            print "   ";
         }
         print "$globe::xtag_struct{$seed}{name}";
         if (exists ($globe::xtag_struct{$seed}{comment})) {
            print "\t      \/\/ $globe::xtag_struct{$seed}{comment}\n";
         } else {
            print "\n";
         }
         $level++;
      }
      foreach $child (@{$globe::xtag_struct{$seed}{child}}) { 
         # print "name:  \"$hname\"\n";
         &rec_trace_struct2 ($child, $level);
         if (0) {
            print "seed: \"$seed\"\n";
         }
      }
      if ($globe::struct_start !~ /$seed/) {
         $level--;
         for ($i=1; $i<=$level; $i++){
            print "   ";
         }
         print "\[\]";
         print "\t      \/\/ $globe::xtag_struct{$seed}{name}\n";
      }
   } else {
      # means we're at a child level with no children
      # can print out and return
      if ($globe::struct_start !~ /$seed/) {
         for ($i=1; $i<=$level; $i++){
            print "   ";
         }
         if (1) {
            # print "ddddefault :$globe::xtag_struct{$seed}{default}\n";
            if (exists ($globe::xtag_struct{$seed}{default})) {
               if (exists ($globe::xtag_struct{$seed}{comment})) {
                  print "$globe::xtag_struct{$seed}{name} ";
                  print "$globe::xtag_struct{$seed}{plus}\= ";
                  print "$globe::xtag_struct{$seed}{default}";
                  print "\t      \/\/ $globe::xtag_struct{$seed}{comment}\n";
               } else {
                  print "$globe::xtag_struct{$seed}{name} ";
                  print "$globe::xtag_struct{$seed}{plus}\= ";
                  print "$globe::xtag_struct{$seed}{default}";
                  # print "\t      \/\/ no comment Dude!!!\n";
                  print "\n";
               }
            } else {
               if (exists ($globe::xtag_struct{$seed}{comment})) {
                  print "$globe::xtag_struct{$seed}{name}";
                  print "\t      \/\/ $globe::xtag_struct{$seed}{comment}\n";
               } else {
                  print "$globe::xtag_struct{$seed}{name}";
                  # print "\t      \/\/ no comment Dude!!!\n";
                  print "\n";
               }
            }
         } # if (1)
      }
   } 
} # rec_trace_struct2

##############################################################################
##! @brief Returns a list of the owning sections for a given INI item.
##!
##! @param pseed The point to start at in the structure.
##! @param html Whether or not to output html for the list.
##!
##! @return It returns a string that either is formatted as HTML or is
##! formatted as simply text. The HTML version is fully hyperlinked.
##!
##! From the initial starting point, this keeps asking for the parent
##! until it reaches the top most level.
##!
##! At each level, it appends the name information to a string.
##!       $globe::xtag_struct{$hname}{parent} = owning section
##############################################################################
sub parent_trace {
   local ($pseed) = $_[0];
   local ($html) = $_[1];
   local ($org_seed) = $pseed;
   local ($outpath) = "$globe::xtag_struct{$pseed}{name}";
   local ($before) = "";
   local ($after) = "";
   local ($piece) = "";
   
   while ($globe::struct_start !~ /$pseed/) {
      $pseed = $globe::xtag_struct{$pseed}{parent};
      if ($globe::struct_start !~ /$pseed/) {
         $outpath = "<a href=\"$globe::xtag_struct{$pseed}{html}\">$globe::xtag_struct{$pseed}{name}</a>\.$outpath";
      } elsif ($html) {
         $outpath = "<a href=\"$globe::xtag_struct{$pseed}{html}\#$org_seed\">\{Context\}</a>\.$outpath";
      } else {
         # for non-html, we don't need the top level context reference.
      }
   }
   if (!($html)) {
      # strip out all html references
      while ((0) && ($outpath =~ /\</)) {
         # print "yes1";
         ($before, $piece, $after) = &globe::get_tag_chunk( $outpath,
                                  "\<", "\>", 0);
         $outpath = join ("", $before, $after);
      }
      while ((1) && ($outpath =~ /<a href/)) {
         # print "yes1";
         ($before, $piece, $after) = &globe::get_tag_chunk( $outpath,
                                  "<a href", "\>", 0);
         $outpath = join ("", $before, $after);
      }
      while ((1) && ($outpath =~ /<\/a/)) {
         # print "yes2";
         ($before, $piece, $after) = &globe::get_tag_chunk( $outpath,
                                  "<\/a", "\>", 0);
         $outpath = join ("", $before, $after);
      }
   }
   return ($outpath);
} # parent_trace


##############################################################################
##! @brief Creates an example INI file in HTML based on what is known.
##! 
##! @param seed Starting point in the structure.
##! @param level Starting level in the structure. Used to control indenting in
##! the output.
##!
##! @retval out_string is a global variable holding the desired output. This
##! output is fully hyperlinked.
##!
##! This calls itself recursively as it steps through the xtag_struct for
##! each parent and child array. It creates appropriate index tags
##! for subsequent tool calls. Its purpose is to show the context of an INI
##! item.
##!
##! @bug This is called recursively. The $out_string is used in a global
##! manner and exist from call-to-call.
##!
# &rec_example_topic ($globe::struct_start, 0);
##############################################################################
sub rec_example_topic {
   local ($seed) = $_[0];
   local ($level) = $_[1];
   
   if (0) {
      print "seed: \"$seed\"\n";
   }
   if (exists ($globe::xtag_struct{$seed}{child})) {
      # means we need to trace down into it
      if ($globe::struct_start !~ /$seed/) {
         $out_string .= "<pre class=\"$globe::pre_class\">";
         for ($i=1; $i<=$level; $i++){
            $out_string .= "   ";
         }
         #####
         # Puts in anchor/index/target to topic
         # <a name="zzzz" class="v_index" value="token text">
         #####
         $temp = $globe::xtag_struct{$seed}{name};
         $temp =~ s/\[//; # strip out brackets
         $temp =~ s/\]//; # strip out brackets
         $out_string .= "<a name=\"";
         $out_string .= "$seed";
         $out_string .= "\" class=\"v_index\" value=\"$temp\">";
         $out_string .= "</a>";
         #####
         # Puts in hyperlink to topic
         #####
         $out_string .= "<a href=\"";
         $out_string .= "$globe::xtag_struct{$seed}{html}";
         $out_string .= "\">";
         $out_string .= "$globe::xtag_struct{$seed}{name}";
         $out_string .= "</a>";
         $out_string .= "</pre>\r\n";
         $level++;
      }
      foreach $child (@{$globe::xtag_struct{$seed}{child}}) { 
         &rec_example_topic ($child, $level);
      }
      if ($globe::struct_start !~ /$seed/) {
         $level--;
         $out_string .= "<pre class=\"$globe::pre_class\">";
         for ($i=1; $i<=$level; $i++){
            $out_string .= "   ";
         }
         $out_string .= "\[\]";
         $out_string .= "   \/\/ $globe::xtag_struct{$seed}{name}";
         $out_string .= "</pre>\r\n";
      }
   } else {
      # means we're at a child level with no children
      # can print out and return
      if ($globe::struct_start !~ /$seed/) {
         $out_string .= "<pre class=\"$globe::pre_class\">";
         for ($i=1; $i<=$level; $i++){
            $out_string .= "   ";
         }
         #####
         # Puts in anchor/index/target to topic
         # <a name="zzzz" class="v_index" value="token text">
         #####
         $temp = $globe::xtag_struct{$seed}{name};
         $temp =~ s/\[//; # strip out brackets
         $temp =~ s/\]//; # strip out brackets
         $out_string .= "<a name=\"";
         $out_string .= "$seed";
         $out_string .= "\" class=\"v_index\" value=\"$temp\">";
         $out_string .= "</a>";
         #####
         # Puts in hyperlink to topic
         #####
         $out_string .= "<a href=\"";
         $out_string .= "$globe::xtag_struct{$seed}{html}";
         $out_string .= "\">";
         if (exists ($globe::xtag_struct{$seed}{default})) {
            $out_string .= "$globe::xtag_struct{$seed}{name}";
            $out_string .= "</a>";
            $out_string .= " $globe::xtag_struct{$seed}{plus}\=";
            $out_string .= " $globe::xtag_struct{$seed}{default}";
         } else {
            $out_string .= "$globe::xtag_struct{$seed}{name}";
            $out_string .= "</a>";
         }
        
         $out_string .= "</pre>\r\n";
         
         { # indent brackets.
            # for when a section is defined but has no children.
            if ($globe::xtag_struct{$seed}{name} =~ /^\[/) {
               if ($globe::struct_start !~ /$seed/) {
                  # $level--;
                  $out_string .= "<pre class=\"$globe::pre_class\">";
                  for ($i=1; $i<=$level; $i++){
                     $out_string .= "   ";
                  }
                  $out_string .= "\[\]";
                  $out_string .= "   \/\/ $globe::xtag_struct{$seed}{name}";
                  $out_string .= "</pre>\r\n";
               }
            } # when section has no children
         } # indent brackets
        
         
      }
   } 
} # rec_example_topic


##############################################################################
##! @brief Traces the xtag_struct recursively.
##! 
##! @param seed Starting point in the structure.
##!
##! This is not used directly. However, it serves as an example for other
##! operations that might need to recursively step through the data structure.
##! Several other routines for generating output or manipulating navigation
##! were based upon this routine.
##! 
##! @bug This is called recursively.
##############################################################################
sub rec_trace_struct {
   local ($seed) = $_[0];
   
   if (0) {
      print "seed: \"$seed\"\n";
   }
   if (exists ($globe::xtag_struct{$seed}{child})) {
      # means we need to trace down into it
      if (1) {
         print "section:$globe::xtag_struct{$seed}{name}\n";
      }
      foreach $child (@{$globe::xtag_struct{$seed}{child}}) { 
         # print "name:  \"$hname\"\n";
         &rec_trace_struct ($child);
      }
   } else {
      # means we're at a child level with no children
      # can print out and return
      if (1) {
         print "final child: \"$globe::xtag_struct{$seed}{name}\" with  \"$globe::xtag_struct{$seed}{parent}\" \n";
      }
   }
} # rec_trace_struct

##############################################################################
##! @brief Steps through the xtag_struct to create a simple linear array
##! with the topic order.
##! 
##! @param seed Starting point in the structure.
##! @param level Starting level in the structure. Used to control indenting in
##! the output.
##!
##! @retval globe::topic_order is an array with just the hash keys into the
##! xtag_struct in the order that they should appear.
##! 
##! The topic_order array can be more convenient for various operations, such
##! as finding the previous/next elements and for printing out the output. It
##! is simplier and easier to step through than the recursive algorithms.
##! 
##! @bug This is called recursively.
##############################################################################
sub find_topic_order {
   local ($seed) = $_[0];
   local ($level) = $_[1];
   
   local ($html_name) = $seed;
   $html_name =~ s/^$globe::struct_start//;
   $html_name =~ s/^\-//;
   $html_name .= ".html";
   
   if (0) {
      print "hola seed: \"$seed\"\n";
      # print "hola html: \"$html_name\"\n";
   }
   push (@globe::topic_order, $seed);
   if (exists ($globe::xtag_struct{$seed}{child})) {
      # means we need to trace down into it
      if ($globe::struct_start !~ /$seed/) {
         if (0) {
            print "$globe::xtag_struct{$seed}{name}\n";
         }
         $level++;
      }
      
      foreach $child (@{$globe::xtag_struct{$seed}{child}}) { 
         &find_topic_order ($child, $level);
      }
      if ($globe::struct_start !~ /$seed/) {
         $level--;
      }
   } else {
      # means we're at a child level with no children
      # can print out and return
      
      if ($globe::struct_start !~ /$seed/) {
         if (0) {
            print "$globe::xtag_struct{$seed}{name}\n";
         } # if (1)
      }
   }
   return (1); 
} # find_topic_order



##############################################################################
##! @brief Creates the HTML descriptions for the hash elements.
##! 
##! @param None.
##!
##! @retval Creates xtag_struct{$name}{desc_html} which is just a copy
##! of the xtag_struct{$name}{comment_full}.
##! 
##! This doesn't do much, but could in the future.
# 08/01/2002 This needs to be completed properly
##############################################################################
sub generate_desc_html {
   if (1) {
      # 08/01/2002 This assumes that the comments in the INI file are
      # already HTML tagged.
      for ($i=0; $i <= $#globe::topic_order; $i++) {
         $globe::xtag_struct{$globe::topic_order[$i]}{desc_html} = $globe::xtag_struct{$globe::topic_order[$i]}{comment_full};
         # print "$i:$globe::topic_order[$i]\n";
      }
   }
   return (1);
} # generate_desc_html


##############################################################################
# 08/06/2002 
##! @brief Removes elements from the HTML in the comments that we don't
##! want anymore.
##!
##! @param zap_it_string The section containing things to eliminate.
##! 
##! @return The same string but with elements removed.
##! 
##! @bug This uses the $globe::m_info{_array_html_zap}[] array which gets
##! defined by the master file read in. A trick is to use a different master
##! file for creating the initial HTML than the one used by voyant_nav.pl
##! to get a standard look and feel. More things can be uniquely zapped.
##!
##############################################################################
sub comment_zap {
   local ($zap_it_string) = $_[0];
   local ($debug_all) = 0;  # turns on/off debug for this whole routine
   
   if ((0) && ($debug_all)) {
      print "cz:HTML zapping ===\n";
      # print "$piece\n";
      for ($i=0; $i <= $#{$globe::m_info{_array_html_zap}}; $i++){
         print "cz:$i $globe::m_info{_array_html_zap}[$i]\n";
      }
   }
   if ($debug_all) {
      print "cz:BEFORE:$zap_it_string\n";
   }
   
   #################################
   # Remove certain specified HTML tags
   #################################
   if (($#{$globe::m_info{_array_html_zap}} == 0) || ($globe::m_info{_array_html_zap}[0] =~ /none/)){
      # Do nothing
      if ($debug_all) {
         print "NO ZAP: \"$zap_it_string\" has no tags to zap.\n";
      }
   } else {
      # Remove the tags in question
      for ($i=0; $i<= $#{$globe::m_info{_array_html_zap}}; $i++){
         # Determine start and Stop tags
         @zap_tag = split ( /\,/, $globe::m_info{_array_html_zap}[$i]);
         # clean up tag
         for ($j=0; $j <= $#zap_tag; $j++){
            $zap_tag[$j] =~ s/^[\s]*\"//;
            $zap_tag[$j] =~ s/\"[\s]*$//;
            if ((0) && ($debug_all)) {
               print "cz:Zap this =$zap_tag[$j]\n";
            }
         }
         $t_cnt=0;
         while ($zap_it_string =~ /$zap_tag[0]/i){  # case insensitive test
             $t_cnt++;
             if ((1) && ($debug_all)) {
                print "cz:$t_cnt: Removing everything between \"$zap_tag[0]\" and \"$zap_tag[1]\"\n";
             }
             # First tag part A
             @pre_chunks = split ( /$zap_tag[0]/, $zap_it_string, 2);
             # First tag part B
             @post_chunks = split ( /$zap_tag[1]/, $pre_chunks[1], 2);
             if ($#zap_tag > 1){
                # Second tag
                # Hit the first occurrence of the ending tag
                $post_chunks[1] =~ s/$zap_tag[2]//i; # do it only once, case insensitive
             }
             # rebuild
             $zap_it_string = join ("", $pre_chunks[0], $post_chunks[1]);
         } # while it still is infested with the tag
      } # for all zap tags
      if ((0) && ($debug_all)) {
         print "cz:$zap_it_string was tested for tags to zap.\n";
      }
   } # if zap tags to remove
   undef(@pre_chunks);
   undef(@post_chunks);
   #################################
   
   if ($debug_all) {
      print "cz:AFTER:$zap_it_string\n";
   }
   
   return ($zap_it_string);
} # comment_zap

##############################################################################
##############################################################################
sub check_tbd {

   if ((1) || ($debug_verbose)) { # debug
      print "Topic ordering\n=====\n";
   } # debug
   for ($i=0; $i <= $#globe::topic_order; $i++) {
      if ((0) || ($debug_verbose)) { # debug
         print "$i:$globe::topic_order[$i]\n";
      }
      $hname = $globe::topic_order[$i];
      ####
      # Set up default error text
      ####
      local ($ttt_error_out) = "_TBD_: ";
      $ttt_error_out .= "\"$globe::xtag_struct{$globe::xtag_struct{$hname}{parent}}{name}\.";
      $ttt_error_out .= "$globe::xtag_struct{$hname}{name}\" ";
      $ttt_error_out .= "is not commented.";
      ####
      if (exists ($globe::xtag_struct{$hname}{comment})) {
        # A single comment is better than nothing
        $ttt_out = $globe::xtag_struct{$hname}{desc_html};
        # Strip out all html.
        while ($ttt_out =~ /</) {
            ($before, $piece, $after) = &globe::get_tag_chunk( $ttt_out,
                  "<", ">", 0);
            $ttt_out = join ("", $before, $after); 
         }
         $ttt_out =~ s/\r//g;  # strip out Windows
         $ttt_out =~ s/\n//g;  # strip out carriage return
         $ttt_out =~ s/^[\s]+//;  # strip out leading white space
         $ttt_out =~ s/[\s]+$//;  # strip out trailing white space
            
         # If what is leftover is more than a word, we might have a true description.
         if ((0) || ($debug_verbose)) { # debug
            print "ec:\"$globe::xtag_struct{$hname}{name}\" with:\n$ttt_out\n=====\n";
         }
         if ($ttt_out =~ /[\w]+/) {
            # Means that we have some information and its probably documented.
            # No further action required.
            @ttt_chunk = split (/[\s]+/, $ttt_out);
            if ($#ttt_chunk < 7) {
               # Overwrite the default definition
               $ttt_error_out = "_TBD_: ";
               $ttt_error_out .= "$globe::xtag_struct{$globe::xtag_struct{$hname}{parent}}{name}\.";
               $ttt_error_out .= "$globe::xtag_struct{$hname}{name} ";
               $ttt_error_out .= "is weak \($#ttt_chunk words\) and probably not commented.";
            } else {
               ####
               # The only case where we feel it is adequately documented.
               ####
               $ttt_error_out = "";
            }
         } else {
            # Means that we have too little information.
            # Use the default definition for $ttt_error_out
         }
      }
      if ($ttt_error_out) {
         if ($debug_verbose) {
            print "$ttt_error_out\n";
         }
         push (@file_errors, "$ttt_error_out");
      }
      undef ($ttt_out);
      undef ($ttt_error_out);
   }  # for loop over array
   if ((0) || ($debug_verbose)) { # debug
      print "Finished the cheap error checking routines.\n";
   } # debug
} # check_tbd


##############################################################################
##! @brief Steps through the topic_order array of keys into the hash
##! xtag_struct to find the previous and next html topics.
##! 
##! @param None
##!
##! @retval xtag_struct has elements filled in for:
##! 
##!       $globe::xtag_struct{$name}{prev} = previous html file
##!       $globe::xtag_struct{$name}{next} = next html file
##!       $globe::xtag_struct{$name}{html} = associated html file
##! 
##! @globe::topic_order has the proper ordering of topics and represents
##! the keys into the $globe::xtag_struct
##############################################################################
sub topic_prev_next {
   local ($prev) = "_TBD_";
   local ($next) = "_TBD_";
   
   local ($html_name) = "_TBD_";

   if (0) {
      print "topic_prev_next\n=====\n";
      for ($i=0; $i <= $#globe::topic_order; $i++) {
         print "$i:$globe::topic_order[$i]\n";
      }
   }
   #####
   # Assign names to the topics
   #####
   for ($i=0; $i <= $#globe::topic_order; $i++) {
      $html_name = $globe::topic_order[$i];
      $html_name =~ s/^$globe::struct_start/$globe::start_name/;
      $html_name =~ s/^\-//;
      $html_name .= ".html";
      # print "$i:$globe::topic_order[$i]\n";
      # print "$i:$html_name\n";
      $globe::xtag_struct{$globe::topic_order[$i]}{html} = $html_name;
   }
   
   #####
   # Assign next file pointers to the topics
   # go 1st to second from last
   #####
   # Avoid the seed value
   # for ($i=1; $i < $#globe::topic_order; $i++) {
   for ($i=0; $i < $#globe::topic_order; $i++) {
      $globe::xtag_struct{$globe::topic_order[$i]}{next} = $globe::xtag_struct{$globe::topic_order[$i+1]}{html};
      # print "$i:$globe::topic_order[$i]\n";
   }
   #####
   # Assign prev file pointers to the topics
   # go 2nd to very last
   #####
   # Avoid the seed value
   # for ($i=2; $i <= $#globe::topic_order; $i++) {
   for ($i=1; $i <= $#globe::topic_order; $i++) {
      $globe::xtag_struct{$globe::topic_order[$i]}{prev} = $globe::xtag_struct{$globe::topic_order[$i-1]}{html};
      # print "$i:$globe::topic_order[$i]\n";
   }
   #####
   # Fix first and last previous/next pointers
   # 08/01/2002 You'll need to fix this if overview topics are created.
   #####
   # Avoid the seed value
   $globe::xtag_struct{$globe::topic_order[0]}{prev} = $globe::xtag_struct{$globe::topic_order[$#globe::topic_order]}{html};
   if (0) {
      print "Tweak to 1 prev: $globe::xtag_struct{$globe::topic_order[1]}{prev}\n";
   }
   $globe::xtag_struct{$globe::topic_order[$#globe::topic_order]}{next} = $globe::xtag_struct{$globe::topic_order[0]}{html};
   if (0) {
      print "Tweak to 1 next: $globe::xtag_struct{$globe::topic_order[$#globe::topic_order]}{next}\n";
   }

   #####
   # Check on assignment
   #####
   if (0) {
      # Avoid the seed value
      for ($i=0; $i <= $#globe::topic_order; $i++) {
         print "$i:\<\t$globe::xtag_struct{$globe::topic_order[$i]}{prev}\n";
         print "$i:$globe::xtag_struct{$globe::topic_order[$i]}{html}\n";
         print "$i:\>\t$globe::xtag_struct{$globe::topic_order[$i]}{next}\n";
      }  
      for ($i=0; $i <= 2; $i++) {
         print "$i:\<\t$globe::xtag_struct{$globe::topic_order[$i]}{prev}\n";
         print "$i:$globe::xtag_struct{$globe::topic_order[$i]}{html}\n";
         print "$i:\>\t$globe::xtag_struct{$globe::topic_order[$i]}{next}\n";
      }  
   }
   return (1);
} # topic_prev_next





##############################################################################
##! @brief Creates an HTML file for each element in the topic_order array.
##!
##! @param None.
##!
##! @return Creates an HTML file in the globe::path directory.
##! 
##! This steps through each element in the topic_order array, which is a
##! sorted list of keys into the xtag_struct hash.
##! 
##! Using the key into the hash, it creates an HTML topic based on information
##! from the hash for the item.
##! 
##! Only the first topic -- the top-level array element and starting point
##! for any recursive seed operation into the hash -- has something unique
##! given for it. Instead of nothing, it produces a generic INI sample that
##! is fully hyperlinked and index-marked. This way, users can see the elements
##! in context.
## @param file_out The name of the file to output.
##       $globe::xtag_struct{$hname}{name} = actual item name
##       $globe::xtag_struct{$hname}{parent} = owning section
##       $globe::xtag_struct{$hname}{child}[] = array of associated children
##       $globe::xtag_struct{$hname}{comment} = comment flag
##       $globe::xtag_struct{$hname}{comment_full} = full comments w/o tags
##       $globe::xtag_struct{$hname}{desc_html} = description in html
##       $globe::xtag_struct{$hname}{desc_man} = description in man
##
##       $name = name associated with number
##       $globe::xtag_struct{$name}{prev} = previous html file
##       $globe::xtag_struct{$name}{next} = next html file
##       $globe::xtag_struct{$name}{html} = associated html file
##!
##! @ingroup tp_xhelp
##############################################################################
sub write_output_html {

   $def_type = "variable";
   $total = $#globe::topic_order;

   OHTM: for ($i=0; $i <= $#globe::topic_order; $i++) {
      # $globe::xtag_struct{$globe::topic_order[$i]}{prev}
      $name = $globe::topic_order[$i];
      $file_out = "$globe::path";
      $file_out .= "$globe::xtag_struct{$name}{html}";

      unless (open ( OUT_FILE, ">$file_out")) {
         push (@file_errors, "3 Cannot open file \"$file_out\"\n");
         print "3 Cannot open file \"$file_out\"\n";
         next OHTM;
      }
      if (1) {
         print "Creating $file_out...\n";
      }
      print (OUT_FILE "<html>\n <head>\r\n");
      # print (OUT_FILE "<title>$name</title>\r\n");
      print (OUT_FILE "$globe::m_info{header}\r\n");

      print (OUT_FILE "$globe::m_define{$def_type}[0] ");
      
      print (OUT_FILE "$globe::m_var{define}{prev}$globe::xtag_struct{$name}{prev}");
      print (OUT_FILE "$globe::m_var{define}{next}$globe::xtag_struct{$name}{next}");
      if (0) {
         print "$globe::m_var{define}{prev}$globe::xtag_struct{$name}{prev}";
         print "$globe::m_var{define}{next}$globe::xtag_struct{$name}{next}\n";

      }
      if ($i == 0) {
         print (OUT_FILE "$globe::m_var{define}{firstlast}1\#0");
      } elsif ($i == $total) {
         print (OUT_FILE "$globe::m_var{define}{firstlast}0\#1");
      } else {
         print (OUT_FILE "$globe::m_var{define}{firstlast}0\#0");
      }
      
      print (OUT_FILE "$globe::m_define{$def_type}[1] \r\n");
      print (OUT_FILE "</head>\n<body>\r\n");
      #####
      # Fix up the navigation
      #####
      $nav_bars = $globe::m_info{nav_common};
      $nav_bars .= $globe::m_info{nav_book};
      $nav_bars =~ s/$globe::m_var{define}{prev}/$globe::xtag_struct{$name}{prev}/;
      $nav_bars =~ s/$globe::m_var{define}{next}/$globe::xtag_struct{$name}{next}/;
      $out_string = $nav_bars;
      $out_string .= "<h1>$globe::xtag_struct{$name}{name}</h1>\r\n";
      $trace_parent = &parent_trace ($name, 1);  # the 1 says to do html
      $out_string .= "<p class=\"TitleChapterTOC\">Section: $trace_parent</p>\r\n";
      # $out_string .= "<p class=\"TitleChapter\">Section: $trace_parent</p>\r\n";
      if (exists ($globe::xtag_struct{$name}{child})) {
         # means that this is a section
         $out_string .= "<p class=\"TitleChapterTOC\">Syntax:</p>\r\n"; 
         # $out_string .= "<p class=\"TitleChapter\">Syntax:</p>\r\n"; 
         $out_string .= "<p class=\"Head1TOC\">$globe::xtag_struct{$name}{name}</p>\r\n";
         # $out_string .= "<p class=\"Head1\">$globe::xtag_struct{$name}{name}</p>\n";
         $out_string .= "<p class=\"TitleChapterTOC\">Items:</p>\n";
         # $out_string .= "<p class=\"TitleChapter\">Items:</p>\n";
         $out_string .= "<ul>\n";
         for ($j=0; $j <= $#{$globe::xtag_struct{$name}{child}}; $j++) {
            $child = $globe::xtag_struct{$name}{child}[$j];
            $child_name = $globe::xtag_struct{$child}{name};
            $child_html = $globe::xtag_struct{$child}{html};
            $out_string .= "<li>";
            $out_string .= "<a href=\"$child_html\">";
            $out_string .= "$child_name";
            $out_string .= "</a>";
            $out_string .= "</li>\n";
         }
         $out_string .= "</ul>\n";
      } else {
         # means that this is a child
         $out_string .= "<p class=\"TitleChapterTOC\">Syntax:</p>\n"; 
         # $out_string .= "<p class=\"TitleChapter\">Syntax:</p>\n"; 
         $out_string .= "<p class=\"Head1TOC\">$globe::xtag_struct{$name}{name}";
         # $out_string .= "<p class=\"Head1\">$globe::xtag_struct{$name}{name}";
         if (exists ($globe::xtag_struct{$name}{default})) {
            $out_string .= " $globe::xtag_struct{$name}{plus}\=";
            $out_string .= " $globe::xtag_struct{$name}{default}";
         }
         $out_string .= "</p>\n";
      }
      $out_string .= "<p class=\"TitleChapterTOC\">Details:</p>\n"; 
      # $out_string .= "<p class=\"TitleChapter\">Details:</p>\n"; 
      if (exists ($globe::xtag_struct{$name}{desc_html})) {
         # $out_string .= "<p class=\"Head1TOC\">$globe::xtag_struct{$name}{desc_html}</p>\n";
         # $out_string .= "<p class=\"Head1\">$globe::xtag_struct{$name}{desc_html}</p>\n";
         $out_string .= "<p>$globe::xtag_struct{$name}{desc_html}</p>\n";
      } else {
         # $out_string .= "<p class=\"Head1TOC\">No Information.</p>\n";
         # $out_string .= "<p class=\"Head1\">No Information.</p>\n";
         $out_string .= "<p>No Information available.</p>\n";
      }

      $nav_bars = "$globe::m_info{nav_book}$globe::m_info{footer}";
      $nav_bars =~ s/$globe::m_var{define}{prev}/$globe::xtag_struct{$name}{prev}/;
      $nav_bars =~ s/$globe::m_var{define}{next}/$globe::xtag_struct{$name}{next}/;
      $out_string .= "$nav_bars\n";
      print (OUT_FILE "$out_string");
      print (OUT_FILE "</body>\n</html>\r\n");
      close (OUT_FILE);
   } # $#globe::topic_order
   
   ###############################################
   #####
   # Create a unique first topic
   #####
   if (1) { # unique first topic
      $i = 0;
      
      $name = $globe::topic_order[$i];
      $file_out = "$globe::path";
      $file_out .= "$globe::xtag_struct{$name}{html}";

      unless (open ( OUT_FILE, ">$file_out")) {
         push (@file_errors, "4 Cannot open file \"$file_out\"\n");
         print "4 Cannot open file \"$file_out\"\n";
         next OHTM;
      }
      if (1) {
         print "Creating $file_out...\n";
      }
      print (OUT_FILE "<html>\n <head>\r\n");
      # print (OUT_FILE "<title>$name</title>\r\n");
      print (OUT_FILE "$globe::m_info{header}\r\n");

      print (OUT_FILE "$globe::m_define{$def_type}[0] ");
      
      print (OUT_FILE "$globe::m_var{define}{prev}$globe::xtag_struct{$name}{prev}");
      print (OUT_FILE "$globe::m_var{define}{next}$globe::xtag_struct{$name}{next}");
      if (0) {
         print "$globe::m_var{define}{prev}$globe::xtag_struct{$name}{prev}";
         print "$globe::m_var{define}{next}$globe::xtag_struct{$name}{next}\n";

      }
      if ($i == 0) {
         print (OUT_FILE "$globe::m_var{define}{firstlast}1\#0");
      } elsif ($i == $total - 1) {
         print (OUT_FILE "$globe::m_var{define}{firstlast}0\#1");
      } else {
         print (OUT_FILE "$globe::m_var{define}{firstlast}0\#0");
      }
      
      print (OUT_FILE "$globe::m_define{$def_type}[1] \r\n");
      print (OUT_FILE "</head>\n<body>\r\n");
      #####
      # Fix up the navigation
      # GCM still to do... with previous and next...
      #####
      $nav_bars = $globe::m_info{nav_common};
      $nav_bars .= $globe::m_info{nav_book};
      $nav_bars =~ s/$globe::m_var{define}{prev}/$globe::xtag_struct{$name}{prev}/;
      $nav_bars =~ s/$globe::m_var{define}{next}/$globe::xtag_struct{$name}{next}/;
      $out_string = $nav_bars;
      $out_string .= "<h1>$globe::xtag_struct{$name}{name}</h1>\r\n";
      $out_string .= "<p align=\"center\" size=\"small\"><a href=\"\#overview\">\{Overview is below.\}</a></p>\r\n";
      
      #####
      # Go and recursively build a structure based on what is known.
      #####
      &rec_example_topic ($globe::struct_start, 0);

      $out_string .= "<p></p><p></p><p></p>\r\n";
      
      #####
      # Add more information based on what is also in the data base.
      # The first topic can't be as easily hyperlinked using the normal
      # <initag name""> targets. However, we can fake it out by defining
      # <initag name"$globe::struct_start"> in our file and tacking this
      # information to the bottom of what we created.
      #####


      $out_string .= "<a name=\"overview\"></a>\r\n";
      foreach $k (@{$globe::tag_mess{name}{$globe::struct_start}}){
         $tttemp_file = $globe::tag_mess{name}{$globe::struct_start}[$k];
         $out_string .= "$globe::tag_mess{file}{$tttemp_file}{$globe::struct_start}\n";
         if (0) {
            print "woh:file:$tttemp_file\n";
            print "woh:content:$globe::tag_mess{file}{$tttemp_file}{$globe::struct_start}\n"; 
         }
      }
      if (0) {
         print "burp\n";
         exit(1);
      }

      $nav_bars = "$globe::m_info{nav_book}$globe::m_info{footer}";
      $nav_bars =~ s/$globe::m_var{define}{prev}/$globe::xtag_struct{$name}{prev}/;
      $nav_bars =~ s/$globe::m_var{define}{next}/$globe::xtag_struct{$name}{next}/;
      $out_string .= "$nav_bars\r\n";
      print (OUT_FILE "$out_string");
      print (OUT_FILE "</body>\n</html>\r\n");
      close (OUT_FILE);
      
      ####
      # Cheap trick for the man output for the first topic
      ####
      $globe::xtag_struct{$name}{desc_html} = $out_string;
   } # unique first topic

} # write_output_html


##############################################################################
##! @brief Creates an example/sample INI file based on what is known.
##! 
##! @param None.
##!
##! @return Creates a file that ultimately is a sample INI file and
##! is fully commented. The file is tagged appropriately with XML tags. 
##! When the output_html flag is set, it will output two files:
##! one is the sample INI file with only tags;
##! the other is the corresponding HTML file with associated comments.
##! In either event, these files can be round-tripped through this tool
##! and produce equivalent output.
##! 
##! The purpose of this routine is to provide a marked-up starting point
##! for generating a fully commented/documented INI file. It probably won't
##! be used much. Later, it should end up being almost identical to the 
##! original.
##! 
##! @bug Calls another routine that ends up calling itself recursively.
##############################################################################
sub write_sample {
   local ($file_out);
   local ($file_out2);
   local ($file_out_tag);
   local ($file_out_doc);
   local ($out_string) = "";
   local ($out_head) = "";
   local ($output_html) = 0;  # Will output a HTML.
   local ($outtxt) = "";
   local ($outhtm) = "";
   local ($debug_ws) = 0;


   #####
   # Create a unique first topic
   #####
   if (1) { # unique first topic
      $i = 0;
      
      $name = $globe::topic_order[$i];
      $file_out = "$globe::path";
      $file_out .= "$globe::xtag_struct{$name}{html}";
      $file_out =~ s/\.html/\_sample\.txt/;
      $file_out2 = $file_out;
      $file_out_tag = $file_out;
      $file_out_doc = $file_out;
      $file_out2 =~ s/\.txt/\._html/;  # so it will almost be what we want.
      $file_out_tag =~ s/\_sample/\_sample\_tag/;  # just the tags
      $file_out_doc =~ s/\_sample/\_sample\_doc/;  # just the doc

      #####
      # Cheap-o error checking; not completely valid,
      # but will save at least one file from getting overwritten.
      #####
      unless (open ( OUT_FILE, ">$file_out")) {
         push (@file_errors, "5 Cannot open file \"$file_out\"\n");
         print "5a Cannot open file \"$file_out\"\n";
         return (0);
      }
      unless (open ( OUT_FILE2, ">$file_out2")) {
         push (@file_errors, "5 Cannot open file \"$file_out2\"\n");
         print "5b Cannot open file \"$file_out2\"\n";
         return (0);
      }
      unless (open ( OUT_FILE_TAG, ">$file_out_tag")) {
         push (@file_errors, "5 Cannot open file \"$file_out_tag\"\n");
         print "5c Cannot open file \"$file_out_tag\"\n";
         return (0);
      }
      unless (open ( OUT_FILE_DOC, ">$file_out_doc")) {
         push (@file_errors, "5 Cannot open file \"$file_out_doc\"\n");
         print "5d Cannot open file \"$file_out_doc\"\n";
         return (0);
      }
      if (1) {
         print "Creating $file_out...\n";
         if ($output_html) {
            print "Creating $file_out2...\n";
         }
      }
      &rec_sample ($globe::struct_start, 0, $output_html);
      $outtxt = "\/\/\/\/\/\/\/\/\/\/\r\n";
      $outtxt .= "\/\/ This is a commented INI file. \r\n";
      $outtxt .= "\/\/ Follow XML tag references to find comments. \r\n";
      $outtxt .= "\/\/\/\/\/\/\/\/\/\/\r\n";
      $outtxt .= $out_string;
      print (OUT_FILE_TAG $out_string);
      
      #####
      # This will overwrite $out_head with more complete information
      # as it was originally.
      #####
      foreach $tag (sort keys %{$globe::tag_mess{name}}) {
         if ($debug_ws) {
            print "ws:$tag\n";
         }
         foreach $i (@{$globe::tag_mess{name}{$tag}}){
            if ((0) && ($debug_ws)) {
               print "ws:\"$tag\":$globe::tag_mess{name}{$tag}[$i]\n";
            }
            $file = $globe::tag_mess{name}{$tag}[$i];
            if ($debug_ws) {
               # print this is our information
               print "\"$file\":\"$tag\":\n$globe::tag_mess{file}{$file}{$tag}\n";
            }
            if ($output_html) {
               $out_head .= "<p class=\"Head1\">$tag</p>\r\n";
               $out_head .= "<$tag_xml name=\"";
            } else {
               $out_head .= "\/\/ <$tag_xml name=\"";
            }
            $out_head .= "$tag";
            # $out_head .= "\">\r\n";
            $out_head .= "\">";
            # This is the orginal comment.
            $ttt_html = $globe::tag_mess{file}{$file}{$tag};
            $ttt_html =~ s/\r//g; # get rid of all windows line feeds
            if ($output_html) {
               $ttt_html =~ s/\n/\r\n/g; # replace all returns with something for windows
            } else { 
               $ttt_html =~ s/\n/\r\n\/\/ /g; # replace all returns with something for windows
            }
         
            # $out_head .= "\/\/ $ttt_html";
            $out_head .= "$ttt_html";
            $out_head .= "</$tag_xml>\r\n\r\n";
         }
      }
      print (OUT_FILE_DOC $out_head);

      #######
      if ($output_html) {
         $outhtm = "<html>\r\n";
         $outhtm .= "<!-- begin voy_header -->\r\n";
         $outhtm .= "   <meta http-equiv=\"Content-Type\" content=\"text/html;charset=iso-8859-1\">\r\n";
         $outhtm .= "   <title>$globe::xtag_struct{$name}{name}</title>\r\n";
         $outhtm .= "   <link rel=\"stylesheet\" href=\"voyant_fm.css\" type=\"text/css\">\r\n";
         $outhtm .= "   <body link=\"#0000ff\" alink=\"#CC0000\" vlink=\"#0000ff\">\r\n";
         $outhtm .= "<!-- end voy_header -->\r\n";
         $outhtm .= "<\/head>\r\n";
         $outhtm .= "<body>\r\n";
      } else {
         $outtxt .= "\r\n\/\/\/\/\/\/\/\/\/\/\r\n";
         $outtxt .= "\/\/ HTML comments to the INI file entries.\r\n";
         $outtxt .= "\/\/\/\/\/\/\/\/\/\/\r\n";
      }
      if ($output_html) {
         $outhtm .= $out_head;
         $outhtm .= "<\/body>\r\n";
         $outhtm .= "<\/html>\r\n";
         print (OUT_FILE2 $outhtm);
      } else {
         $outtxt .= $out_head;
      }
      print (OUT_FILE $outtxt);

      close (OUT_FILE);
      close (OUT_FILE2);
      close (OUT_FILE_TAG);
      close (OUT_FILE_DOC);
   } # unique first topic

} # write_sample


##############################################################################
##! @brief Creates a sample INI without any HTML or hyperlinking.
##! 
##! @param seed Starting point in the structure.
##! @param level Starting level in the structure. Used to control indenting in
##! the output.
##!
##! @retval out_string is a global variable holding the desired output. This
##! output is XML tagged.
##! 
##! The purpose of this is to generate a sample INI file that can be used
##! to copy information from other sources. It creates it with the information
##! that it knows. It will require editing the first time. Later, this file
##! won't be used except maybe to compare the input with the output.
##! 
##! @bug This is called recursively.
##! This does not catch all xml tags that were used.
# &rec_sample ($globe::struct_start, 0);
##############################################################################
sub rec_sample {
   local ($seed) = $_[0];
   local ($level) = $_[1];
   local ($html_out) = $_[2];
   local ($debug_rs) = 0;
   
   if (0) {
      print "seed: \"$seed\"\n";
   }
   #####
   # Creates a comment block to topic
   #####
   $temp = $globe::xtag_struct{$seed}{name};
   $temp =~ s/\[//;
   $temp =~ s/\]//;
   if (exists ($globe::xtag_struct{$seed}{child})) {
      # means we need to trace down into it
      if ($globe::struct_start !~ /$seed/) {
         for ($i=1; $i<=$level; $i++){
            $out_string .= "   ";
         }
         $out_string .= "$globe::xtag_struct{$seed}{name}";
         $out_string .= "   \/\/ ";
         if (exists ($globe::xtag_struct{$seed}{comment})) {
            $out_string .= "$globe::xtag_struct{$seed}{comment}";
         } else {
            $out_string .= "<$tag_xml target=\"";
            $out_string .= "$temp";
            $out_string .= "\">";
         }
         $out_string .= "\r\n";
         $level++;
      }
      foreach $child (@{$globe::xtag_struct{$seed}{child}}) { 
         &rec_sample ($child, $level, $html_out);
      }
      if ($globe::struct_start !~ /$seed/) {
         $level--;
         for ($i=1; $i<=$level; $i++){
            $out_string .= "   ";
         }
         $out_string .= "\[\]";
         $out_string .= "   \/\/ $globe::xtag_struct{$seed}{name}";
         $out_string .= "\r\n";
      }
   } else {
      # means we're at a child level with no children
      # can print out and return
      if ($globe::struct_start !~ /$seed/) {
         for ($i=1; $i<=$level; $i++){
            $out_string .= "   ";
         }
         #####
         # Puts in hyperlink to topic
         #####
         if (exists ($globe::xtag_struct{$seed}{default})) {
            $out_string .= "$globe::xtag_struct{$seed}{name}";
            $out_string .= " $globe::xtag_struct{$seed}{plus}\=";
            $out_string .= " $globe::xtag_struct{$seed}{default}";
         } else {
            $out_string .= "$globe::xtag_struct{$seed}{name}";
         }
         $out_string .= "   \/\/ ";
         if (exists ($globe::xtag_struct{$seed}{comment})) {
            $out_string .= "$globe::xtag_struct{$seed}{comment}";
            if ($globe::xtag_struct{$seed}{comment} !~ /$tag_xml/) {
               # Let's put in an empty tag anyway.
               $out_string .= "<$tag_xml target=\"";
               $out_string .= "$temp";
               $out_string .= "\">";
               # 08/07/2002 Add the name to the structure to be later output
               push (@{$globe::tag_mess{name}{$temp}}, $org_ini);
               $globe::tag_mess{file}{$org_ini}{$temp} .= "";
            }
         } else {
            $out_string .= "<$tag_xml target=\"";
            $out_string .= "$temp";
            $out_string .= "\">";
            # 08/07/2002 Add the name to the structure to be later output
            push (@{$globe::tag_mess{name}{$temp}}, $org_ini);
            $globe::tag_mess{file}{$org_ini}{$temp} .= "";
         }
         $out_string .= "\r\n";
         # for when a section is defined but has no children.
         if ($globe::xtag_struct{$seed}{name} =~ /^\[/) {
            if ($globe::struct_start !~ /$seed/) {
               # $level--;
               for ($i=1; $i<=$level; $i++){
                  $out_string .= "   ";
               }
               $out_string .= "\[\]";
               $out_string .= "   \/\/ $globe::xtag_struct{$seed}{name}";
               $out_string .= "\r\n";
            }
         } # when section has no children
         
      }
   } 
} # rec_sample





##############################################################################
##! @brief Creates man pages for each hash topic.
##! 
##! @param seed Starting point in the structure.
##!
##! @return A file that represents the equivalent man information for a
##! hash item.
##! 
##! The reason this is called recursively is that the topic order isn't
##! important. Mostly that the man output is generated.
##! 
##! @bug This is called recursively.
##############################################################################
sub rec_write_man {
   local ($seed) = $_[0];
   local ($debug_rwm) = 0;
   
   if ($debug_rwm) {
      print "rwm:seed: \"$seed\"\n";
   }
   &write_output_man($seed);
   if (exists ($globe::xtag_struct{$seed}{child})) {
      # means we need to trace down into it
      if ($debug_rwm) {
         print "rwm:section:$globe::xtag_struct{$seed}{name}\n";
      }
      foreach $child (@{$globe::xtag_struct{$seed}{child}}) { 
         # print "name:  \"$hname\"\n";
         &rec_write_man($child);
      }
   } else {
      # means we're at a child level with no children
      # can print out and return
      if ($debug_rwm) {
         print "rwm:final child: \"$globe::xtag_struct{$seed}{name}\" with  \"$globe::xtag_struct{$seed}{parent}\" \n";
      }
   }
} # rec_write_man

##############################################################################
##! @fn int write_output_man
##! @brief Formats and outputs a man page for the given key into the 
##! the xtag_struct hash.
##! 
##! @param hname the key into the hash.
##!
##! @retval out_string is a global variable holding the desired output. 
##!
##!
##! @ingroup tp_xhelp
##       $globe::xtag_struct{$hname}{name} = actual item name
##       $globe::xtag_struct{$hnamee}{parent} = owning section
##       $globe::xtag_struct{$hname}{child}[] = array of associated children
##       $globe::xtag_struct{$hname}{comment} = comment flag
##       $globe::xtag_struct{$hname}{desc_html} = description in html
##       $globe::xtag_struct{$hname}{desc_man} = description in man
##       &write_output_man($globe::topic_order[$i], $mpath_out, $man_extension);
##############################################################################
sub write_output_man {
   local ($hname) = $_[0];
   local ($trace_parent);
   local ($file_out);
   local ($debug_wom) = 0;
   
   if (0) { # old
      $file_out = $globe::xtag_struct{$hname}{html};
      $file_out =~ s/\.html/\.$man_extension/;
      $file_out = "$mpath_out$file_out";
   }
   if (1) { # new
      $file_out = $hname;
      @bogus = split (/\-/, $file_out);
      $file_out = pop (@bogus);   # we only want the last element
      if ($file_out !~ /[\w]+/) {
         $file_out = $globe::xtag_struct{$hname}{html};
         $file_out =~ s/\.html//;
      }
      $temp_name = $file_out;
      $file_out .= "\.$man_extension";
      $file_out = "$mpath_out$file_out";
   }

   $trace_parent = &parent_trace ($hname, 0);  # the 0 means no html output

   unless (open ( OUT_FILE, ">$file_out")) {
      push (@file_errors, "6 Cannot open file \"$file_out\"\n");
      print "6 Cannot open file \"$file_out\"\n";
      return (0);
   }
   if ((1) || ($debug_wom)) {
      print "Creating $file_out...\n";
      # print "Again $date\n";
   }
   # print (OUT_FILE ".TH \"$globe::xtag_struct{$hname}{name}\" $man_extension $date  \n");
   print (OUT_FILE ".TH \"$temp_name\" $man_extension $date  \n");
   # print (OUT_FILE ".SH NAME\n$globe::xtag_struct{$hname}{name} \- ");
   print (OUT_FILE ".SH NAME\n$temp_name \- ");
   local ($temp) = $globe::xtag_struct{$hname}{desc_man};
   if ((0) || ($debug_wom)) {
      print "=== getting one-liner:\n$temp\n====\n";
   }
   $temp =~ s/\.LP//g;  # get rid of previous text.
   $temp =~ s/\n/ /g;  # get rid of all carriage returns.
   @ttemp = split (/\.[\s]+/, $temp, 2);  # $ttemp[0] is the one line summary.
   $ttemp[0] =~ s/\\fI//g;  # get rid of previous text.
   $ttemp[0] =~ s/\\fB//g;  # get rid of previous text.
   $ttemp[0] =~ s/\\fP//g;  # get rid of previous text.
   $ttemp[0] =~ s/^[\s]+//;  # get rid of previous text.
   if ((0) || ($debug_wom)) {
      print "=== one-liner:\n$ttemp[0].\n====\n";
   }
   print (OUT_FILE "$ttemp[0].\n\n");
   
   print (OUT_FILE ".SH SYNOPSIS\n\.B $trace_parent");

   if (exists ($globe::xtag_struct{$hname}{default})) {
       print (OUT_FILE " $globe::xtag_struct{$hname}{plus}\=");
       print (OUT_FILE " $globe::xtag_struct{$hname}{default}");
   }
   print (OUT_FILE "\n");

   print (OUT_FILE ".SH DESCRIPTION \n");
   print (OUT_FILE "$globe::xtag_struct{$hname}{desc_man}\n");
   # print (OUT_FILE ".SH SEE ALSO \"$globe::xtag_struct{$globe::struct_start}{name}\"\n");
   print (OUT_FILE ".LP\n \n.LP\n$copyright\n");

   close (OUT_FILE);
   
}

##############################################################################
##! @brief Creates a MAN descriptions for the hash elements.
##! 
##! @param None.
##!
##! @retval Creates xtag_struct{$name}{desc_man} which is just a copy
##! of the xtag_struct{$name}{comment_full} without the HTML and marked up
##! for MAN.
##! 
##! @bug This does not work yet. Needs to replace HTML with MAN mark-up.
##############################################################################
sub generate_desc_man {
   local ($debug_gdm) = 0;
   local ($b1);  # before html tag
   local ($a1);  # after html tag
   local ($p1);  # inbetween html < >
   local ($nada) = "_n_a_d_a_";
   local (@start_tag)     = ("<br",    "<p",     "<font" ,  "<i",   "<b",  "<ul",  "<ol",  "<li",      "<" );
   local (@end_tag)       = ($nada,    "<\/p",   "<\/font", "</i",  "</b", "</ul", "</ol", "</li",     $nada);
   local (@replace_start) = ("\.LP\n", "\.LP\n", "\\fI",    "\\fI", "\\fB", "",    "",     "\.HP\n\- ", "" );
   local (@replace_end)   = ("",       "",       "\\fR",    "\\fR", "\\fR", "",    "",     "\n\.RE\n", "" );
   local ($close_tag)     = ">";
   
   if ( ($#start_tag < $#end_tag) || ($#start_tag > $#end_tag) 
     || ($#start_tag < $#replace_start) || ($#start_tag > $#replace_start)
     || ($#start_tag < $#replace_end) || ($#start_tag > $#replace_end)) {
      print "gdm:start_tag, end_tag, and replace_tag arrays mush be of equal length.\n";
      foreach $tag (@start_tag) {
         print "start:$tag\n";
      }
      foreach $tag (@end_tag) {
         print "end:$tag\n";
      }
      foreach $tag (@replace_tag) {
         print "replace:$tag\n";
      }
      return (0);
   }
   
   for ($i=0; $i <= $#globe::topic_order; $i++) {
      # print "$i:$globe::topic_order[$i]\n";
      $shrink = $globe::xtag_struct{$globe::topic_order[$i]}{comment_full};
      if ((1) && ($debug_gdm)) {
         print "IN>>>>\n$shrink";
      }
      for ($k=0; $k <= $#start_tag; $k++) {
         if ($shrink =~ /$start_tag[$k]/) {
            while ($shrink =~ /$start_tag[$k]/) {
               ($b1, $p1, $a1) = &globe::get_tag_chunk( $shrink, 
                    $start_tag[$k], $close_tag, 0);
               if ((0) && ($debug_gdm)) {
                  print "start_tag=>\"$start_tag[$k]\"\n";
                  print "close_tag=>\"$close_tag\"\n";
                  print "b1=>$b1\np1=>$p1\na1=>$a1\n";
               }
               if ($a1 =~ $end_tag[$k]) {
                  ($b2, $p2, $a2) = &globe::get_tag_chunk( $a1, 
                       $end_tag[$k], $close_tag, 0);
                  $shrink = join ("", $b1, $replace_start[$k], $b2, $replace_end[$k], $a2);
               } else {
                  $shrink = join ("", $b1, $replace_start[$k], $a1, $replace_end[$k]);
               }
               if ((0) && ($debug_gdm)) {
                  print "end_tag=>\"$end_tag[$k]\"\n";
                  print "close_tag=>\"$close_tag\"\n";
                  print "b2=>$b2\np2=>$p2\na2=>$a2\n";
               }
            }  # while shrink
         }  # if $start_tag
      } # for $k $#start_tag
      for ($k=0; $k <= $#end_tag; $k++) {
         # Make sure that there aren't any spurious end tags at that level
         if ($shrink =~ /$end_tag[$k]/) {
            while ($shrink =~ /$end_tag[$k]/) {
               ($b1, $p1, $a1) = &globe::get_tag_chunk( $shrink, 
                    $end_tag[$k], $close_tag, 0);
               $shrink = join ("", $b1, $a1);
            }  # while shrink
         }  # if $end_tag
      } # for $k $#end_tag
      if ((1) && ($debug_gdm)) {
         print "\nOUT<<<<<\n$shrink\n";
      }
      $shrink =~ s/\r//g;  # strip out Windows
      $globe::xtag_struct{$globe::topic_order[$i]}{desc_man} = $shrink;
   }
   # exit(1);
   return (1);
} # generate_desc_man




##############################################################################
##! @brief Generates an index file in the format needed by other routines.
##! 
##! @param None.
##!
##! @return The index file.
##!
##! This steps through each item in the globe::topic_order array, which
##! contains ordered keys into the xtag_struct hash. We could have implemented
##! this using a recursive call, but this was more straightforward.
##!
##! @bug In most cases, this routine will be turned off before it is called.
##! The reason is that the HTML file need to be processed by voyant_nav.pl 
##! in order to resolve things in the navigation, such as the button with the
##! manual title and other things from the voyant_master_nav.html. 
##!
##! As such, the index file is generated again anyway. Because the first topic
##! is now fully tagged with target index entries, they will be included in 
##! that version and not this version here.
##!
##! @ingroup tp_xhelp
##      $name = name associated with number
##       $globe::xtag_struct{$name}{prev} = previous html file
##       $globe::xtag_struct{$name}{next} = next html file
##       $globe::xtag_struct{$name}{html} = associated html file
##############################################################################
sub generate_index {
      
   @path_piece = split (/\//, $globe::path);
   $strip_p = pop (@path_piece);
   $strip_p = pop (@path_piece);
   $strip_p .= "\/";

   if (1) {
      print "Topic ordering\n=====\n";
      for ($i=0; $i <= $#globe::topic_order; $i++) {
         print "$i:$globe::topic_order[$i]\n";
      }
      # exit(1);
   }



   # foreach $name (sort keys %{globe::xtag_struct}){
   for ($i=0; $i <= $#globe::topic_order; $i++) {
      $name = $globe::topic_order[$i];
      # $out_string = $name;
      $out_string = $globe::xtag_struct{$name}{name};
      $out_string .= $globe::word_url_boundary;
      $out_string .= "<a href=\"";
      # $out_string .= $strip_p;
      $out_string .= $globe::xtag_struct{$name}{html};
      $out_string .= "\" target=\"basefrm\">";
      push (@globe::index_info, $out_string);
   }
   $file_out = "$globe::path";
   $file_out .= "_index_list";
   unless (open ( OUT_INDEX_LIST, ">$file_out")) {
      push (@file_errors, "Cannot open file \"$file_out\"\n");
      print "Cannot open file \"$file_out\"\n";
   }
   if (1) { # 
      for ($i=0; $i<@globe::index_info; $i++){
         # print "$i = $globe::index_info[$i]\n";
         print (OUT_INDEX_LIST "$globe::index_info[$i]\n");
      }
   }
   close (OUT_INDEX_LIST);
   if (1) {
      print "Creating $file_out...\n";
   }

} # generate_index


##############################################################################
##! @brief Generates a script file used for the Java applet TOC.
##! 
##! @param seed Starting point in the structure.
##! @param level Starting level in the structure. Used to control indenting in
##! the output.
##!
##! @retval A single tree script file that meets the criteria of the system.
##!
##! This steps through the xtag_struct structure and creates the appropriate
##! entry into the script file.
##!
##! @bug This is called recursively.
##       $name = name associated with number
##       $globe::xtag_struct{$name}{prev} = previous html file
##       $globe::xtag_struct{$name}{next} = next html file
##       $globe::xtag_struct{$name}{html} = associated html file
##       $globe::xtag_struct{$name}{severity} = severity associated with number
##       $globe::xtag_struct{$name}{message} = message associated with number
##       $globe::xtag_struct{$name}{comment} = comment associated with number
## 
##############################################################################
sub rec_generate_script {
   local ($seed) = $_[0];
   local ($level) = $_[1];
   
   if (0) {
      print "seed: \"$seed\"\n";
   }
   if (exists ($globe::xtag_struct{$seed}{child})) {
      # means we need to trace down into it
      if ($globe::struct_start !~ /$seed/) {
         $out_string .= "Item level=$level ";
         # $out_string .= "image=nav_doc.gif ";
         $out_string .= "image=nav_folderclosed.gif ";
         # $out_string .= "url=$strip_p$globe::xtag_struct{$seed}{html},basefrm ";
         $out_string .= "url=$globe::xtag_struct{$seed}{html},basefrm ";
         $out_string .= "selected=YES ";
         $out_string .= "text=$globe::xtag_struct{$seed}{name}";
         $out_string .= "\r\n";
         $level++;
      }
      foreach $child (@{$globe::xtag_struct{$seed}{child}}) { 
         &rec_generate_script ($child, $level);
      }
      if ($globe::struct_start !~ /$seed/) {
         $level--;
         if (0) {
            $out_string .= "Item level=$level ";
            $out_string .= "text=\[\]";
            $out_string .= "   \/\/ $globe::xtag_struct{$seed}{name}";
            $out_string .= "\r\n";
         }
      }
   } else {
      # means we're at a child level with no children
      # can print out and return
      if ($globe::struct_start !~ /$seed/) {
         $out_string .= "Item level=$level ";
         $out_string .= "image=nav_doc.gif ";
         # $out_string .= "url=$strip_p$globe::xtag_struct{$seed}{html},basefrm ";
         $out_string .= "url=$globe::xtag_struct{$seed}{html},basefrm ";
         $out_string .= "selected=YES ";
         $out_string .= "text=$globe::xtag_struct{$seed}{name}";
         $out_string .= "\r\n";
      }
   } 
} # rec_generate_script


##############################################################################
##! @fn int using_scheck
##! @brief What to do when no arguments are given.
##! @param None
##! @return None
##! ini_html_gen.pl ../cms_middleware/doc_publish/book_cms_logging/ 
##!    ../../zz_stage/bridgegroup/middleware/supptsys/messages.txt 
##!    ../cms_middleware/voyant_master_nav.html
##!    "Middleware "
##! 
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub using_scheck {
   print "============  Syntax  ==================================\n";
   print "ini_html_gen.pl <output directory> <input message file path \& name> ";
   print "<input master HTML file path \& names> [<title>]\n\n";
   print "- <output directory>  ";
   print "\n\n";
   print "- <input ini file path \& name> is file path and name input INI ";
   print "file that came from development.\n\n";
   print "- <input master HTML file path \& names> is file path and name of the master ";
   print "HTML file. \n";
   print "- [<title>] is an optional title that will be inserted into ";
   print "the TOC script for these messages; make sure it is followed by a ";
   print "space!!!\n";
   print "- [source] is an optional header file with INI tags to ";
   print "include in the output or check for.\n";
   print "\n";
   print "============  Terminate  ==================================\n";
   print "Terminating ini_html_gen.pl without doing anything.\n";
   return(1);
}




##############################################################################
##! @fn int END
##! @brief Code to execute when terminated nicely.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub END {
   undef ($_file_list); # "_file_list";
   undef ($in_file ); #  "";
   undef ($root_path ); #  "./";
   undef (%file_hash);

  
   ##############################################################################
   ## Memory clean-up.
   ## Perform this if it was defined.
   ##############################################################################
   if ($no_scope_file > 0){
      &globe::memory_clean_up();
   }
   
   print "\n============  Finished ini_html_gen.pl ==================================\n";
} # END
