#!/usr/local/bin/perl
package scheck;
##############################################################################
##! @file
##! @brief Generates one or more HTML files based upon what is found in
##! the messages.txt file that contains all of the log messages.
##! 
##! log_html_gen.pl ../cms_middleware/doc_publish/book_cms_logging/ 
##!    ../../zz_stage/bridgegroup/middleware/supptsys/messages.txt 
##!    ../cms_middleware/voyant_master_nav.html
##!    "Middleware "
##!
##! @ingroup tp_tools tp_xhelp
##!
##! @author Glenn C. Maxey
##
##    $Id: log_html_gen.pl,v 1.8 2002/11/20 15:23:04 gmaxe Exp $
##
## Copyright 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Revision Information:
##
## $Log: log_html_gen.pl,v $
## Revision 1.8  2002/11/20 15:23:04  gmaxe
## Added exit codes so that wrapper scripts can catch errors properly.
##
## Revision 1.7  2002/11/20 01:49:12  gmaxe
## Minor changes to exit codes.
##
## Revision 1.6  2002/08/07 01:27:13  gmaxe
## Changed the {comment} to a {comment_full} where appropriate.
## Enhanced the ini gen as appropriate to get the desired output.
##
## Revision 1.5  2002/08/03 00:58:25  gmaxe
## Updates before the weekend. Now it parses target files better;
## had problems with scoping for variables.
##
## Revision 1.4  2002/08/02 00:35:07  gmaxe
## Created a new file for parsing INI files. It is based on
## log_html_gen.pl; common routines were moved from that file
## into globe.pm for better sharing.
##
## Revision 1.3  2002/07/02 23:10:53  gmaxe
## Tweaked to get rid of extra expanding line feeds in voyant_nav.pl.
## Added topper.html and other files to skip list in globe.pm.
## Enhanced log_html_gen.gl to handle new format of message.txt;
## it now has a flatter data structure and no enums numbers.
##
## Revision 1.2  2002/04/22 17:53:33  gmaxe
## Minor enhancements to sorting order and TOC display.
## Also support an additional argument for a title.
##
## Revision 1.1  2002/04/20 11:31:22  gmaxe
## Added support for a tool that generates HTML for the log
## messages. This is one step away from a tool that does
## XML parsing to generate HTML. Some comments are still
## lacking.
##
## Revision 1.1  2002/04/12 22:52:25  gmaxe
## New tool to check on whether or not all SAPI commands are present.
## Uses the files generated by the vproto_gen.pl tool.
##
##
##############################################################################

##############################################################################
##! @fn int BEGIN
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub BEGIN {
   print "\n============  Starting log_html_gen.pl ==================================\n";
   $_file_list = "_file_list";
   $in_file = "";
   $root_path = "./";
   $scope_pm = "globe.pm";
   $tag_xml = "logmsg";
   $no_scope_file = 0;
   $arg_cnt=0;
   
   push (@INC, `pwd`);
   push (@INC, '../perl');
   if (0){
      print (@INC, "\n");
   }
   

   
   # Get scope files if there is one.
   if (1) { # used
      # $scope_pm = @ARGV[$arg_cnt];
      #####
      # All global variables are defined in the following file
      #####
      
      unless (open ( IN_LIST, $scope_pm)) {
         unless (open ( IN_LIST, "../perl/$scope_pm")) {
            push (@file_errors, "Cannot open file \"$scope_pm\"");
            $no_scope_file++;
         }
      }
      close (IN_LIST);
      push (@INC, $scope_pm);
      
      if (!@file_errors) {
         #####
         # All global variables are defined in the following file
         #####
         require $scope_pm;
   
         if (&globe::declare_variables()) {
            print "Variables initialized from $scope_pm.\n";
         } else {
            push (@file_errors, "Could not initialize variables from $scope_pm.");
         }
      } # if not @file_errors
   } else {
      push (@file_errors, "Need to provide a scope file as first argument.");
   } # if 1 or more arguments

 
 if (0) { # commented out because not used
   # Get scope files if there is one.
   if (@ARGV > $arg_cnt) {
      $scope_pm = @ARGV[$arg_cnt];
      
      unless (open ( IN_LIST, $scope_pm)) {
         push (@file_errors, "Cannot open file \"$scope_pm\"");
         $no_scope_file++;
      }
      close (IN_LIST);
      push (@INC, $scope_pm);
      
      if (!@file_errors) {
         #####
         # All global variables are defined in the following file
         #####
         require $scope_pm;
   
         if (&globe::declare_variables()) {
            print "Variables initialized from $scope_pm.\n";
         } else {
            push (@file_errors, "Could not initialize variables from $scope_pm.");
         }
      } # if not @file_errors
   } else {
      push (@file_errors, "Need to provide a scope file as first argument.");
   } # if 1 or more arguments
   $arg_cnt++;
 } # commented out because not used  

   if (!(exists ($globe::x_define{$tag_xml}))) {
      push (@file_errors, "\"$tag_xml\" does not exist in the global definitions (\"$scope_pm\").");
   }

   # Get path to code files 
   if (@ARGV > $arg_cnt) {
      $root_path = @ARGV[$arg_cnt];
      $globe::path = @ARGV[$arg_cnt];
      if ($root_path =~ /\/$/) {
         print "The path specified is $root_path\n";
      } else {
         push (@file_errors, "The input argument \"$root_path\" requires a forward slash (\/) at the end.\n");
      }
   } # if 1 or more arguments
   $arg_cnt++;


   ##! log_html_gen.pl ../cms_middleware/doc_publish/book_cms_logging/ ../../zz_stage/bridgegroup/middleware/supptsys/messages.txt ../cms_middleware/voyant_master_nav.html

   if (@ARGV > $arg_cnt) {
      $org_msg = @ARGV[$arg_cnt];
      print "The message input file is $org_msg \n";
   } else {
      push (@file_errors, "Need to have the input message file.");
   } # if 3 or more arguments
   $arg_cnt++;

   if (@ARGV > $arg_cnt) {
      $globe::master_nav_file = @ARGV[$arg_cnt];
      print "The master HTML input file is $globe::master_nav_file \n";
      unless (open ( IN_MASTER, $globe::master_nav_file)) {
         push (@file_errors, "Cannot open file \"$globe::master_nav_file\"\n");
      }
   } else {
      push (@file_errors, "Need to have the input master HTML file.");
   } # if 3 or more arguments
   $arg_cnt++;

   if (@ARGV > $arg_cnt) {
      $org_title = @ARGV[$arg_cnt];
      print "The title for output is $org_title \n";
   } # if 3 or more arguments
   $arg_cnt++;

} # BEGIN
   
   
   

##############################################################################
##! @fn int main
##! @brief The main program.
##!
##! @param file_errors If file errors are encountered before the main
##! routine is hit, it prints them out and exits. 
##!
##! @return None.
##!
##! If the needed data structures in $scope_pm have not been enabled,
##! this program errors out immediately.
##! 
##! Controls the program flow. It creates a temporary file with the
##! error results.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
# sub main {
{
   ##############################################################################
   ## Start of Main Program
   ##############################################################################
   if (@file_errors) {
      # Makes no sense to go on if input parameters are off.
      print "\n============  Summary of errors ==================================\n";
      for ($i=0; $i<@file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
      &scheck::using_scheck();
      exit(1);
   }


   if ((1) && (!(&master_file))){
      print "Master file failed some things.\n";
   } 
   
   if ((1) && (!(&globe::read_code_n_comment($org_msg, 1)))){
      # The 1 means that we care about separating comments in the
      # message.txt file from the lines that represent a log message.
      print "Message file failed some things.\n";
   } 
   # At this point, $globe::comment_file was created and populated,
   # as was $globe::entire_file.

   if ((1) && (!(&globe::process_comments_from_file($org_msg, $tag_xml)))){
      print "1 Processing comments from file failed some things.\n";
   } 
   # At this point, $globe::comment_file has been parsed and
   # can be overwritten.
   

   if ((1) && (!(&create_data_structure))){
      print "Creating data structure failed some things.\n";
   } 
   # At this point, the $globe::entire_file, which had the
   # log messages in it, has been processed into a data structure.
   # Hence, $globe::entire_file is no longer needed.
   
   
   # At this point it is safe to call globe::read_code_n_comment again with
   # another file, because $globe::comment_file and $globe::entire_file
   # have both been placed into the data structure already.
   # Deep down in the next routine it does get called.
   
   if ((1) && (!(&globe::xtag_into_mess_structure($org_msg, $tag_xml)))){
      print "Getting logmsg into message structure failed some things.\n";
   } 
   if (0) {
      print "Searching1 for stuff...\n";
      foreach $f (sort keys %{$globe::tag_mess{file}}) {
         foreach $t (sort keys %{$globe::tag_mess{file}{$f}}){
            print "\"$f\":\"$t\":\n$globe::tag_mess{file}{$f}{$t}\n";
         }
      }
      foreach $t (sort keys %{$globe::tag_mess{name}}) {
         foreach $i (@{$globe::tag_mess{name}{$t}}){
            print "\"$t\":$globe::tag_mess{name}{$t}[$i]\n";
         }
      }
      print "Searching2 for stuff...\n";
      exit(1);
   }
 
   
   if (1) {
      &write_output_html();
   }
   if (0) {
      &generate_index;
   }

   if (1) {
      &generate_script;
   }
   
   
   ##############################################################################
   ## Spit out errors
   ##############################################################################
   if (@file_errors) {
      print "\n============  Summary of errors ==================================\n";
      for ($i=0; $i<@file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
   }

   ##############################################################################
   ## End of most of work
   ##############################################################################
   exit(0);
}


##############################################################################
##! @brief Reads in the entire master HTML file
##! 
##! @bug Fills up $globe::master variables.
##############################################################################
sub master_file {
   local ($before) = "";
   local ($after) = "";
   local ($piece) = "";

   # Get the master definitions
   while (<IN_MASTER>){    # entire master file into memory.
      $globe::master_nav .= $_;
   }

   if (&globe::get_master_nav_info() =~ 1){
      print "Master Navigation file \"$globe::master_nav_file\" has definition errors.\n";
      if (@file_errors) {
         print "\n============  Summary of errors ==================================\n";
         for ($i=0; $i<@file_errors; $i++){
            print "$i = $file_errors[$i]\n";
         }
      }
      return (0);
   } else {
     # Create special navigation in the footer that repeats buttons.
     $globe::voy_footer_fm = "$globe::m_define{footer}[0]\n<br><br>\n";
     $def_type = "nav_book";
     ($before, $piece, $after) = &globe::get_tag_chunk( $globe::m_info{$def_type},
                                 $globe::m_define{$def_type}[0], 
                                 $globe::m_define{$def_type}[1], 
                                 $very_critical);

     $globe::voy_footer_fm .= "$piece\n";
     
     $def_type = "footer";
     ($before, $piece, $after) = &globe::get_tag_chunk( $globe::m_info{$def_type},
                                 $globe::m_define{$def_type}[0], 
                                 $globe::m_define{$def_type}[1], 
                                 $very_critical);
     
     $globe::voy_footer_fm .= "$piece\n";
     $globe::voy_footer_fm .= "$globe::m_define{footer}[1]\n";

     
     $globe::master_order_key = @globe::master_order;  # sets the default value
     for ($i=0; $i <= $#globe::master_order -1; $i++){
        # does not go to the last one, because that is the default, already set
        if ($globe::path =~ /$globe::master_order[$i]/){
           $globe::master_order_key = $i;
           # print "$i = $globe::master_order[$globe::master_order_key]\n";
        }
     }
     if (0) {
        print "=== Master Header ==\n$globe::m_info{header}\n";
        print "=== Master Common Top ==\n$globe::m_info{nav_common}\n";
        print "=== Master Doxygen ==\n$globe::m_info{nav_doxygen}\n";
        print "=== Master FM Book ==\n$globe::m_info{nav_book}\n";
        print "=== Master Footer ==\n$globe::m_info{footer}\n";
        print "=== Master Dox Footer ==\n$globe::voy_footer_dox\n";
        print "=== Master FM Footer ==\n$globe::voy_footer_fm\n";
        print "=== Master variables ==\n$globe::m_info{variable}\n";
        # return (0);
     }
   }
   return (1);
} # master file


##############################################################################
##! @brief Creates a data structure from each line in the message file.
##! 
##!       $name = name associated with number
##!       $globe::xtag_struct{$name}{severity} = severity associated with number
##!       $globe::xtag_struct{$name}{message} = message associated with number
##!       $globe::xtag_struct{$name}{comment} = comment associated with number
##! 
##! @bug 
##############################################################################
sub create_data_structure {

   undef (%globe::xtag_struct);
   
   @_input_line = split (/\n/, $globe::entire_file);
   
   $j=0;
   LMSG: foreach $line (@_input_line){
      @_line_field = split (/\|/, $line);
      if ($_line_field[0] =~ /enumstart/i) {
         next LMSG;
      }
      if (0) {
         print "\n$j: Now let's output this Guy =====\n"; 
         $i = 0;
         foreach $field (@_line_field) {
            print "a$i = $field; ";
            $i++;
         }
         print "\n============\n";
         for ($i=0; $i<=$#_line_field; $i++) {
            print "b$i = $_line_field[$i]; ";
         }
         print "\n============\n";
      }
      $j++;
      
     
      #####
      # 07/02/2002 GCM message file changed format;
      # field[0] is now the name. So the {name} goes away, as does {num}.
      #####
      #####
      # Create the data structure for messages.
      #####
      # print "$_line_field[0]\n";
      $globe::xtag_struct{$_line_field[0]} = $_line_field[0];
      $globe::xtag_struct{$_line_field[0]}{severity} = $_line_field[1];
      $globe::xtag_struct{$_line_field[0]}{message} = $_line_field[2];
      $globe::xtag_struct{$_line_field[0]}{comment} = $_line_field[3];
      # 07/02/2002
   }
   
   if (0) {
      print "Testing the output...\n";
      $j=0;
      foreach $i (sort keys %{globe::xtag_struct}) {
         if (1) {
            print "==== name $i =====\n";
            foreach $item (keys %{$globe::xtag_struct{$i}}){
               print "   $i: $item: $globe::xtag_struct{$i}{$item}\n";
            }
         }
         $j++;
      }
      print "Done Testing the output...\n";
      # exit(1);
   }
   
   return (1);
 
 
} # create_data_structure



##############################################################################
##! @fn int write_output_html
##! @brief Sorts the data structure by name and creates the html files.
##!
##! @param file_out The name of the file to output.
##!       $name = name associated with number
##!       $globe::xtag_struct{$name}{prev} = previous html file
##!       $globe::xtag_struct{$name}{next} = next html file
##!       $globe::xtag_struct{$name}{html} = associated html file
##!       $globe::xtag_struct{$name}{severity} = severity associated with number
##!       $globe::xtag_struct{$name}{message} = message associated with number
##!       $globe::xtag_struct{$name}{comment} = comment associated with number
##!
##! @ingroup tp_xhelp
##############################################################################
sub write_output_html {
   local ($by_name) = 1;
   if (1) {
     print "Getting ready to write output html.\n";
   }
   if (0) {
      print "Testing the output...\n";
      $j=0;
      foreach $i (sort keys %{globe::xtag_struct}) {
         if (1) {
            print "==== name $i =====\n";
            foreach $item (keys %{$globe::xtag_struct{$i}}){
               print "   $i: $item: $globe::xtag_struct{$i}{$item}\n";
            }
         }
         $j++;
      }
      print "Done Testing the output...\n";
      # exit(1);
   }
   
   if (0) {
      print "$globe::m_info{header}\n";
      print "$globe::m_info{footer}\n";
      print "$globe::m_info{variable}\n";
      $def_type = "variable";
      print "$def_type $globe::m_define{$def_type}[0]\n";
      foreach $var (keys %{$globe::m_var{define}}) {
         print "$var $globe::m_var{define}{$var}\n";
      }
      exit(1);
   }
   
   
   #####
   # Assign previous and next values for the navigation
   #####
   if ($by_name) { # new way sorts by name
      $prev = "_TBD_";
      $i=0;
      foreach $name (sort {$a cmp $b} keys %{globe::xtag_struct}){
         $globe::xtag_struct{$name}{prev} = $prev;
         # Check to see if file exists
         $globe::xtag_struct{$name}{html} = "$name.html";
         $prev = "$name.html";
         if ($i == 0) {
            $first = $name;
            $first_prev = $prev;
         }
         $i++;
      }
      $next = "_TBD_";
      $i=0;
   }
   $next = "_TBD_";
   $i=0;
   if ($by_name) { # new way sorts by name
      foreach $name (sort {$b cmp $a} keys %{globe::xtag_struct}){
         $globe::xtag_struct{$name}{next} = $next;
         $next = "$name.html";
         if ($i == 0) {
            $last = $name;
            $last_next = $next;
         }
         $i++;
      }
   }
   $total=$i;
   #####
   # Fix the first and last previous and next assignments.
   #####
   $globe::xtag_struct{$last}{next} = $first_prev;
   $globe::xtag_struct{$first}{prev} = $last_next;

   $def_type = "variable";
   $i=0;
     
   # new way sorts by name, old way by number
   ENUM: foreach $name (sort {$a cmp $b} keys %{globe::xtag_struct}){
      if (0) {
         print "Getting ready for $name\n";
      }
      $file_out = "$globe::path";
      $file_out .= "$name.html";

      #####
      # Cheap-o error checking; not completely valid,
      # but will save at least one file from getting overwritten.
      #####
      if ((0) && (open ( OUT_FILE, "$file_out"))) {
         close (OUT_FILE);
         push (@file_errors, "\"$file_out\" already exists.\n");
         print "Cannot open file \"$file_out\"\n";
         next ENUM;
      }

      unless (open ( OUT_FILE, ">$file_out")) {
         push (@file_errors, "Cannot open file \"$file_out\"\n");
         print "Cannot open file \"$file_out\"\n";
         next ENUM;
      }
      if (1) {
         print "Creating $file_out...\n";
      }
      print (OUT_FILE "<html>\n <head>\n");
      # print (OUT_FILE "<title>$name</title>\n");
      print (OUT_FILE "$globe::m_info{header}\n");
      print (OUT_FILE "$globe::m_define{$def_type}[0] ");
      
      print (OUT_FILE "$globe::m_var{define}{prev}$globe::xtag_struct{$name}{prev}");
      print (OUT_FILE "$globe::m_var{define}{next}$globe::xtag_struct{$name}{next}");
      if (0) {
         print "$globe::m_var{define}{prev}$globe::xtag_struct{$name}{prev}";
         print "$globe::m_var{define}{next}$globe::xtag_struct{$name}{next}\n";

      }
      if ($i == 0) {
         print (OUT_FILE "$globe::m_var{define}{firstlast}1\#0");
      } elsif ($i == $total - 1) {
         print (OUT_FILE "$globe::m_var{define}{firstlast}0\#1");
      } else {
         print (OUT_FILE "$globe::m_var{define}{firstlast}0\#0");
      }
      
      print (OUT_FILE "$globe::m_define{$def_type}[1] \n");
      print (OUT_FILE "</head>\n<body>\n");
      #####
      # Fix up the navigation
      # GCM still to do... with previous and next...
      #####
      $nav_bars = $globe::m_info{nav_common};
      $nav_bars .= $globe::m_info{nav_book};
      $nav_bars =~ s/$globe::m_var{define}{prev}/$globe::xtag_struct{$name}{prev}/;
      $nav_bars =~ s/$globe::m_var{define}{next}/$globe::xtag_struct{$name}{next}/;
      $out_string = $nav_bars;
      $out_string .= "<h1>$name</h1>\n";
      $out_string .= "<p class=\"TitleChapterTOC\">Syntax:</p>\n"; 
      $out_string .= "<p class=\"Head1TOC\">$globe::xtag_struct{$name}{message}</p>\n";
      $out_string .= "<p class=\"TitleChapterTOC\">Severity:</p>\n"; 
      $out_string .= "<p class=\"Head1TOC\">$globe::xtag_struct{$name}{severity}</p>\n";
      $out_string .= "<p class=\"TitleChapterTOC\">Details:</p>\n"; 
      $out_string .= "<p class=\"Head1TOC\">$globe::xtag_struct{$name}{comment_full}</p>\n";
      $nav_bars = "$globe::m_info{nav_book}$globe::m_info{footer}";
      $nav_bars =~ s/$globe::m_var{define}{prev}/$globe::xtag_struct{$name}{prev}/;
      $nav_bars =~ s/$globe::m_var{define}{next}/$globe::xtag_struct{$name}{next}/;
      $out_string .= "$nav_bars\n";
      print (OUT_FILE "$out_string");
      print (OUT_FILE "</body>\n</html>\n");
      close (OUT_FILE);
      $i++;
   }


   for ($i=0; $i<@file_errors; $i++){
      print (OUT_FILE "$i = $file_errors[$i]\n");
   }
} # write_output_html


##############################################################################
##!       $name = name associated with number
##!       $globe::xtag_struct{$name}{prev} = previous html file
##!       $globe::xtag_struct{$name}{next} = next html file
##!       $globe::xtag_struct{$name}{html} = associated html file
##!       $globe::xtag_struct{$name}{severity} = severity associated with number
##!       $globe::xtag_struct{$name}{message} = message associated with number
##!       $globe::xtag_struct{$name}{comment} = comment associated with number
##############################################################################
sub generate_index {
      
   @path_piece = split (/\//, $globe::path);
   $strip_p = pop (@path_piece);
   $strip_p = pop (@path_piece);
   $strip_p .= "\/";

   foreach $name (sort keys %{globe::xtag_struct}){
      $out_string = $name;
      $out_string .= $globe::word_url_boundary;
      $out_string .= "<a href=\"";
      # $out_string .= $strip_p;
      $out_string .= $globe::xtag_struct{$name}{html};
      $out_string .= "\" target=\"basefrm\">";
      push (@globe::index_info, $out_string);
   }
   $file_out = "$globe::path";
   $file_out .= "_index_list";
   unless (open ( OUT_INDEX_LIST, ">$file_out")) {
      push (@file_errors, "Cannot open file \"$file_out\"\n");
      print "Cannot open file \"$file_out\"\n";
   }
   if (0) { # 
      for ($i=0; $i<@globe::index_info; $i++){
         print "$i = $globe::index_info[$i]\n";
         print (OUT_INDEX_LIST "$globe::index_info[$i]\n");
      }
   }
   close (OUT_INDEX_LIST);
   if (1) {
      print "Creating $file_out...\n";
   }

} # generate_index


##############################################################################
##!       $name = name associated with number
##!       $globe::xtag_struct{$name}{prev} = previous html file
##!       $globe::xtag_struct{$name}{next} = next html file
##!       $globe::xtag_struct{$name}{html} = associated html file
##!       $globe::xtag_struct{$name}{severity} = severity associated with number
##!       $globe::xtag_struct{$name}{message} = message associated with number
##!       $globe::xtag_struct{$name}{comment} = comment associated with number
##############################################################################
sub generate_script {
      
   $file_out = "$globe::path";
   $file_out .= "tree.script";
   unless (open ( OUT_SCRIPT, ">$file_out")) {
      push (@file_errors, "Cannot open file \"$file_out\"\n");
      print "Cannot open file \"$file_out\"\n";
      return (0);
   }



   @path_piece = split (/\//, $globe::path);
   $strip_p = pop (@path_piece);
   $strip_p = pop (@path_piece);
   $strip_p .= "\/";

   { # filler bracket
      #####
      # This top title is part of the master and does not need to be
      # part of this.
      #####
      print (OUT_SCRIPT "Item level=1 ");
      print (OUT_SCRIPT "image=nav_folderclosed.gif ");
      # print (OUT_SCRIPT "url=$s_path,basefrm ");
      # print (OUT_SCRIPT "selected=YES ");
      print (OUT_SCRIPT "text=$org_title Log Messages\r\n");
   } # filler bracket


   foreach $name (sort {$a cmp $b} keys %{globe::xtag_struct}){
      print (OUT_SCRIPT "Item level=2 ");
      print (OUT_SCRIPT "image=nav_doc.gif ");
      # print (OUT_SCRIPT "url=$strip_p$globe::xtag_struct{$name}{html},basefrm ");
      print (OUT_SCRIPT "url=$globe::xtag_struct{$name}{html},basefrm ");
      print (OUT_SCRIPT "selected=YES ");
      print (OUT_SCRIPT "text=$name\r\n");
   }

   close (OUT_SCRIPT);
   if (1) {
      print "Creating $file_out...\n";
   }
   
   return (1);

} # generate_script


##############################################################################
##! @fn int using_scheck
##! @brief What to do when no arguments are given.
##! @param None
##! @return None
##! log_html_gen.pl ../cms_middleware/doc_publish/book_cms_logging/ 
##!    ../../zz_stage/bridgegroup/middleware/supptsys/messages.txt 
##!    ../cms_middleware/voyant_master_nav.html
##!    "Middleware "
##! 
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub using_scheck {
   print "============  Syntax  ==================================\n";
   print "log_html_gen.pl <output directory> <input message file path \& name> ";
   print "<input master HTML file path \& names> [<title>]\n\n";
   print "- <output directory>  ";
   print "\n\n";
   print "- <input message file path \& name> is file path and name input message ";
   print "file that came from development.\n\n";
   print "- <input master HTML file path \& names> is file path and name of the master ";
   print "HTML file. \n";
   print "- [<title>] is an optional title that will be inserted into ";
   print "the TOC script for these messages; make sure it is followed by a ";
   print "space!!!\n";
   print "\n";
   print "============  Terminate  ==================================\n";
   print "Terminating log_html_gen.pl without doing anything.\n";
   return(1);
}




##############################################################################
##! @fn int END
##! @brief Code to execute when terminated nicely.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub END {
   undef ($_file_list); # "_file_list";
   undef ($in_file ); #  "";
   undef ($root_path ); #  "./";
   undef (%file_hash);

  
   ##############################################################################
   ## Memory clean-up.
   ## Perform this if it was defined.
   ##############################################################################
   if ($no_scope_file > 0){
      &globe::memory_clean_up();
   }
   
   print "\n============  Finished log_html_gen.pl ==================================\n";
} # END
