#!/usr/local/bin/perl
##############################################################################
##! @file
##! @brief Updates the date and version in the supplied master file.
##! 
##! @param $globe::master_nav_file [optional path and] filename for the 
##! HTML file to use as the master for information.
##! This file has several specially flagged
##! HTML comment sections that are required. Information from
##! the tagged sections is updated based on the input.
##!
##!
##! @ingroup tp_tools tp_nav
##!
##! @author Glenn C. Maxey
##!
##    $Id: master_update.pl,v 1.2 2002/11/21 17:57:14 gmaxe Exp $
##
## 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Permission to use, copy, modify, and distribute this software and its 
## documentation under the terms of the GNU General Public License is hereby 
## granted. No representations are made about the suitability of this software 
## for any purpose. It is provided "as is" without express or implied warranty. 
## See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
## for more details.
## 
## Documents produced by this script are derivative works derived from the 
## input used in their production; they are not affected by this license.
##
##    Revision Information:
##
##    $Log: master_update.pl,v $
##    Revision 1.2  2002/11/21 17:57:14  gmaxe
##    Minor updates to version updating.
##
##    Revision 1.1  2002/11/21 17:17:56  gmaxe
##    Tools to automatically update version and date in an HTML file.
##    Important when SCM runs our tool.
##
##
##############################################################################


##############################################################################
##! @fn int BEGIN
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub BEGIN {
   print "\n============  Starting master_update.pl ==================================\n";
   $_file_list = "_file_list";
   $_index_file = "_index_list";
   $scope_pm = "globe.pm";
   $_arg_inc = 0;
   $in_file = "";
   $f_type = "htm";
   $no_scope_file = 0;

   push (@INC, `pwd`);
   push (@INC, '../perl');
   push (@INC, '/rtfm/techpubs/perl');
   if (0){
      print (@INC, "\n");
   }
   #####
   # All global variables are defined in the following file
   #####
   unless (open ( IN_LIST, $scope_pm)) {
      unless (open ( IN_LIST, "../perl/$scope_pm")) {
         unless (open ( IN_LIST, "/rtfm/techpubs/perl/$scope_pm")) {
            push (@file_errors, "Cannot open file \"$scope_pm\" or \"../perl/$scope_pm\" or \"/rtfm/techpubs/perl/$scope_pm\"\n");
            $no_scope_file++;
         }
      }
   }
   close (IN_LIST);
   push (@INC, $scope_pm);
   push (@INC, "../perl/$scope_pm");
   push (@INC, "/rtfm/techpubs/perl/$scope_pm");

   if (!@file_errors) {
      #####
      # All global variables are defined in the following file
      #####
      require $scope_pm;
   
      if (&globe::declare_variables()) {
         print "Variables initialized from $scope_pm.\n";
      } else {
         push (@file_errors, "Could not initialize variables from $scope_pm.\n");
      }
   } # if not @file_errors

   if (@ARGV < 1) {
      push (@file_errors, "No HTML file specified as first argument.\n");
   }
   
   if (@ARGV > $_arg_inc) {
      $globe::master_nav_file = @ARGV[$_arg_inc];
      print "The master file is $globe::master_nav_file\n";
      unless (open ( IN_MASTER, $globe::master_nav_file)) {
         push (@file_errors, "Cannot open file \"$globe::master_nav_file\"\n");
      }
      # Get the master definitions
      while (<IN_MASTER>){    # entire master file into memory.
         $globe::master_nav .= $_;
      }
      close (IN_MASTER);
   } # if 2 or more arguments
   $_arg_inc++;

   if (@ARGV > $_arg_inc) {
      $def_type = "version";
      $globe::m_info{$def_type} = @ARGV[$_arg_inc];
      print "The version is $globe::m_info{$def_type}\n";
   } else {
      $def_type = "version";
      $globe::m_info{$def_type} = "_NONE_";
      print "The version is $globe::m_info{$def_type}\n";
   } # if 2 or more arguments
   $_arg_inc++;
}

##############################################################################
##! @fn int main
##! @brief The main program.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_nav
##############################################################################
# sub main {
{
   ##############################################################################
   ## Program start
   ##############################################################################

   if (@file_errors) {
      # Makes no sense to go on if input parameters are off.
      print "\n============  Summary of errors ==================================\n";
      for ($i=0; $i<@file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
      &using_update();
      exit(1);
   }
   
   $globe::entire_file = $globe::master_nav;

   $not_critical = 0;
   $def_type = "version";
   if (0) {
      print "$def_type $globe::m_define{$def_type}[0]\n";
   }
   if (1) {
      print "$def_type $globe::m_info{$def_type}\n";
   }
   if ( $globe::entire_file =~ /$globe::m_define{$def_type}[0]/ )  {
      ($before, $piece, $after) = &globe::get_tag_chunk( $globe::entire_file,
                $globe::m_define{$def_type}[0], 
                $globe::m_define{$def_type}[1], 
                $not_critical);
      if ($piece) {
         if ($globe::m_info{$def_type} =~ "_NONE_") {
            # This means that what is already in the file is more accurate
            # and should be left...
            $globe::m_info{$def_type} = $piece;
         } else {
            $globe::entire_file = sprintf ("%s%s%s%s%s",
               $before,
               $globe::m_define{$def_type}[0],
               $globe::m_info{$def_type},
               $globe::m_define{$def_type}[1],
               $after);
         }
      }
   }
   if (1) {
      print "true $def_type $globe::m_info{$def_type}\n";
   }
   if (0){
      print "=== header ==== $globe::entire_file\n";
      exit(1);
   }
   
   $def_type = "date";
   use POSIX 'strftime';
   $globe::m_info{$def_type} = strftime "%m/%d/%Y", localtime;
   if (1) {
      print "$def_type $globe::m_info{$def_type}\n";
   }
   
   if ( $globe::entire_file =~ /$globe::m_define{$def_type}[0]/ )  {
      ($before, $piece, $after) = &globe::get_tag_chunk( $globe::entire_file,
                $globe::m_define{$def_type}[0], 
                $globe::m_define{$def_type}[1], 
                $not_critical);
      if ($piece) {
         $globe::entire_file = sprintf ("%s%s%s%s%s",
            $before,
            $globe::m_define{$def_type}[0],
            $globe::m_info{$def_type},
            $globe::m_define{$def_type}[1],
            $after)
      }
   }
   if (0){
      print "=== header ==== $globe::entire_file\n";
      exit(1);
   }
   unless (open ( IN_MASTER, ">$globe::master_nav_file")) {
      push (@file_errors, "Cannot open file \"$globe::master_nav_file\"\n");
   }
   print (IN_MASTER $globe::entire_file);
   close (IN_MASTER);

   ##############################################################################
   ## End of Program
   ##############################################################################
   exit(0);
} # the main routine

##############################################################################
##! @fn int using_update  ( )
##! @brief What to do when no arguments are given.
##! @param None
##! @return None
##! 
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub using_update {
   print "\nmaster_update.pl operates on an HTML file. \n";
   print "It updates the date and version tags contained within the file.\n";
   print "-- The first argument is file to use for the master.\n";
   print "   It must have sections for voy_version and voy_date.\n";
   print "-- The second argument is the version number.\n";
   print "\nTerminating master_update.pl without doing anything.\n";
   return;
}

##############################################################################
##! @fn int END
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub END {
   undef ($_file_list);     # "_file_list";
   undef ($_index_file); # = "_index_list";
   undef ($in_file);     # "";
   undef ($f_type);     # "htm";

   
   ##############################################################################
   ## Memory clean-up.
   ##############################################################################
   if ($no_scope_file > 0){
      &globe::memory_clean_up();
   }

   if (@file_errors) {
      print "\n============  Summary of errors ==================================\n";
      for ($i=0; $i<@file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
   }

   print "\n============  Finished master_update.pl ==================================\n";
} # END

