#!/usr/local/bin/perl
package gen_nav;

##############################################################################
##! @file
##! @brief Takes a tree js file from Doxygen and outputs a script file
##! used by the TOC java applet.
##! 
##!
##! @ingroup tp_tools tp_nav
##!
##! @author Glenn C. Maxey
##!
##    $Id: tree_js_2_script.pl,v 1.7 2002/12/24 15:05:53 gmaxe Exp $
##
## 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Permission to use, copy, modify, and distribute this software and its 
## documentation under the terms of the GNU General Public License is hereby 
## granted. No representations are made about the suitability of this software 
## for any purpose. It is provided "as is" without express or implied warranty. 
## See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html) 
## for more details.
## 
## Documents produced by this script are derivative works derived from the 
## input used in their production; they are not affected by this license.
##
##    Revision Information:
##
##    $Log: tree_js_2_script.pl,v $
##    Revision 1.7  2002/12/24 15:05:53  gmaxe
##    New name and tweaks to add full paths.
##
##    Revision 1.6  2002/11/20 15:23:05  gmaxe
##    Added exit codes so that wrapper scripts can catch errors properly.
##
##    Revision 1.5  2002/11/20 01:49:14  gmaxe
##    Minor changes to exit codes.
##
##    Revision 1.4  2002/03/29 00:19:41  gmaxe
##    Fixed it so that it handles dependencies in the build.
##    Relative paths are handled.
##
##    Revision 1.3  2002/03/05 02:05:30  gmaxe
##    Fixed so that it won't hyperlink to things that don't exist in directory.
##
##    Revision 1.2  2002/03/05 01:47:41  gmaxe
##    Supports previous/next on Doxygen files;
##    supports better script generation.
##
##    Revision 1.1  2002/03/02 01:30:03  gmaxe
##    Updated to support tree.js files.
##
##
##############################################################################


##############################################################################
##! @fn int BEGIN
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub BEGIN {
   print "\n============  Starting tree_js_2_script.pl ==================================\n";
   $_file_list = "_file_list";
   $_toc_file = "tree.html";
   $_index_file = "_index_list";
   $scope_pm = "globe.pm";
   $in_file = "tree.js";
   $f_type = "tp";

   push (@INC, `pwd`);
   push (@INC, '/rtfm/techpubs/perl');
   if (0){
      print (@INC, "\n");
   }
   #####
   # All global variables are defined in the following file
   #####
   unless (open ( IN_LIST, $scope_pm)) {
      unless (open ( IN_LIST, "/rtfm/techpubs/perl/$scope_pm")) {
         push (@file_errors, "Cannot open file \"$scope_pm\" or \"/rtfm/techpubs/perl/$scope_pm\"\n");
         $no_scope_file++;
      }
   }
   close (IN_LIST);
   push (@INC, $scope_pm);
   push (@INC, "/rtfm/techpubs/perl/$scope_pm");

   if (!@file_errors) {
      #####
      # All global variables are defined in the following file
      #####
      require $scope_pm;
   
      if (&globe::declare_variables()) {
         print "Variables initialized from $scope_pm.\n";
      } else {
         push (@file_errors, "Could not initialize variables from $scope_pm.\n");
      }
   } # if not @file_errors

   if (@ARGV < 1) {
      push (@file_errors, "No path specified as first argument.\n");
   }
   
   # Get path if there is one.
   if (@ARGV > 0) {
      $globe::path = @ARGV[0];
      if ($globe::path =~ /\//) {
         print "The path specified is $globe::path\n";
      
         @path_chunks = split ( /\//, $globe::path);
         if (@path_chunks < 2) {
            $globe::rel_path_to_start_point = "./";
         } else {
            $globe::rel_path_to_start_point = "../";
         }
      } else {
         push (@file_errors, "The input argument \"$globe::path\" requires a forward slash (\/).\n");
      }
   } # if 1 or more arguments
   
   if (@ARGV <= 1) {
      push (@globe::file_errors, "ERROR: 2nd argument master HTML file is required.");
   } # if 2 or more arguments

   if (@ARGV > 1) {
      $in_file = @ARGV[1];
      print "The input file is $in_file.\n";
      unless (open ( IN_MASTER, "$in_file")) {
         push (@file_errors, "Cannot open file \"$in_file\"\n");
      }
   } # if 2 or more arguments


   
} # BEGIN

##############################################################################
##! @fn int main
##! @brief The main program.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_nav
##############################################################################
# sub main {
{
   ##############################################################################
   ## Program start
   ##############################################################################

   if (0){
      print "=== Definitions 3 \n";
      exit(1);
   }

   if (@file_errors) {
      # Makes no sense to go on if input parameters are off.
      print "\n============  Summary of errors ==================================\n";
      for ($i=0; $i<@file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
      &using_voy_nav();
      exit(1);
   }


   &get_input_file ("$in_file");
   if (0) {
      print "got this far...\n";
   }

   if (@ARGV > 2) {
      &output_toc_script (@ARGV[2]);
   } else {
      $temp = "$globe::path$_toc_file";
      push (@globe::file_errors, "WARNING: Using default $temp for output toc.");
      $temp = "$globe::path$_toc_file";
      &output_toc_script ($temp);
   } # if 3 or more arguments


   ##############################################################################
   ## End of Program
   ##############################################################################
   exit(0);
} # the main routine

##############################################################################
##! @fn int using_voy_nav  ( )
##! @brief What to do when no arguments are given.
##! @param None
##! @return None
##! 
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub using_voy_nav {
   print "\ntree_js_2_script.pl operates on text files that indicate heirarchical \n";
   print "structure of the system. \n";
   print "1st argument is the root path; required. \n";
   print "2nd argument name of tree js file; optional. \n";
   print "3nd argument of the output toc script file; optional. \n";

   return;
}


##############################################################################
##! @fn int get_input_file
##! @brief Grabs the information from the input file and puts into an array
##! of hash elements.
##!
##! The index into the array is a number that signifies it's placement.
##! Each index element can have three hashes:
##!
##! {level} An indication of the heirarchical placement.
##! {title} The plain text title.
##! {url} The associated URL for the title.
##!
##! @ingroup tp_nav
##############################################################################
sub get_input_file {
   $tp_file = $_[0];
   $entire_file = "";
   $folderstree = "foldersTree";
   $aux = "aux";
   $glnk = "gLnk";
   $gfld = "gFld";
   $insfld = "insFld";
      
   print "The tp input file is $tp_file\n";
   unless (open ( IN_FILE, $tp_file)) {
      print "Cannot open file \"$tp_file\"\n";
      push (@globe::file_errors,  "Cannot open file \"$tp_file\".");
      exit(1);
   }
   $_comment = 0;
   $_cnt = -1;
   # Get the definitions
   TP_LINE: while (<IN_FILE>){    # entire file into memory.
      $entire_file .= $_;
   }
   close (IN_FILE);
   
   @ft = split (/\n/, $entire_file);
   for ($i=1; $i <= $#ft; $i++) {
      # we are purposely ignoring the 0 element, because this is retrieved elsewhere
      # from voy_order in a later process.
      &get_gfld_title ($ft[$i], $i);
      if (0) {
         foreach $ddd (keys %{$tp[$index]} ) {
            print "$index = $ddd = $tp[$index]{$ddd}\n";
         }
      }
   }

   
} # get_input_file


##############################################################################
##! @fn int get_gfld_title
##! @brief 
##!
##!
##!
##! @ingroup tp_nav
##############################################################################
sub get_gfld_title {
   $in_chunk = $_[0];
   $index = $_[1];
   
   # split on closing parenthesis
   @t_chunk = split (/$glnk|$gfld/, $in_chunk, 2);
   if (0) {
      print "$folderstree $aux Glenn, you are here...";
   }
   
   # Level
   if ($t_chunk[0] =~ /$folderstree/i) {
      $tp[$index]{level} = 1;
      if (0) {
         print "what ya doin' here? $folderstree $t_chunk[0]\n";
      }
   } elsif ($t_chunk[0] =~ /$aux/i) {
      if ($t_chunk[0] =~ /$insfld/i) {
         @t1_chunk = split (/$insfld/i, $t_chunk[0], 2);
         @t2_chunk = split (/$aux/i, $t1_chunk[1], 2);
         @t3_chunk = split (/\,/, $t2_chunk[1], 2);
      } else {
         @t2_chunk = split (/$aux/i, $t_chunk[0], 2);
         @t3_chunk = split (/\,/, $t2_chunk[1], 2);
      }
      $tp[$index]{level} = $t3_chunk[0]+1;
      if (0){
         print "hey $t3_chunk[0], $t3_chunk[1], $t3_chunk[2]\n";
      }
   } else {
      if (0) {
         print "You shouldn't be here...\n$t_chunk[0]\n";
      }
   }
   
   # Title & URL
   @t2_chunk = split (/\,/, $t_chunk[1], 3);

   # Title
   @t3_chunk = split (/\"/, $t2_chunk[0], 3);
   $tp[$index]{title} = $t3_chunk[1];
   if ((1) && (exists ($globe::dox_name_change{$t3_chunk[1]} ) ) ){
      # Changes the name of items to be more meaningful to our environment
      $tp[$index]{title} = $globe::dox_name_change{$t3_chunk[1]};
   }
 
   # URL
   @t3_chunk = split (/\"/, $t2_chunk[2], 3);
   $tp[$index]{url} = $t3_chunk[1];

   if (0) {
      foreach $ddd (keys %{$tp[$index]} ) {
         print "$index = $ddd = $tp[$index]{$ddd}\n";
      }
      exit(1);
   }
   
} # get_gfld_title



##############################################################################
##! @fn int output_toc_script
##! @brief 
##!
##!
##!
##! @ingroup tp_nav
##############################################################################
sub output_toc_script {
   local ($htree_file) = $_[0];
   
   if (($htree_file =~ /(\.html)$/i) || ($htree_file =~ /(\.htm)$/i)){
      @_name = split (/\.htm/i, $htree_file, 2);
      $htree_file = $_name[0];
   }
   $htree_file .= ".script";
   
   print "The output TOC script file is \"$htree_file\".\n";

   unless (open(OUT_SCRIPT, ">$htree_file")) {
      push (@file_errors, "Cannot open file \"$htree_file\".");
      return(1);
   }

   ##################################################
   # Root level of tree.
   ##################################################

   ##################################################
   # This is the main loop; it starts at a first topic and then traces the next
   # topics from there. When a next topic is determined to be a last topic for
   # that section, it then goes and gets the next first topic.
   ##################################################
   for ($i=1; $i <= $#ft; $i++) {
      # we are purposely ignoring the 0 element, because this is retrieved elsewhere
      # from voy_order in a later process.
      if ((exists ($tp[$i]{url})) && ($tp[$i]{url} =~ /[\w+]/)) {
         if (!&html_exist($tp[$i]{url})) {
            # It doesn't seem to exist
            # check to see if it's a path to be fixed.
            if ($tp[$i]{url} =~ /$globe::fix_path_flag/){
               # we go under the assumption that it exists
               # and that it is a relative path issue.
               if (0) {
                  print "Fixing $tp[$i]{url}\n";
               }
               $tp[$i]{url} =~ s/$globe::fix_path_flag/$globe::fix_path_to/;
               if (0) {
                  print "Fixed $tp[$i]{url} to be relative.\n";
               }
               print (OUT_SCRIPT "Item level=$tp[$i]{level} ");
               print (OUT_SCRIPT "image=nav_doc.gif ");
               print (OUT_SCRIPT "url=$tp[$i]{url},basefrm ");
               print (OUT_SCRIPT "selected=YES ");
               print (OUT_SCRIPT "text=$tp[$i]{title}\r\n");
            } else { # $globe::fix_path_flag
               # it really doesn't exist
               # if it doesn't exist, don't hyperlink it.
               print (OUT_SCRIPT "Item level=$tp[$i]{level} ");
               print (OUT_SCRIPT "text=$tp[$i]{title}\r\n");
            } # $globe::fix_path_flag
         } else { # (!&html_exist($tp[$i]{url})
            # It does exist
            print (OUT_SCRIPT "Item level=$tp[$i]{level} ");
            print (OUT_SCRIPT "image=nav_doc.gif ");
            print (OUT_SCRIPT "url=$tp[$i]{url},basefrm ");
            print (OUT_SCRIPT "selected=YES ");
            print (OUT_SCRIPT "text=$tp[$i]{title}\r\n");
         }
      } else {
         # it really doesn't exist
         # if it doesn't exist, don't hyperlink it.
         print (OUT_SCRIPT "Item level=$tp[$i]{level} ");
         print (OUT_SCRIPT "text=$tp[$i]{title}\r\n");
      }

   } # for $i

   
   ##################################################
   # Clean up navigation.
   ##################################################
   close (OUT_SCRIPT);

} # output_toc_script


##############################################################################
##! @fn int html_exist
##! @brief Tests that the URL to a file exists.
##!
##! Discovered that doxygen's installdox does not update the tree.js files,
##! which leads to entries in the TOC that 404. Test it and return an 
##! appropriate response.
##!
##! @ingroup tp_nav
##############################################################################
sub html_exist {
   # Test the damn URL to make sure the file even exists.
   $test_file = $_[0];

   unless (open(TEST, "$globe::path$test_file")) {
      if (0){
         print "$globe::path$test_file doesn't exist.\n";
      }
      return(0);
   }
   close (TEST);
   return (1);


} # html_exist


##############################################################################
##! @fn int END
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_nav
##############################################################################
sub END {
   &globe::do_errors;
   undef ($_file_list);     # "_file_list";
   undef ($_index_file); # = "_index_list";
   undef ($in_file);     # "";
   undef ($f_type);     # "htm";

   
   ##############################################################################
   ## Memory clean-up.
   ##############################################################################
   if ($no_scope_file > 0){
      &globe::memory_clean_up();
   }

   if (@file_errors) {
      print "\n============  Summary of errors ==================================\n";
      for ($i=0; $i<@file_errors; $i++){
         print "$i = $file_errors[$i]\n";
      }
   }

   print "\n============  Finished tree_js_2_script.pl ==================================\n";
} # END

