#!/usr/local/bin/perl
package xscope;


##############################################################################
##! @file
##! @brief Defines global variables that can be used by other programs.
##! 
##! Most of the variables refer to tags that we expect to find in the 
##! HTML files. Others define variables that we want to use in a global way.
##!
##! @ingroup tp_tools tp_xhelp
##!
##! @author Glenn C. Maxey
##
##    $Id: xhelp_all.pm,v 1.3 2002/03/28 21:20:50 gmaxe Exp $
##
## Copyright 2002 Created by Voyant Technologies, Inc., Westminster, Colorado, USA.
##
## Revision Information:
##
## $Log: xhelp_all.pm,v $
## Revision 1.3  2002/03/28 21:20:50  gmaxe
## Added support for ingroup extraction.
##
## Revision 1.2  2002/02/21 01:38:59  gmaxe
## Working version of xhelp command extractor and prototype generator.
##
## Revision 1.1  2002/02/12 01:50:31  gmaxe
## Added support for more level chunking;
## added files to support xhelp;
## added file to support generating TOC, index from lists.
## PTR#:
##
##
##############################################################################


##############################################################################
##! @fn int BEGIN
##! @brief Code to execute when first entered.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub BEGIN {
   if (0) {
      print "xhelp_all.pm BEGIN routine.\n";
   }
} # BEGIN

##############################################################################
##! @fn int main
##! @brief Main routine.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
# sub main {
{
   ######
   ## Main Subroutine doesn't do anything
   ######
   if (0) {
      print "xhelp_all.pm main routine.\n";
   }
  
   # Needed in order for the package to work correctly from other perl programs.
   return(1);
}

##############################################################################
##! @fn int declare_variables
##! @brief Allocates memory for all globally used variables.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub declare_variables {

   ##############################################################################
   ## Declarations: all variables are global.
   ##############################################################################
   @include_f_type = ("h", "cpp", "c");
   
   # Exclude file types, files, and dir aren't used
   if (0) {
      @exclude_f_type = ("ben", "ini", "asn", "pl", "awk", 
                      "txt", "so", "tcl", "inc", "020", 
                      "s4", "sun", "vxworks-ppc860");
                      
      @exclude_file = ("Entries", "Repository", "Root", "Makefile", "CVS");
      @exclude_dir = ("CVS", "cvs");
   }
   
   @x_names = ("xCtrlp", "xdspi", "xmake", "xdump", "xshow", "xset", 
               "xget", "xload", "xclear", "xcache", "xupdate", "xmaint", 
               "xflush", "xenable", "xdisable", "xfree", "xscript", "xsnd", 
               "xstart", "xstop", "xresrcld", "xgo", "xnetm", "xrun", "xptr", 
               "xturn", "xvoip", "xgrow", "xprint", "xconnect", "xrcv", 
               "xshell", "xquery", "xclass", "xshutdown", "xpkt");
               
   @needed_in = ("xw", "wst", "wcs", "ctiCacheStats", "setMsg", "clearMsg", 
                 "resetMsgDefaults", "showMsgs", "setLocalMsg", "clearLocalMsg", 
                 "showLocalMsgs", "resetLocalMsgDefaults", "setConsoleOutputFlag", 
                 "log_dump", "startPktTrace", "sysxLogDump", "sysCardType", 
                 "showMasterClocks", "showBackupClocks", "removeAllProcesses", 
                 "xiveChangeExceptionMap", "xping", "xkillHeartbeat", "xuptime");
                 
   @ingroup = ("xhelp");
                 
   if (0) {
      # When testing, this reduces the list and the number of greps.
      # @x_names = ("bogus");
      undef (@x_names);
               
      @needed_in = ("xfreeBPTS", "xdspiChngConfState", "xdspiChngTkrMode", "xdspiCtrlPlay",
                    "xdspiPlayDate", "xfreePktTrack");
      @needed_in = ("xfreeBPTS", "xshowDspResrc", "xshellSndDiag", "xshowChip");
      @ingroup = ("xhelp");
   }
                 


   @needed_out = ("bogus");
               
   @return_types = ("VOID", "XSTAT", "RESRCHNDL", "XBOOL");
   
   if (0) {
      print "=== Definitions 2 \n";
   }

   return (1);

} # declare_variables

##############################################################################
##! @fn int memory_clean_up
##! @brief Cleans up after all variables that were defined.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_xhelp
##############################################################################
sub memory_clean_up {
   undef (@exclude_f_type);
                      
   undef (@exclude_file);
   undef (@exclude_dir);
   
   undef (@x_names);
               
   undef (@return_types);
   
   return (1);
}

##############################################################################
##! @fn int END
##! @brief Code to run at the end of this routine.
##!
##! @param None. 
##!
##! @return None.
##!
##! @bug None
##! @ingroup tp_tools
##############################################################################
sub END {
   if (0) {
      print "xhelp_all.pm END routine.\n";
   }
}

